package org.rcisoft.business.bagency.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bagency.entity.BAgency;
import org.rcisoft.business.bagency.service.BAgencyService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;

/**
* Created by  on 2018-4-4 8:57:37.
*/
@RestController
@RequestMapping("/bAgency")
public class BAgencyController extends PaginationController<BAgency> {

    @Autowired
    private BAgencyService bAgencyServiceImpl;


    @ApiOperation(value="添加教学单位", notes="添加教学单位")
    @ApiImplicitParams({@ApiImplicitParam(name = "code", value = "教学单位编号(1-50)", required = true, dataType = "varchar"),
                        @ApiImplicitParam(name = "name", value = "教学单位名(1-100)", required = true, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(@Valid @ApiIgnore BAgency bAgency, BindingResult bindingResult) {
        PersistModel data = bAgencyServiceImpl.save(bAgency);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        bAgency);
    }

    @ApiOperation(value="逻辑删除教学单位", notes="逻辑删除教学单位")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public Result delete(@PathVariable String businessId, BAgency bAgency) {
        bAgency.setBusinessId(businessId);
        PersistModel data = bAgencyServiceImpl.remove(bAgency);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

//    @ApiOperation(value="查询教学单位下是否有用户", notes="查询教学单位下是否有用户")
//    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
//    @GetMapping(value = "/selectBAgencyHaveUser")
//    public String HaveUserOrNot(@ApiIgnore BAgency bAgency) {
//        bAgency.setCreateBy(UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID));
//        String result = bAgencyServiceImpl.findBAgencyHaveUserOrNot(getPaginationUtility(), bAgency);
//        return result;
//    }

    @ApiOperation(value="修改教学单位", notes="修改教学单位")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
                        @ApiImplicitParam(name = "code", value = "教学单位编号(1-50)", required = true, dataType = "varchar"),
                        @ApiImplicitParam(name = "name", value = "教学单位名(1-100)", required = true, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public Result update(@Valid @ApiIgnore BAgency bAgency, BindingResult bindingResult) {
        PersistModel data = bAgencyServiceImpl.merge(bAgency);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        bAgency);
    }

    @ApiOperation(value="查看单个教学单位", notes="查看单个教学单位")
    @GetMapping("/detail/{businessId:\\w+}")
    public Result detail(@PathVariable String businessId) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        bAgencyServiceImpl.findById(businessId));
    }

    @ApiOperation(value="根据条件查询教学单位表(不传条件默认为查全部)", notes="根据条件查询教学单位表(不传条件默认为查全部)")
    @GetMapping(value = "/queryBAgencyByPagination")
    public GridModel listByPagination(@ApiIgnore BAgency bAgency) {
        bAgency.setCreateBy(UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID));
        bAgencyServiceImpl.findAllByPagination(getPaginationUtility(), bAgency);
        return getGridModelResponse();
    }

    /**
    @ApiOperation(value="查询学期开过课的企业", notes="查询学期开过课的企业")
    @ApiImplicitParams({@ApiImplicitParam(name = "termCode", value = "学期编码（长度最小为1,最大为50）", required = true, dataType = "varchar")})
    @GetMapping(value = "/queryMoreByPagination")
    public GridModel queryMoreByPagination(@ApiIgnore BAgency bAgency) {
        bAgency.setCreateBy(UserUtil.getUserInfoProp(getToken(), UserUtil.USER_ID));
        bAgencyServiceImpl.queryMoreByPagination(getPaginationUtility(), bAgency);
        return getGridModelResponse();
    }*/

}
