package org.rcisoft.business.bagency.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.bagency.dao.BAgencyRepository;
import org.rcisoft.business.bagency.entity.BAgency;
import org.rcisoft.business.bagency.service.BAgencyService;
import org.rcisoft.business.bcompany.dao.BCompanyRepository;
import org.rcisoft.business.bcompany.entity.BCompany;
import org.rcisoft.business.bteacher.dao.BTeacherRepository;
import org.rcisoft.business.bteacher.entity.BTeacher;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Created by  on 2018-4-4 8:57:37.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BAgencyServiceImpl implements BAgencyService {

    @Autowired
    private BAgencyRepository bAgencyRepository;

    @Autowired
    private BTeacherRepository  bTeacherRepository;

    @Autowired
    private BCompanyRepository bCompanyRepository;

    @Autowired
    private SysUserMapper sysUserMapper;


    /**
    * 保存 bAgency
    * @param bAgency
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BAgency bAgency){
        BAgency agency = new BAgency(bAgency.getCode());
        if(bAgencyRepository.selectOne(agency)!=null){
            throw new ServiceException(ResultServiceEnums.CODE_IS_EXISTS);
        };
        //增加操作
        UserUtil.setCurrentPersistOperation(bAgency);
        int line = bAgencyRepository.insertSelective(bAgency);
        log.info(UserUtil.getUserInfoProp(bAgency.getToken(), UserUtil.USER_USERNAME)+"新增了ID为"+
            bAgency.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

//    public String findBAgencyHaveUserOrNot(PageUtil paginationUtility, BAgency bAgency){
//        List<BAgency> teacherAgency = bAgencyRepository.queryTeacherAgency(bAgency.getBusinessId());
//        List<BAgency> companyAgency = bAgencyRepository.queryCompanyAgency(bAgency.getBusinessId());
//        if(teacherAgency.size()>0||companyAgency.size()>0){
//            return "教学单位下存在用户";
//        }
//        return "教学单位下不存在用户";
//    }


    /**
    * 逻辑删除
    * @param bAgency
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BAgency bAgency){
//        //利用教学单位code删除所对应的teacher和company的字段
//        BAgency bagency = bAgencyRepository.selectOneBagency(bAgency.getBusinessId());
//        String code = bagency.getCode();
//        BTeacher bTeacher = new BTeacher();
//        BCompany bCompany = new BCompany();
//        bTeacher.setAgencyCode(code);
//        bCompany.setAgencyCode(code);
//        //根据agency_code查询b_teacher
//        List<BTeacher> teacherList = bTeacherRepository.selectByAgencyCode(bTeacher);
//        //根据agency_code查询b_company
//        List<BCompany> companyList = bCompanyRepository.selectByAgencyCode(bCompany);
//        //删除b_teacher
//        bTeacherRepository.batchDeleteBycode(teacherList);
//        //删除b_company
//        bCompanyRepository.batchDeleteBycode(companyList);
//        //根据code查询s_user
//        List<SysUser> usersByTeacher = sysUserMapper.selectByTeacherCode(teacherList);
//        List<SysUser> usersByCompany = sysUserMapper.selectByCompanyCode(companyList);
//        //删除s_user
//        sysUserMapper.batchDeleteByTeacherCode(teacherList);
//        sysUserMapper.batchDeleteByCompanyCode(companyList);
//        //删除s_r_user_role
//        sysUserMapper.deleteTeacherUserRole(usersByTeacher);
//        sysUserMapper.deleteCompanyUserRole(usersByCompany);
//        UserUtil.setCurrentMergeOperation(bAgency);
//        bAgency.setDeleted();
//        int line = bAgencyRepository.logicalDelete(bAgency);
//        log.info(UserUtil.getUserInfoProp(bAgency.getToken(), UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
//        bAgency.getBusinessId()+"的信息");
//        return new PersistModel(line);
        //先查询该教学单位是否被使用
        List<BAgency> teacherAgency = bAgencyRepository.queryTeacherAgency(bAgency.getBusinessId());
        List<BAgency> companyAgency = bAgencyRepository.queryCompanyAgency(bAgency.getBusinessId());
        if(teacherAgency.size()>0||companyAgency.size()>0){
            throw new ServiceException(ResultServiceEnums.DATA_HAS_USED);
        }
        UserUtil.setCurrentMergeOperation(bAgency);
        bAgency.setDeleted();
        int line = bAgencyRepository.logicalDelete(bAgency);
        log.info(UserUtil.getUserInfoProp(bAgency.getToken(), UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
                bAgency.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 bAgency
    * @param bAgency
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BAgency bAgency){
        /*BAgency agency = new BAgency(bAgency.getName());
        if(bAgencyRepository.selectOne(agency)!=null){
            throw new ServiceException(ResultServiceEnums.CODE_IS_EXISTS);
        };*/
        UserUtil.setCurrentMergeOperation(bAgency);
        int line = bAgencyRepository.updateByPrimaryKeySelective(bAgency);
        log.info(UserUtil.getUserInfoProp(bAgency.getToken(), UserUtil.USER_USERNAME)+"修改了ID为"+
            bAgency.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bAgency
    * @param id
    * @return
    */
    @Override
    public BAgency findById(String id){
        return bAgencyRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bAgency
    * @param bAgency
    * @return
    */
    @Override
    public List<BAgency> findAllByPagination(PageUtil paginationUtility,
        BAgency bAgency){
        return bAgencyRepository.listBAgencys(bAgency);
    }

    @Override
    public List<BAgency> queryMoreByPagination(PageUtil paginationUtility, BAgency bAgency) {
        return bAgencyRepository.queryMoreByPagination(bAgency);
    }

}
