package org.rcisoft.business.barrange.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.barrange.entity.BArrange;
import org.rcisoft.business.barrange.service.BArrangeService;
import org.rcisoft.business.bslschedule.service.BSlScheduleService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

/**
* Created by  on 2018-4-19 10:28:59.
*/
@RestController
@RequestMapping("/barrange")
public class BArrangeController extends PaginationController<BArrange> {

    @Autowired
    private BArrangeService bArrangeServiceImpl;

    @Autowired
    private BSlScheduleService bSlScheduleServiceImpl;

    @ApiOperation(value="添加", notes="添加")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(String json) {
        return Result.builder(bArrangeServiceImpl.save(json),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                json);
    }

    @ApiOperation(value="按id查询排课", notes="按id查询排课")
    @ApiImplicitParam(name = "businessId", value = "排课Id", required = true, dataType = "varchar")
    @GetMapping(value = "/findArrangeById")
    public Result findArrangeById(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bArrangeServiceImpl.findArrange(businessId));
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "json", value = "json", required = true, dataType = "varchar"),})
    @PostMapping("/update")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result update(String json) {
        PersistModel data = bArrangeServiceImpl.merge(json);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                json);
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @ApiImplicitParam(name = "termCode", value = "学期编号", required = false, dataType = "varchar")
    @GetMapping(value = "/findAllByPagination")
    public GridModel findAllByTermCodeByPagination(BArrange bArrange) {
        bArrangeServiceImpl.findAllByPagination(getPaginationUtility(),bArrange);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="企业查看个人排课信息", notes="企业查看个人排课信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "agencyCode", value = "企业编号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "status", value = "状态", required = false, dataType = "varchar")})
    @GetMapping(value = "/findAllByAgencyCodeByPagination")
    public GridModel findAllByAgencyCodeByPagination(@ApiIgnore BArrange bArrange) {
        bArrangeServiceImpl.findAllByAgencyCodeByPagination(getPaginationUtility(),bArrange);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="excel导入", notes="excel导入排课信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "importFile", value = "excel文件", required = true, dataType = "MultipartFile"),
            @ApiImplicitParam(name = "subAgencyId", value = "b_sub_agency.business_id", required = true, dataType = "varchar")})
    @PostMapping(value = "excelImport")
    public Result excelImport(MultipartFile importFile,String subAgencyId) throws Exception {
        //将数据插入列表视图(b_schedule_dto)
        PersistModel data = bArrangeServiceImpl.importExcel(MultipartFile2HSSFWorkbookConverter.convert(importFile),getToken(),subAgencyId);
        //将数据插入周视图(b_sl_schedule)
        bArrangeServiceImpl.importExcel2(MultipartFile2HSSFWorkbookConverter.convert(importFile),getToken(),subAgencyId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data.getInfluenceReason());
    }

    @ApiOperation(value="按照subAgencyId查列表视图", notes="按照subAgencyId查列表视图")
    @ApiImplicitParams({@ApiImplicitParam(name = "subAgencyId", value = "subAgencyId", required = true, dataType = "varchar")})
    @GetMapping(value = "/findDtoBybSubtaskId")
    public Result findDtoBybSubtaskId(String subAgencyId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bArrangeServiceImpl.queryDtoBySubAgencyId(subAgencyId));
    }


//    @ApiOperation(value="查看  集合", notes="查看单  集合")
//    @GetMapping(value = "/findDtoByMany")
//    public Result findDtoByMany(String agencyCode,String name,String termCode,String teacherCode) {
//        return Result.builder(new PersistModel(1),
//                MessageConstant.MESSAGE_ALERT_SUCCESS,
//                MessageConstant.MESSAGE_ALERT_ERROR,
//                bArrangeServiceImpl.queryDtoByMany(agencyCode, name, termCode,teacherCode));
//    }

    @ApiOperation(value="排课审核通过后的开课接口", notes="排课审核通过后的开课接口")
    @GetMapping(value = "/changeArrangeToSl")
    public Result changeArrangeToSl(String businessId,String subTaskId,String termCode) {
        int data = bArrangeServiceImpl.changeArrangeToSl(businessId,subTaskId,termCode);
        return Result.builder(new PersistModel(data),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value="按照subAgencyId查具体信息与审核内容", notes="按照subAgencyId查具体信息与审核内容")
    @ApiImplicitParams({@ApiImplicitParam(name = "subAgencyId", value = "subAgencyId", required = true, dataType = "varchar")})
    @GetMapping(value = "/selectSubAgencyAndOption")
    public Result selectSubAgencyAndOption(String subAgencyId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bArrangeServiceImpl.selectSubAgencyAndOption(subAgencyId));
    }

    @ApiOperation(value="子任务提交", notes="子任务提交")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),})
    @PutMapping("/submit")
    public Result updateSubAgency(String businessId) {
        PersistModel data = bArrangeServiceImpl.submit(businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value="发布", notes="发布")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),})
    @PostMapping("/release")
    public Result release(String businessId) {
        PersistModel data = bArrangeServiceImpl.release(businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value="逻辑删除排课", notes="逻辑删除排课")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete")
    public Result delete(String businessId) {
        PersistModel data = bArrangeServiceImpl.remove(businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }


}
