package org.rcisoft.business.barrange.service.impl;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.barrange.dao.BArrangeRepository;
import org.rcisoft.business.barrange.dao.BSubAgencyRepository;
import org.rcisoft.business.barrange.dao.BSubtaskRepository;
import org.rcisoft.business.barrange.entity.BArrange;
import org.rcisoft.business.barrange.entity.BSubAgency;
import org.rcisoft.business.barrange.entity.BSubtask;
import org.rcisoft.business.barrange.entity.ScheduleDto;
import org.rcisoft.business.barrange.service.BArrangeService;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bclass.dao.BClassRepository;
import org.rcisoft.business.bdirection.dao.BDirectionRepository;
import org.rcisoft.business.beduclass.dao.BEduClassRepository;
import org.rcisoft.business.beduclass.entity.BEduClass;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.entity.BLessonDirection;
import org.rcisoft.business.brooms.dao.BRoomsRepository;
import org.rcisoft.business.brooms.entity.BRooms;
import org.rcisoft.business.brslstudent.dao.BRSlStudentRepository;
import org.rcisoft.business.brslstudent.entity.BRSlStudent;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.business.bslschedule.dao.BSlScheduleRepository;
import org.rcisoft.business.bslschedule.entity.BSlSchedule;
import org.rcisoft.business.bslschedule.util.FindTime;
import org.rcisoft.business.bstudent.dao.BStudentRepository;
import org.rcisoft.business.bstudent.entity.BStudent;
import org.rcisoft.business.bstudent.service.BStudentService;
import org.rcisoft.business.bteacher.dao.BTeacherRepository;
import org.rcisoft.business.bteacher.entity.BTeacher;
import org.rcisoft.business.bterm.dao.BTermRepository;
import org.rcisoft.business.bterm.entity.BTerm;
import org.rcisoft.business.bterm.utils.Week;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * Created by  on 2018-4-19 10:28:59.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BArrangeServiceImpl implements BArrangeService {

    @Autowired
    private BArrangeRepository bArrangeRepository;

    @Autowired
    private BTermRepository bTermRepository;

    @Autowired
    private BSlRepository bSlRepository;

    @Autowired
    private BChapterService chapterService;

    @Autowired
    private BLessonRepository bLessonRepository;

    @Autowired
    private BEduClassRepository bEduClassRepository;

    @Autowired
    private BDirectionRepository bDirectionRepository;

    @Autowired
    private BSlScheduleRepository bSlScheduleRepository;

    @Autowired
    private BRoomsRepository bRoomsRepository;

    @Autowired
    private BSubAgencyRepository bSubAgencyRepository;

    @Autowired
    private BSubtaskRepository bSubtaskRepository;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private BStudentRepository bStudentRepository;

    @Autowired
    private BTeacherRepository bTeacherRepository;

    @Autowired
    private BRSlStudentRepository brSlStudentRepository;

    @Autowired
    private Global global;

    /**
    * 保存 bArrange
    * @param json
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(String json){
        BArrange bArrange = new BArrange();
        List<BSubtask> subtaskList = new ArrayList<>();
        JSONObject jsonObject = JSONObject.fromObject(json);
        JSONObject arrange = jsonObject.getJSONObject("arrange");
//        if (arrange.isEmpty()){
//            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
//        }
        if(arrange.getString("desc").length()>255){
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
        }
        bArrange.setDescription(arrange.getString("desc"));
        if (arrange.getString("termCode").equals("")){
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
        }
        bArrange.setTermCode(arrange.getString("termCode"));
        bArrange.setStatus("0");
        UserUtil.setCurrentPersistOperation(bArrange);
        int line = bArrangeRepository.insertSelective(bArrange);
        JSONArray childList = arrange.getJSONArray("childList");
//        if (childList.isEmpty()){
//            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
//        }
        for(int i = 0; i < childList.size() ; i++){
            JSONObject subtask = childList.getJSONObject(i);
            BSubtask bSubtask = new BSubtask();
            List<BSubAgency> bSubAgencyList = new ArrayList<>();
            if (subtask.getString("name").equals("") ||subtask.getString("name").length()>256){
                throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
            }
            bSubtask.setName(subtask.getString("name"));
            bSubtask.setArrangeId(bArrange.getBusinessId());
            bSubtask.setStatus("0");
            bSubtask.setSubtaskSeq(String.valueOf(i));
            UserUtil.setCurrentPersistOperation(bSubtask);
            subtaskList.add(bSubtask);
            JSONArray childList2 = subtask.getJSONArray("childList");

            for(int j = 0; j < childList2.size() ; j++){
                JSONObject agency = childList2.getJSONObject(j);
                BSubAgency bSubAgency = new BSubAgency();
                bSubAgency.setSubAgencySeq(String.valueOf(j));
                if (agency.get("agencyCode") != null && !agency.get("agencyCode").equals("")){
                    bSubAgency.setAgencyCode(agency.getString("agencyCode"));
                    bSubAgency.setStatus("0");
                    bSubAgency.setSubtaskId(bSubtask.getBusinessId());
                    UserUtil.setCurrentPersistOperation(bSubAgency);
                    bSubAgencyRepository.insertSelective(bSubAgency);
//                    bSubAgencyList.add(bSubAgency);
                }
                else {
                    throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
                }
            }
//            bArrangeRepository.insertBSubAgency(bSubAgencyList);
        }
        bArrangeRepository.insertBSubtask(subtaskList);
        return new PersistModel(line);
    }

    /**
     * 根据id查询 bArrange
     * @param businessId
     * @return
     */
    @Override
    public BArrange findArrange(String businessId){
        return bArrangeRepository.selectArrangeById(businessId);
    }

    /**
     * 修改排课
     * @param json
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(String json){
        BArrange bArrange = new BArrange();
        //json解析
        JSONObject jsonObject = JSONObject.fromObject(json);
        //arrange对象修改操作
        JSONObject arrange = jsonObject.getJSONObject("arrange");
        bArrange.setDescription(arrange.getString("desc"));
        bArrange.setTermCode(arrange.getString("termCode"));
        bArrange.setBusinessId(arrange.getString("businessId"));
        bArrangeRepository.updateByPrimaryKeySelective(bArrange);
        //清除原来的subTask 物理删除
        bSubtaskRepository.deleteSubtask(arrange.getString("businessId"));
        //重新添加subTask
        JSONArray childList = arrange.getJSONArray("childList");
        for(int i = 0; i < childList.size() ; i++){
            JSONObject subtask = childList.getJSONObject(i);
            BSubtask bSubtask = new BSubtask();
            bSubtask.setName(subtask.getString("name"));
            bSubtask.setArrangeId(arrange.getString("businessId"));
            bSubtask.setSubtaskSeq(String.valueOf(i));
            UserUtil.setCurrentPersistOperation(bSubtask);
            bSubtaskRepository.insertSelective(bSubtask);
            JSONArray childList2 = subtask.getJSONArray("childList");
            for(int j = 0; j < childList2.size() ; j++){
                JSONObject agency = childList2.getJSONObject(j);
                BSubAgency bSubAgency = new BSubAgency();
                   if (agency.get("agencyCode") != null){
                       bSubAgency.setAgencyCode(agency.getString("agencyCode"));
                   }
                //再插入新记录
                bSubAgency.setSubAgencySeq(String.valueOf(j));
                UserUtil.setCurrentPersistOperation(bSubAgency);
                bSubAgency.setSubtaskId(bSubtask.getBusinessId());
                bSubAgencyRepository.insertSelective(bSubAgency);
                }
            }
        return new PersistModel(1);
    }


    /**
    * 分页查询 bArrange
    * @return
    */
    @Override
    public List<BArrange> findAllByPagination(PageUtil paginationUtility, BArrange bArrange){
        bArrange.setNotDeleted();
        bArrange.setStart();
        List<BArrange> list =   bArrangeRepository.queryAll(bArrange);
        return list;
    }

    @Override
    public List<BArrange> findAllByAgencyCodeByPagination(PageUtil paginationUtility,BArrange bArrange) {
        //前台传status=1 是查未完成 对应数据库status=2
        if (bArrange.getStatus().equals("1")) {
            bArrange.setStatus("2");
        }
        else {
            bArrange.setStatus("1,2,3,4");
        }
        bArrange.setStart();
        bArrange.setNotDeleted();
        return bArrangeRepository.queryByAgencyCode(bArrange);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel importExcel(HSSFWorkbook hwb, String token , String subAgencyId) {
        ArrayList<ScheduleDto> scheduleDtos = new ArrayList<ScheduleDto>();
        Boolean contanSunday = ExcelUtil.containSunday(hwb);
        String[] headers = {"方案计划名","方案人数","方案课组","课程号","课程名","开课院系","课程属性","总学分","总学时",
                "授课学时","上机学时","实验学时","考试类型","任课教师", "教师编号","教学班组名","教学班组编号","周次","周学时","周一",
                "周二","周三","周四","周五","周六","上课地点","教室编号","备注"};
        if(contanSunday){
            headers = new String[]{"方案计划名", "方案人数", "方案课组", "课程号", "课程名", "开课院系", "课程属性", "总学分", "总学时",
                    "授课学时", "上机学时", "实验学时", "考试类型", "任课教师", "教师编号", "教学班组名", "教学班组编号", "周次", "周学时", "周一",
                    "周二", "周三", "周四", "周五", "周六","周日", "上课地点", "教室编号", "备注"};
        }
        //ArrayList<String[]> values = ExcelUtil.importExcel(hwb, headers,true,2);  //获取excel数据
        ArrayList<String[]> values = ExcelUtil.importExcelv2(hwb, headers,2);  //获取excel数据
        if(values.get(0)[3]==null || values.get(0)[14]==null ||values.get(0)[16]==null  ||values.get(0)[26]==null ||
                values.get(0)[3].equals("") || values.get(0)[14].equals("") ||values.get(0)[16].equals("") ||values.get(0)[26].equals("")){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_HAS_NULL);
        }
        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        List<String> lessonCodes = new ArrayList<>();
        List<String> teacherCodes = new ArrayList<>();
        List<String> classCodes = new ArrayList<>();
        List<String> roomCodes = new ArrayList<>();
        int index = 0;
        ScheduleDto lessonCellHead = null;
        ScheduleDto teacherCellHead = null;
        ScheduleDto classCellHead = null;
        ScheduleDto roomCellHead = null;
        for (String[] value : values) {
            ScheduleDto scheduleDto = new ScheduleDto();
            //判断1:根据课程编号查询是否有此门课程；
            if(bLessonRepository.select(new BLesson(value[3].trim())).size() != 1){
                lessonCodes.add(value[3]);
                continue;
            }
            //判断2:根据教师编号查询是否有此教师；
            SysUser user1 =new SysUser();
            user1.setLoginName(value[14]);
            user1.setDelFlag("0");
            user1.setFlag("1");
            if (sysUserMapper.select(user1).size() != 1) {
                teacherCodes.add(value[14]);
                continue;
            }
            //判断3:根据班级编号查询是否有此班级；
            if (bEduClassRepository.select(new BEduClass(value[16].trim())).size() != 1) {
                classCodes.add(value[16]);
                continue;
            }
            //判断4:根据教室编号查询是否有此教室；
            //如果存在星期日，向后推一格
            if(contanSunday){
                if(bRoomsRepository.select(new BRooms(value[27].trim())).size() != 1){
                    roomCodes.add(value[27]);
                    continue;
                }
            }else {
                if(bRoomsRepository.select(new BRooms(value[26].trim())).size() != 1){
                    roomCodes.add(value[26]);
                    continue;
                }
            }
            //合法:通过了四次判断，证明可建立该排课
            scheduleDto.setLessonCode(value[3]);
            scheduleDto.setTeacherCode(value[14]);
            scheduleDto.setClassCode(value[16]);
            if(contanSunday){
                scheduleDto.setRoomCode(value[27]);
            }else{
                scheduleDto.setRoomCode(value[26]);
            }
            scheduleDto.setWeek(value[17]);
            scheduleDto.setClassHour(value[18]);
            scheduleDto.setMonday(value[19]);
            scheduleDto.setTuesday(value[20]);
            scheduleDto.setWednesday(value[21]);
            scheduleDto.setThursday(value[22]);
            scheduleDto.setFriday(value[23]);
            scheduleDto.setSaturday(value[24]);
            if(contanSunday)
                scheduleDto.setSunday(value[25]);
            scheduleDto.setSubAgencyId(subAgencyId);
            //设置序号
            scheduleDto.setDtoSeq(String.valueOf(values.indexOf(value)));
            scheduleDto.setCommonBusinessId();
            UserUtil.setCurrentPersistOperation(scheduleDto);
            scheduleDtos.add(scheduleDto);

            //提示合并单元格
            if(contanSunday){
                if(value[28].equals("false")){
                    lessonCellHead = scheduleDtos.get(values.indexOf(value));
                    lessonCellHead.setLessonCell(1);
                }else{
                    lessonCellHead.setLessonCell(lessonCellHead.getClassCell() + 1);
                }
                if(value[29].equals("false")){
                    teacherCellHead = scheduleDtos.get(values.indexOf(value));
                    teacherCellHead.setTeacherCell(1);
                }else{
                    teacherCellHead.setTeacherCell(teacherCellHead.getTeacherCell() + 1);
                }
                if(value[30].equals("false")){
                    classCellHead = scheduleDtos.get(values.indexOf(value));
                    classCellHead.setClassCell(1);
                }else{
                    classCellHead.setClassCell(classCellHead.getClassCell() + 1);
                }
                if(value[31].equals("false")){
                    roomCellHead = scheduleDtos.get(values.indexOf(value));
                    roomCellHead.setRoomCell(1);
                }else{
                    roomCellHead.setRoomCell(roomCellHead.getRoomCell() + 1);
                }
            }else{
                if(value[27].equals("false")){
                    lessonCellHead = scheduleDtos.get(values.indexOf(value));
                    lessonCellHead.setLessonCell(1);
                }else{
                    lessonCellHead.setLessonCell(lessonCellHead.getClassCell() + 1);
                }
                if(value[28].equals("false")){
                    teacherCellHead = scheduleDtos.get(values.indexOf(value));
                    teacherCellHead.setTeacherCell(1);
                }else{
                    teacherCellHead.setTeacherCell(teacherCellHead.getTeacherCell() + 1);
                }
                if(value[29].equals("false")){
                    classCellHead = scheduleDtos.get(values.indexOf(value));
                    classCellHead.setClassCell(1);
                }else{
                    classCellHead.setClassCell(classCellHead.getClassCell() + 1);
                }
                if(value[30].equals("false")){
                    roomCellHead = scheduleDtos.get(values.indexOf(value));
                    roomCellHead.setRoomCell(1);
                }else{
                    roomCellHead.setRoomCell(roomCellHead.getRoomCell() + 1);
                }
            }

            //修改sub_agency表状态为保存
            BSubAgency bSubAgency = new BSubAgency(subAgencyId,"1");
            bSubAgencyRepository.updateByPrimaryKeySelective(bSubAgency);

            //修改sl_schedule表状态为保存
            bSlScheduleRepository.updateStatusBySubAgencyId(subAgencyId,"1");

        }
        String result = "";
        if (scheduleDtos.size()>0){
            int deleteNum = bArrangeRepository.avoidRepeatDto(subAgencyId);
            int line1 = bArrangeRepository.insertList(scheduleDtos);
            if (line1 < 1 ) {
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);
            }else{
                if(deleteNum > 0){
                    result+="覆盖原先数据，成功导入"+scheduleDtos.size()+"条数据。";
                }else{
                    result+="成功导入"+scheduleDtos.size()+"条数据。";
                }

            }
        }

        int resultFlag = 1;

        if(lessonCodes.size()>0){
            result+="以下课程编号不存在："+ JSON.toJSONString(lessonCodes)+"。";
            resultFlag = 0;
        }
        if(teacherCodes.size()>0){
            result+="以下教师编号不存在："+ JSON.toJSONString(teacherCodes)+"。";
            resultFlag = 0;
        }
        if(classCodes.size()>0){
            result+="以下班级编号不存在："+ JSON.toJSONString(classCodes)+"。";
            resultFlag = 0;
        }
        if(roomCodes.size()>0){
            result+="以下教室编号不存在："+ JSON.toJSONString(roomCodes)+"。";
            resultFlag = 0;
        }
        return new PersistModel(resultFlag, result);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel importExcel2(HSSFWorkbook hwb, String token, String subAgencyId) {
        String termCode = bArrangeRepository.selectTermCodeByBSubtaskId(subAgencyId);
        BTerm bTerm = bTermRepository.queryBtermByCode(termCode).get(0);
        //获取时间集合
        Map<String,Date> timeMap = FindTime.findTime(bTerm.getStartDate(),bTerm.getEndDate());
        Boolean contanSunday = ExcelUtil.containSunday(hwb);
        String[] headers = {"方案计划名","方案人数","方案课组","课程号","课程名","开课院系","课程属性","总学分","总学时",
                "授课学时","上机学时","实验学时","考试类型","任课教师", "教师编号","教学班组名","教学班组编号","周次","周学时","周一",
                "周二","周三","周四","周五","周六","上课地点","教室编号","备注"};
        if(contanSunday){
            headers = new String[]{"方案计划名", "方案人数", "方案课组", "课程号", "课程名", "开课院系", "课程属性", "总学分", "总学时",
                    "授课学时", "上机学时", "实验学时", "考试类型", "任课教师", "教师编号", "教学班组名", "教学班组编号", "周次", "周学时", "周一",
                    "周二", "周三", "周四", "周五", "周六","周日", "上课地点", "教室编号", "备注"};
        }
        ArrayList<String[]> values = ExcelUtil.importExcelv2(hwb, headers,2);  //获取excel数据
        if(values.get(0)[3]==null || values.get(0)[14]==null ||values.get(0)[16]==null  ||values.get(0)[26]==null ||
            values.get(0)[3].equals("") || values.get(0)[14].equals("") ||values.get(0)[16].equals("") ||values.get(0)[26].equals("")){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_HAS_NULL);
        }
        List<BSlSchedule> list = new ArrayList<>();
        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }
        List<String> lessonCodes = new ArrayList<>();
        List<String> teacherCodes = new ArrayList<>();
        List<String> classCodes = new ArrayList<>();
        List<String> roomCodes = new ArrayList<>();
        List<String> weekErrors = new ArrayList<>();

        for (String[] value : values) {

            //判断1:根据课程编号查询是否有此门课程；
            if(bLessonRepository.select(new BLesson(value[3].trim())).size() != 1){
                lessonCodes.add(value[3]);
                continue;
            }
            //判断2:根据教师编号查询是否有此教师；
            SysUser user1 =new SysUser();
            user1.setLoginName(value[14]);
            user1.setDelFlag("0");
            user1.setFlag("1");
            if (sysUserMapper.select(user1).size() != 1) {
                teacherCodes.add(value[14]);
                continue;
            }
            //判断3:根据班级编号查询是否有此班级；
            if (bEduClassRepository.select(new BEduClass(value[16].trim())).size() != 1) {
                classCodes.add(value[16]);
                continue;
            }
            //判断4:根据教室编号查询是否有此教室；
            //如果存在星期日，向后推一格
            if(contanSunday){
                if(bRoomsRepository.select(new BRooms(value[27].trim())).size() != 1){
                    roomCodes.add(value[27]);
                    continue;
                }
            }else {
                if(bRoomsRepository.select(new BRooms(value[26].trim())).size() != 1){
                    roomCodes.add(value[26]);
                    continue;
                }
            }
            String week = value[17];
            if(week.contains("-")){
                int location = week.indexOf("-");
                String start = week.substring(0,location);
                int numStart = Integer.valueOf(start);
                String end = week.substring(location + 1,week.length());
                int numEnd = Integer.valueOf(end);

                //按周分割
                for(int i = numStart ; i <= numEnd ; i++){
                    //如果周一的课不唯一.......以此类推
                    if(value[19].contains("-")){
                        //找到这节课所对应的时间
                        Date date = timeMap.get(i + ",1");
                        list.addAll(setDetail(19,value,subAgencyId,date,termCode,contanSunday));
                    }
                    if(value[20].contains("-")){
                        Date date = timeMap.get(i + ",2");
                        list.addAll(setDetail(20,value,subAgencyId,date,termCode,contanSunday));
                    }
                    if(value[21].contains("-")){
                        Date date = timeMap.get(i + ",3");
                        list.addAll(setDetail(21,value,subAgencyId,date,termCode,contanSunday));
                    }
                    if(value[22].contains("-")){
                        Date date = timeMap.get(i + ",4");
                        list.addAll(setDetail(22,value,subAgencyId,date,termCode,contanSunday));
                    }
                    if(value[23].contains("-")){
                        Date date = timeMap.get(i + ",5");
                        list.addAll(setDetail(23,value,subAgencyId,date,termCode,contanSunday));
                    }
                    if(value[24].contains("-")){
                        Date date = timeMap.get(i + ",6");
                        list.addAll(setDetail(24,value,subAgencyId,date,termCode,contanSunday));
                    }
                    if(contanSunday && value[25].contains("-")){
                        Date date = timeMap.get(i + ",7");
                        list.addAll(setDetail(25,value,subAgencyId,date,termCode,contanSunday));
                    }
                }
            }else {
                if(value[19].contains("-")){
                    //找到这节课所对应的时间
                    Date date = timeMap.get(week + ",1");
                    list.addAll(setDetail(19,value,subAgencyId,date,termCode,contanSunday));
                }
                if(value[20].contains("-")){
                    Date date = timeMap.get(week + ",2");
                    list.addAll(setDetail(20,value,subAgencyId,date,termCode,contanSunday));
                }
                if(value[21].contains("-")){
                    Date date = timeMap.get(week + ",3");
                    list.addAll(setDetail(21,value,subAgencyId,date,termCode,contanSunday));
                }
                if(value[22].contains("-")){
                    Date date = timeMap.get(week + ",4");
                    list.addAll(setDetail(22,value,subAgencyId,date,termCode,contanSunday));
                }
                if(value[23].contains("-")){
                    Date date = timeMap.get(week + ",5");
                    list.addAll(setDetail(23,value,subAgencyId,date,termCode,contanSunday));
                }
                if(value[24].contains("-")){
                    Date date = timeMap.get(week + ",6");
                    list.addAll(setDetail(24,value,subAgencyId,date,termCode,contanSunday));
                }
                if(contanSunday && value[25].contains("-")){
                    Date date = timeMap.get(week + ",7");
                    list.addAll(setDetail(25,value,subAgencyId,date,termCode,contanSunday));
                }
            }
            //合法:通过了四次判断，证明可建立该排课


        }


        String result = "";
        if (list.size()>0){
            int deleteNum = bSlScheduleRepository.avoidRepeatBSlSchedule(subAgencyId);
            int line1 = bSlScheduleRepository.insertList(list);
            if (line1 < 1 ) {
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);
            }else{
                if(deleteNum > 0){
                    result+="覆盖原先数据，成功导入"+list.size()+"条数据。";
                }else{
                    result+="成功导入"+list.size()+"条数据。";
                }
            }
        }

        if(lessonCodes.size()>0){
            result+="以下课程编号不存在："+ JSON.toJSONString(lessonCodes)+"。";
        }
        if(teacherCodes.size()>0){
            result+="以下教师编号不存在："+ JSON.toJSONString(teacherCodes)+"。";
        }
        if(classCodes.size()>0){
            result+="以下班级编号不存在："+ JSON.toJSONString(classCodes)+"。";
        }
        if(roomCodes.size()>0){
            result+="以下教室编号不存在："+ JSON.toJSONString(roomCodes)+"。";
        }
//        if(weekErrors.size()>0){
//            result+="以下周超过最大周数："+ JSON.toJSONString(weekErrors)+"。";
//        }

        return new PersistModel(1, result);
    }

    @Override
    public List<ScheduleDto> queryDtoBySubAgencyId(String SubAgencyId) {
        return bArrangeRepository.queryDtoBySubAgencyId(SubAgencyId);
    }

    @Override
    public List<ScheduleDto> queryDtoByMany(String agencyCode,String name,String termCode,String teacherCode) {
        return bArrangeRepository.queryDtoByAgencyMany(agencyCode,name,termCode,teacherCode);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public int changeArrangeToSl(String businessId, String subTaskId ,String termCode) {
        int result = 0;
        String arrangeId = bSubtaskRepository.queryArrangeIdById(subTaskId);
        List<String> subTaskStatus = bArrangeRepository.querySubTaskStatus(subTaskId);
        if(subTaskStatus.size()>1||subTaskStatus.get(0).equals("0")||subTaskStatus.get(0).equals("1")){
            //当前子任务下有教学单位未完成审核，不能进行开课
            throw new ServiceException(ResultServiceEnums.AGENCY_NOT_PASS);
        }
        List<String> subAgencyIds = bArrangeRepository.querySubAgencyId(subTaskId);
        if(subAgencyIds.size()<1){
            //子任务下没有教学单位
            throw new ServiceException(ResultServiceEnums.NO_AGENCY_SUBMISSION);
        }
        List<BSl> sls = new ArrayList<>();
        List<BLessonDirection> bLessonDirections = new ArrayList<>();
        List<BRSlStudent> slStudents = new ArrayList<>();
        int maxSlCode = bTermRepository.queryMaxSlCodeByTermCode(termCode);
        for(String subAgencyId : subAgencyIds){
            //查询要开的课
            List<ScheduleDto> schedules = bArrangeRepository.queryLessonsBySubAgencyId(subAgencyId);
            for(ScheduleDto schedule : schedules){
                BSl sl = new BSl();
                //根据lessonCode查询课程信息
                BLesson bLesson = bLessonRepository.queryBLessonByCode(schedule.getLessonCode());

                //学期编号+学期中存放的开课序号最大值+1作为新的开课序号

                //验证开课序号是否重复
                String slCode = "";
                for(int i=1;i<99999;i++){
                    slCode = termCode+String.valueOf(maxSlCode+i);
                    if(bSlRepository.selectOne(new BSl(slCode))==null)
                        maxSlCode = maxSlCode + i;
                        break;
                }
                sl.setCode(slCode);//课序号
                sl.setLessonCode(schedule.getLessonCode());//课程编号
                sl.setTeacherCode(schedule.getTeacherCode());//教师编号
                sl.setTermCode(termCode);//学期编号
                sl.setAuditStatus("0");//审核状态
                sl.setClassHour(bLesson.getClassHour());//课时
                sl.setCredits(bLesson.getCredits());//学分
                sl.setSource("1");
                sl.setSlCoverUrl(global.getDEFAULT_COURSE_LOCATION());
                UserUtil.setCurrentPersistOperation(sl);
                sls.add(sl);
                //修改b_sl_schedule slId
                bSlScheduleRepository.updateSlIdBySubAgencyId(sl.getBusinessId(),subAgencyId);
                //课程方向
                BLessonDirection bLessonDirection = new BLessonDirection();
                bLessonDirection.setBusinessId(IdGen.uuid());
                bLessonDirection.setSlId(sl.getBusinessId());
                bLessonDirection.setDirectionId(bLesson.getDirectionId());
                bLessonDirections.add(bLessonDirection);

                //构造学生选课信息
                List<BStudent> students = bStudentRepository.queryStudentByEduClassCode(schedule.getClassCode());
                for(BStudent student : students){
                    BRSlStudent brSlStudent = new BRSlStudent();
                    brSlStudent.setBusinessId(IdGen.uuid());
                    brSlStudent.setSlCode(sl.getCode());
                    brSlStudent.setStudentCode(student.getCode());
                    slStudents.add(brSlStudent);
                }


                //复制章节及文件目录
                chapterService.addBslFormLesson(null,sl.getLessonCode(), sl.getBusinessId());
            }

        }

        BSubtask bSubtask = new BSubtask(subTaskId,"5");
        bSubtaskRepository.updateByPrimaryKeySelective(bSubtask);//将子任务状态置为已开课
        bTermRepository.MaxSlCodeByTermCode(termCode,String.valueOf(maxSlCode));//将该学期的开课序号存入
        bDirectionRepository.insertList(bLessonDirections);//批量插入课程方向
        brSlStudentRepository.insertList(slStudents);//批量插入学生选课关系
        //批量插入
        result =bSlRepository.insertList(sls);

        //查询此排课下是否还存在没有开课的子任务
        //int count = bSubtaskRepository.judgeSubtask(arrangeId);
        //如果没有状态不为5的子任务，则设定此排课任务完成
        /*if(count == 0){
            BArrange bArrange = new BArrange();
            bArrange.setBusinessId(arrangeId);
            bArrange.setStatus("3");
            bArrangeRepository.updateByPrimaryKeySelective(bArrange);
        }*/
        return result;
    }

    @Override
    public PersistModel release(String businessId) {
        BArrange bArrange = new BArrange(businessId, "2");
        int line = bArrangeRepository.updateByPrimaryKeySelective(bArrange);
        return new PersistModel(line);
    }

    @Override
    public Map<String,Object> selectSubAgencyAndOption(String subAgencyId) {
        Map<String,Object> map = new HashMap<>();
        BSubAgency bSubAgency = bArrangeRepository.selectSubAgencyAndOption(subAgencyId);
        BTerm bTerm = bTermRepository.queryBtermByCode(bSubAgency.getTermCode()).get(0);
        Map<String,Object> weeks = Week.buildWeek(bTerm.getStartDate(),bTerm.getEndDate());
        List<BTeacher> bTeachers = bTeacherRepository.selectTeacherBySubAgencyId(subAgencyId);
        List<BEduClass> bEduClasses = bEduClassRepository.selectClassBySubAgencyId(subAgencyId);
        map.put("bSubAgency",bSubAgency);
        map.put("teacherList",bTeachers);
        map.put("classList",bEduClasses);
        int currentlyWeek = (int) weeks.get("currentlyWeek");
        if(currentlyWeek == 0){
            currentlyWeek = -1;
        }
        map.put("currentlyWeek",currentlyWeek);
        return map;
    }

    /**
     * 子任务提交 修改subAgency的status 0 -> 2
     * @param businessId
     * @return
     */
    @Override
    public PersistModel submit(String businessId) {
        return new PersistModel(1, bArrangeRepository.submitSubAgency(businessId));
    }

    //设置具体信息
    public List<BSlSchedule> setDetail(int i,String[] value,String subAgencyId,Date date,String termCode,Boolean contanSunday){
        List<BSlSchedule> list = new ArrayList<>();
        int location1 = value[i].indexOf("-");
        String start1 = value[i].substring(0,location1);
        int numStart1 = Integer.valueOf(start1);
        String end1 = value[i].substring(location1 + 1,value[i].length());
        int numEnd1 = Integer.valueOf(end1);
        //按照星期几划分,将所有当前天的课程放入一个List集合
        for(int j = numStart1 ; j <= numEnd1 ; j++){
            BSlSchedule bSlSchedule = new BSlSchedule();
            bSlSchedule.setTeaCode(value[14]);
            bSlSchedule.setSeq(String.valueOf(j));
            if(contanSunday){
                bSlSchedule.setRoomCode(value[27]);
            }else{
                bSlSchedule.setRoomCode(value[26]);
            }
            bSlSchedule.setSubAgencyId(subAgencyId);
            bSlSchedule.setStatus("0");
            bSlSchedule.setSlState("0");
            bSlSchedule.setStartDay(date);
            bSlSchedule.setLessonCode(value[3]);
            bSlSchedule.setEduClassCode(value[16]);
            bSlSchedule.setTermCode(termCode);
            UserUtil.setCurrentPersistOperation(bSlSchedule);
            list.add(bSlSchedule);
        }

        return list;
    }


    /**
     * 删除排课
     * @param businessId
     * @return
     */
    @Override
    public PersistModel remove(String businessId) {
        return new PersistModel(1,bArrangeRepository.deleteArrange(businessId));
    }


}
