package org.rcisoft.business.bcarousel.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bcarousel.entity.BCarousel;
import org.rcisoft.business.bcarousel.service.BCarouselService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UploadUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.Map;

/**
* Created by gwf on 2017-8-4 16:30:27.
*/
@RestController
@RequestMapping("/BCarousel")
public class BCarouselController extends PaginationController<BCarousel> {

    @Autowired
    private BCarouselService bCarouselServiceImpl;

    @Autowired
    private Global global;

    @ApiOperation(value="添加/编辑轮播图", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "carouselDes", value = "轮播描述（1-200）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "carouselUrl", value = "轮播图片（1-200）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "carouselSort", value = "排序", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isOpen", value = "是否显示 0未显示，1显示", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isNewWindow", value = "是否跳转窗口", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "linkUrl", value = "链接（0-200）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageType", value = "图片类型（0-30）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageId", value = "图片id（0-60）", required = false, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/add")
    public Result add(BCarousel bCarousel) {
        PersistModel data = bCarouselServiceImpl.persist(bCarousel,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCarousel);
    }

    @ApiOperation(value="删除轮播图", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bCarouselServiceImpl.removeBCarousel(id,getToken());
        return Result.builder(data,MessageConstant.MESSAGE_ALERT_SUCCESS,MessageConstant.MESSAGE_ALERT_ERROR,id);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCarouselServiceImpl.selectOne(businessId));
    }


    @ApiOperation(value="获取图片分页", notes="根据URL中的参数分页查询图片")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "carouselSort", value = "排序", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isOpen", value = "是否显示 0未显示，1显示", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageType", value = "图片类型", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageId", value = "图片id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBCarouselsByPaginationWithoutAuth")
    public GridModel queryBCarouselsByPagination(BCarousel param) {
        bCarouselServiceImpl.queryBCarouselsByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="获取图片", notes="根据URL中的参数分页查询全部图片")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "carouselSort", value = "排序", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isOpen", value = "是否显示 0未显示，1显示", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageType", value = "图片类型", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageId", value = "图片id", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBCarousels")
    public Result queryBCarousels(BCarousel param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCarouselServiceImpl.queryBCarousels(param));

    }

    @ApiOperation(value="上传图片", notes="上传图片到服务器")
    @ApiImplicitParam(name = "file", value = "图片文件", required = true, dataType = "MultipartFile")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/upload")
    public Result upload(MultipartFile file){
        String path = global.getBASE_UPLOAD_SERVER_LOCATION();
        String sl_logo_path = global.getIMAGE_LOCATION()+global.getIMAGES_LOCATION();
        boolean type = false;
        if("1".equals(global.getIS_SERVER_LINUX())){
            type = true;
        }
        Map<String, Object> uploadResult = UploadUtil.picImport(path, sl_logo_path, file, type);
        if ((boolean) uploadResult.get(UploadUtil.IS_SUCCESSS)) {
            String url = (String) uploadResult.get(UploadUtil.URL);
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,url);

        } else
            throw new ServiceException(ResultServiceEnums.UPLOAD_FILE_ERROR);
    }
}
