package org.rcisoft.business.bcarousel.service.impl;

import org.rcisoft.business.bcarousel.dao.BCarouselRepository;
import org.rcisoft.business.bcarousel.entity.BCarousel;
import org.rcisoft.business.bcarousel.service.BCarouselService;
import org.rcisoft.business.bimage.dao.BImageRepository;
import org.rcisoft.business.bimage.entity.BImage;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.DelStatus;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by gwf on 2017-8-4 16:30:27.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BCarouselServiceImpl implements BCarouselService {
    @Autowired
    private BCarouselRepository bCarouselRepository;

    @Autowired
    private BImageRepository bImageRepository;

    @Override
    public BCarousel selectOne(String businessId){
        return bCarouselRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BCarousel> queryBCarouselsByPagination(PageUtil pageUtil, BCarousel model) {
        return bCarouselRepository.queryBCarousels(getParames(model));
    }

    @Override
    public List<BCarousel> queryBCarousels(BCarousel model) {
        Map param = new HashMap<String,Object>();
        return bCarouselRepository.queryBCarousels(getParames(model));
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel persist(BCarousel model, String token){
        int line = 0;
        if(!StringUtils.isEmpty(model.getBusinessId())){
            BImage image = new BImage();
            UserUtil.setCurrentMergeOperation(image);
            bImageRepository.updateTime(image);
            UserUtil.setCurrentMergeOperation(model);
            line = bCarouselRepository.updateCarouselById(model);
        }else{
            UserUtil.setCurrentPersistOperation(model);
            line = bCarouselRepository.insertSelective(model);
        }
        if(line!=1)
            throw new ServiceException(ResultServiceEnums.UPDATE_ERROR);
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel removeBCarousel(String id, String token) {
        BCarousel bCarousel = bCarouselRepository.selectByPrimaryKey(id);
        bCarousel.setDelFlag(DelStatus.DELETED.getStatus());
        UserUtil.setCurrentMergeOperation(bCarousel);
        int line = bCarouselRepository.updateByPrimaryKeySelective(bCarousel);
        return new PersistModel(line);
    }

    private Map<String,Object> getParames(BCarousel model){
        Map param = new HashMap<String,Object>();
        param.put("imageId", model.getImageId());
        param.put("isOpen",model.getIsOpen());
        return param;
    }
}
