package org.rcisoft.business.bchapter.entity;


import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;
import org.hibernate.validator.constraints.NotEmpty;
import org.hibernate.validator.constraints.Range;
import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.core.entity.IdEntity;

import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.util.Date;
import java.util.List;

/**
* Created with gwf on 2017-7-21 15:08:46.
*/
@Entity
@Table(name = "b_chapter")
@Data
@NoArgsConstructor
@AllArgsConstructor
public class BChapter extends IdEntity<BChapter> {

    private static final long serialVersionUID = -8023388256391391541L;
    /*开课id*/
    //@NotEmpty(message = "开课id必填")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String slId;
    /*章节名称*/
    @NotBlank
    @Length(min = 1,max = 50,message = "长度最小为1,最大为50")
    private String chapterName;
    /*等级*/
    @Range(max = 2,min = 1,message = "等级最小为1,最大为2")
    private Byte chapterLevel;
    /*父ID*/
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String pid;
    /*课程url*/
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String chapterUrl;

    /**任务书*/
    private String mdFile;

    /*1:实验  2:视频   3:PPT*/
    //@Length(min = 1,max = 20,message = "长度最小为1,最大为15")
    private String isTest;

    /*课程或开课   0：开课   1：课程*/
    private String lessonOrSl;

    /*状态 0关闭 1开启*/
    private String state;

    /*课程ID*/
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String lessonId;

    /*课时*/
    @Length(min = 1,max = 8,message = "长度最小为1,最大为8")
    private String classHour;

    /*是否智评  0否  1是*/
    private String automatic;

    /*实验类型,1:java单文件 2:java工程 3:html单文件 4:html工程*/
    private String experimentType;

    /*章节排序*/
    private Integer sort;
    @Transient
    private List<BChapterRepository> childList;

    @Transient
    private String studentNum;
    @Transient
    private String stuScore;


    @Transient
    private String videoUrl;

    @Transient
    private String fileUrl;

    @Transient
    private String pptUrl;

    @Transient
    private String pdfUrl;

    /**
     * 课程的父ID(当作map的key存放开课的父ID用)
     */
    @Transient
    private String lPid;

    @Transient
    private String sPid;

    /*学生的用户ID，查询实验是否完成时用*/
    @Transient
    private String userId;

    @Transient
    private String stuId;

    @Transient
    private String isComplete;

    @Transient
    private String studentName;

    @Transient
    private String type;

    @Transient
    private String pptFinish;

    @Transient
    private String pdfFinish;

    @Transient
    private String videoFinish;

    @Transient
    private String paperFinish;

    @Transient
    private String paperUrl;

    @Transient
    private String fileName;


    public BChapter(String businessId) {
        this.businessId = businessId;
    }
}

