package org.rcisoft.business.bclass.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.bagency.dao.BAgencyRepository;
import org.rcisoft.business.bagency.entity.BAgency;
import org.rcisoft.business.bclass.dao.BClassRepository;
import org.rcisoft.business.bclass.entity.BClass;
import org.rcisoft.business.bclass.service.BClassService;
import org.rcisoft.business.brclassstudent.dao.BRClassStudentRepository;
import org.rcisoft.business.bstudent.entity.BStudent;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.model.SelectModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BClassServiceImpl implements BClassService {
    @Autowired
    private BClassRepository bClassRepository;
    @Autowired
    private BRClassStudentRepository brClassStudentRepository;
    @Autowired
    private BAgencyRepository bAgencyRepository;

    @Override
    public BClass selectOne(String businessId){
        return bClassRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BClass> queryBClasssByPagination(PageUtil pageUtil, BClass model) {
        return bClassRepository.queryBClasss(model);
    }

    @Override
    public List<BClass> queryBClasss(BClass model) {
        return bClassRepository.queryBClasss(model);
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel persist(BClass model, String token){
        int line = 0;
        if(model.getBusinessId()==null) {
            if (bClassRepository.selectOne(new BClass(model.getCode())) != null)
                throw new ServiceException(ResultServiceEnums.CODE_IS_EXISTS);
            UserUtil.setCurrentPersistOperation(model);
            line = bClassRepository.insertSelective(model);
        }else {
            if (bClassRepository.selectClassesByName(new BClass(model.getBusinessId(),model.getClassName())).size()>0)
                throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
            UserUtil.setCurrentMergeOperation(model);
            line = bClassRepository.updateByPrimaryKeySelective(model);
        }
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel removeBClass(String id) {
        if(brClassStudentRepository.selectStuentNumByClassId(id)>0) {
            throw new ServiceException(ResultServiceEnums.DATA_HAS_USED);
        }
        int line = bClassRepository.deleteByPrimaryKey(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    /**
     @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
     @Override
     public PersistModel importExcel(HSSFWorkbook hwb, String token) throws IOException {
     ArrayList<BClass> bClasses = new ArrayList<BClass>();
     ArrayList<String> repeatCode = new ArrayList<String>();

     String[] headers = {"班级编号","班级名称"};
     ArrayList<String[]> values = ExcelUtil.importExcel(hwb,headers,false);  //获取excel数据

     if (values.size()<=0){
     throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
     }

     for(String[] value:values){   //将数据封装到entity
     BClass bClass = null;
     BClass b = new BClass(value[0]);
     if(value[0]==null||value[0].equals("")||value[1]==null||value[1].equals("")){
     continue;
     }
     if((bClass=bClassRepository.selectOne(b))!=null) {
     repeatCode.add(value[0]);
     continue;
     }
     bClass= new BClass(value[0],value[1]);
     bClass.setCommonBusinessId();
     UserUtil.setCurrentPersistOperation(bClass);

     if(!bClasses.contains(bClass)){
     bClasses.add(bClass);
     }
     }

     if(bClasses.size()<1){
     if(repeatCode.size()<1)
     throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
     else
     throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST.getCode(),"Excel内所有数据之前都已经导入:"+ JSON.toJSONString(repeatCode));
     }

     String result = ExcelUtil.checkRepeat(repeatCode);

     int line = bClassRepository.insertList(bClasses);
     return new PersistModel(line,result);
     }
     */

    /**
     * 行政班导入
     * @param hwb
     * @param token
     * @return
     * @throws IOException
     */
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel administrativeImportExcel(HSSFWorkbook hwb, String token) throws IOException {
        ArrayList<BClass> bClasses = new ArrayList<BClass>();
        ArrayList<String> repeatCode = new ArrayList<String>();
        String innerRepeatResult = "";
        String[] headers = {"班级编号","学号","所属年级"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb,headers,false,1);  //获取excel数据
        //判断年级是否存在
        List<String> valuesGrade = new ArrayList<>();

        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        //外循环
        outLoop : for(String[] value:values){   //将数据封装到entity
            BClass bClass = null;
            BClass b = new BClass(value[0]);
            if(value[0]==null||value[0].equals("")||value[1]==null||value[1].equals("")){
                continue;
            }
            if((bClass=bClassRepository.selectOne(b))!=null) {
                repeatCode.add(value[0]);
                continue;
            }
            //判断:学年是否存在，存在才能录入
            BClass b1 = new BClass();
            b1.setGradeCode(value[2]);
            if (bClassRepository.checkGrade(b1).size() == 0){
                valuesGrade.add(value[2]);
                continue;
            }
            //表中去重
            for(BClass bClass1 : bClasses){
                if(b1.getCode().equals(bClass1.getCode())){
                    innerRepeatResult += "编号 " + b1.getCode() + " 在Excel中重复出现，除第一条以外的数据插入失败 ";
                    continue outLoop;
                }
            }

            bClass= new BClass(value[0],value[1]);
            bClass.setGradeCode(value[2]);
            bClass.setCommonBusinessId();
            UserUtil.setCurrentPersistOperation(bClass);

            if(!bClasses.contains(bClass)){
                bClasses.add(bClass);
            }
        }

        if(bClasses.size()<1){
            if(repeatCode.size()<1)
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
            else
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST.getCode(),"Excel内所有数据之前都已经导入:"+ JSON.toJSONString(repeatCode));
        }

        String result = ExcelUtil.checkRepeat(repeatCode);

        int line = bClassRepository.insertList(bClasses);
        if(valuesGrade.size()>0){
            result+="以下班级的所属年级信息不存在："+ JSON.toJSONString(valuesGrade)+"。";
        }

        return new PersistModel(line,result + innerRepeatResult);
    }

    @Override
    public PersistModel businessImportExcel(HSSFWorkbook hwb, String token) {
        ArrayList<BClass> bClasses = new ArrayList<BClass>();
        ArrayList<String> repeatCode = new ArrayList<String>();
        ArrayList<String> AgencyCode = new ArrayList<String>();
        String innerRepeatResult = "";
        String[] headers = {"班级编号","班级名称","所属年级","教学单位"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb,headers,false,1);  //获取excel数据
        //判断年级是否存在
        List<String> valuesGrade = new ArrayList<>();

        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        outLoop : for(String[] value:values){   //将数据封装到entity
            BClass bClass = null;
            BClass b = new BClass(value[0]);
            if(value[0]==null||value[0].equals("")||value[1]==null||value[1].equals("")){
                continue;
            }
            if((bClass=bClassRepository.selectOne(b))!=null) {
                repeatCode.add(value[0]);
                continue;
            }
            //判断:学年是否存在，存在才能录入
            BClass b1 = new BClass();
            b1.setGradeCode(value[2]);
            if (bClassRepository.checkGrade(b1).size() == 0){
                valuesGrade.add(value[2]);
                continue;
            }

            //判断教学单位是否存在
            List<BAgency> bAgencies =  bAgencyRepository.queryBAgencysByCode(value[3]);
            if(bAgencies.size()<1){
                AgencyCode.add(value[3]);
                continue;
            }

            bClass= new BClass(value[0],value[1]);
            bClass.setGradeCode(value[2]);
            bClass.setAgencyCode(value[3]);
            bClass.setType("1");
            bClass.setCommonBusinessId();
            UserUtil.setCurrentPersistOperation(bClass);
            //表中去重
            for(BClass bClass1 : bClasses){
                if(b1.getBusinessId().equals(bClass1.getBusinessId())){
                    innerRepeatResult += "班级编号 " + b1.getBusinessId() + " 在Excel中重复出现，除第一条以外的数据插入失败 ";
                    continue outLoop;
                }
            }

            if(!bClasses.contains(bClass)){
                bClasses.add(bClass);
            }
        }

        if(bClasses.size()<1){
            if(repeatCode.size()<1)
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
            else
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST.getCode(),"Excel内所有数据之前都已经导入:"+ JSON.toJSONString(repeatCode));
        }

        String result = ExcelUtil.checkRepeat(repeatCode);

        int line = bClassRepository.insertBusinessClassList(bClasses);
        if(valuesGrade.size()>0){
            result+="以下班级的所属年级信息不存在："+ JSON.toJSONString(valuesGrade)+"。";
        }
        if(AgencyCode.size()>0){
            result+="以下教学单位不存在："+ JSON.toJSONString(AgencyCode)+"。";
        }

        return new PersistModel(line,result + innerRepeatResult);
    }

    @Override
    public List<SelectModel> queryBClassList(String type) {
        List<BClass> bClass = bClassRepository.queryBClassList(type);
        ArrayList<SelectModel> comBoxModel = new ArrayList<SelectModel>();
        for(BClass c : bClass){
            SelectModel selectModel = new SelectModel();
            selectModel.setId(c.getCode());
            selectModel.setText(c.getClassName());
            comBoxModel.add(selectModel);
        }
        return comBoxModel;
    }

    private Map<String,Object> queryParamHandler(BClass model){
        Map param = new HashMap<String, Object>();
        if(!model.getClassName().equals("")&&model.getClassName()!="")
            param.put("name","%"+model.getClassName()+"%");
        else
            param.put("name","%%");
        if(model.getStudentNum()!=null&&!model.getStudentNum().equals(""))
            param.put("num", Integer.parseInt(model.getStudentNum()));
        else
            param.put("num",null);
        return param;
    }
}
