package org.rcisoft.business.bcode.model;

import org.apache.commons.io.FileUtils;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;

import java.io.File;

/**
 * 智能评分
 * Created by yangzhaojun on 2018/1/22.
 *
 * 智能评分规则：
 *
 * 【单文件】
 *    1) 50——文件内容为空
 *    2) 60——文件内容非空
 *    3) 80——.java 文件字符个数>77； .html 文件字符个数>140；
 *    4) 100——.java 文件字符个数>140； .html 文件字符个数>280；
 *
 * 【多文件】
 *    1) 50——.java 文件个数为0或1；.html文件个数为0或1
 *    2) 60——.java 文件个数大于1；.html文件个数为大于1
 *    3) “ .java 文件字符个数>77”有一个就+10分，上限100；“ .html、.css、.js 文件字符个数>140”有一个就+10分，上限100；
 */
public class IntellEvaluation {

    public static int IntellEvaluation(String filePath, String studentCode, String experimentType) throws Exception{
        int score = 0;
        String content = "";

        if(experimentType.equals("1")){
            //java单文件
            File file = new File(filePath+studentCode+".java");
            if(!file.exists()){
                throw new ServiceException(ResultServiceEnums.STUDENT_CODE_NOT_EXISTS);
            }
            content = FileUtils.readFileToString(file);
            score = getJavaSingleScore(content);


        }else if(experimentType.equals("2")){
            //java工程
            File file = new File(filePath+studentCode);
            score = getJavaProjectScore(file);


        }else if(experimentType.equals("3")){
            //html单文件
            File file = new File(filePath+studentCode+".html");
            if(!file.exists()){
                throw new ServiceException(ResultServiceEnums.PASSWORD_ERROR);
            }
            content = FileUtils.readFileToString(file);
            score = getHtmlSingleScore(content);


        }else if(experimentType.equals("4")){
            //html工程
            File file = new File(filePath+studentCode);
            score = getHtmlProjectScore(file);


        }
        return  score;
    }


    //java单文件评分
    private static int getJavaSingleScore(String content) {
        int score = 0;
        if(!content.equals("")&&content!=null){
            if(content.length()>=77&&content.length()<140){
                score = 80;
            }else if(content.length()>=140){
                score = 100;
            }else{
                score = 60;
            }
        }else{
            score = 50;
        }
        return score;
    }


    //java工程评分
    private static int getJavaProjectScore(File file) throws Exception{
        if(!file.exists()){
            throw new ServiceException(ResultServiceEnums.PASSWORD_ERROR);
        }
        File[] dirFiles = file.listFiles();
        int score = 0;
        int javaFileNum = 0;//java文件个数
        int BonusScore = 0;//额外加分
        for(File javaFile : dirFiles){
            if(javaFile.isFile()&&javaFile.getAbsolutePath().endsWith(".java")){
                javaFileNum += 1;
                String content = FileUtils.readFileToString(javaFile);
                if(content.length()>=77){
                    BonusScore += 10;
                }
            }
        }
        if(javaFileNum>1){
            score = 60 + BonusScore;
            if(score>100){
                score = 100;
            }
        }else{
            score = 50 + BonusScore;
            if(score>100){
                score = 100;
            }
        }
        return score;
    }

    //html单文件评分
    private static int getHtmlSingleScore(String content) {
        int score = 0;
        if(!content.equals("")&&content!=null){
            if(content.length()>=140&&content.length()<280){
                score = 80;
            }else if(content.length()>=280){
                score = 100;
            }else{
                score = 60;
            }
        }else{
            score = 50;
        }
        return score;
    }


    //html工程评分
    private static int getHtmlProjectScore(File file)  throws Exception{
        if(!file.exists()){
            throw new ServiceException(ResultServiceEnums.PASSWORD_ERROR);
        }
        File[] dirFiles = file.listFiles();
        int score = 0;
        int htmlFileNum = 0;//java文件个数
        int BonusScore = 0;//额外加分
        for(File htmlFile : dirFiles){
            if(htmlFile.isFile()&&htmlFile.getAbsolutePath().endsWith(".html")){
                htmlFileNum += 1;
                String content = FileUtils.readFileToString(htmlFile);
                if(content.length()>=140){
                    BonusScore += 10;
                }
            }
            if((htmlFile.isFile()&&htmlFile.getAbsolutePath().endsWith(".css"))||(htmlFile.isFile()&&htmlFile.getAbsolutePath().endsWith(".js"))){
                String content = FileUtils.readFileToString(htmlFile);
                if(content.length()>=140){
                    BonusScore += 10;
                }
            }
        }
        if(htmlFileNum>1){
            score = 60 + BonusScore;
            if(score>100){
                score = 100;
            }
        }else{
            score = 50 + BonusScore;
            if(score>100){
                score = 100;
            }
        }
        return score;
    }
}
