package org.rcisoft.business.bcode.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.TriggerBuilder;
import org.quartz.TriggerKey;
import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bcode.model.*;
import org.rcisoft.business.bcode.service.BCodeLxcService;
import org.rcisoft.business.bcode.service.BCodeService;
import org.rcisoft.business.brslstudent.dao.BRSlStudentRepository;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.business.bstudent.entity.BStudentDto;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.CommandResult;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.ResultExceptionEnum;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.service.RcRedisService;
import org.rcisoft.core.service.SerializationUtils;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.ZipCompress;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.SchedulerFactoryBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.*;
import java.util.*;

/**
 * Created by lcy on 18/1/3.
 */
@Service
@Slf4j
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BCodeServiceImpl implements BCodeService {

    @Autowired
    private Global global;

    @Autowired
    private RcRedisService rcRedisServiceImpl;

    @Autowired
    private BChapterRepository bChapterRepository;

    @Autowired
    private BRSlStudentRepository brSlStudentRepository;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private BSlRepository bSlRepository;

    @Autowired
    private SchedulerFactoryBean schedulerFactoryBean;

    @Autowired
    private BCodeLxcService bCodeLxcServiceImpl;


    /**
     * 读文件
     * @param slId
     * @param chapId
     * @param examType
     * @param userInfo
     * @return
     */
    @Override
    public List<BCodeFile> readExamFiles(String slId, String chapId, String examType, String userInfo, String studentCode) {

        /*java 单文件*/
        if(examType.equals(global.getJavaSimple()))
            return this.readSimpleFile(slId,chapId,CodeType.SIM_JAVA,userInfo,studentCode);
        /*java 工程*/
        if(examType.equals(global.getJavaProject()))
            return this.readProjectFile(slId,chapId,userInfo,studentCode,global.getJavaProject());
        /*html 单文件*/
        if(examType.equals(global.getHtmlSimple()))
            return this.readSimpleFile(slId,chapId,CodeType.SIM_HTML,userInfo,studentCode);
        /*html 工程*/
        if(examType.equals(global.getHtmlProject()))
            return this.readProjectFile(slId,chapId,userInfo,studentCode,global.getHtmlProject());
        return null;
    }



    /**
     * 读取单文件
     * @param slId
     * @param chapId
     * @param examType
     * @param userInfo
     * @return
     */
    private List<BCodeFile> readSimpleFile(String slId, String chapId, CodeType examType, String userInfo, String studentCode) {
        List<BCodeFile> codeFileList = new ArrayList<>();
        BCodeFile codeFile = null;
        String path = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getSL_LOCATION() + File.separator
                + slId + File.separator
                + chapId + File.separator
                + global.getCOURSE_CODE_LOCATION() + File.separator
                + userInfo + File.separator;
        String file = path + studentCode + examType.getPostfix();
        try{
            File simplePath = new File(path);
            if(!simplePath.exists()) {
                simplePath.mkdirs();
            }
            File simpleFile = new File(file);
            if(!simpleFile.exists()) {
                simpleFile.createNewFile();
                codeFile = new BCodeFile("0", "", null, simpleFile.getName(),IdGen.uuid(), "","1");
                codeFileList.add(codeFile);
            }else{
                String content = FileUtils.readFileToString(simpleFile);
                ///List<String> fileName = new ArrayList<>();
                //fileName.add(simpleFile.getName());
                codeFile = new BCodeFile("0", content,new Date(simpleFile.lastModified()), simpleFile.getName(), IdGen.uuid(),"1","");
                codeFileList.add(codeFile);
            }
        }catch (Exception e){
            log.debug(file + " is not exist");
            codeFile = new BCodeFile("0", "", null, "","", "","0");
            codeFileList.add(codeFile);
        }

        return codeFileList;
    }


    /**
     * 读java工程
     * @param slId
     * @param userInfo
     * @return
     */
    private List<BCodeFile> readProjectFile(String slId, String chapterId, String userInfo, String StudentCode,
        String examCode) {
        BCodeLxc lxc = new BCodeLxc();
        Map param = new HashMap<>();
        List<BCodeFile> codeFile = new ArrayList<>();
        /*工程存储路径*/
        String path = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getSL_LOCATION() + File.separator
                + slId + File.separator
                + chapterId +File.separator
                + global.getCOURSE_PROJECT_LOCATION() + File.separator
                + userInfo + File.separator
                + StudentCode;
        /*工程 容器docker dir*/
        String dirPath = global.getPHYSICAL_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getSL_LOCATION() + File.separator
                + slId + File.separator
                + chapterId +File.separator
                + global.getCOURSE_PROJECT_LOCATION() + File.separator
                + userInfo + File.separator;
        lxc.setUserId(userInfo);
        lxc.setWorkingDir(dirPath);
        lxc.setCode(examCode);
        lxc.setUserCode(StudentCode);
        lxc.setSl(slId);
        lxc.setChapterId(chapterId);

        /*容器是否已启动*/
        byte[] results = rcRedisServiceImpl.getBytes(global.getLxcPrefix() + "_" + lxc.getUserId());
        if(null != results) {
            lxc = SerializationUtils.deserializer(results, BCodeLxc.class);
            /*别的课或别的节已开启容器,返回错误*/
            if(null != lxc && (!slId.equals(lxc.getSl()) || !chapterId.equals(lxc.getChapterId()))){
                throw new ServiceException(ResultExceptionEnum.EXIST_LXC_SERVER);
            }
            else if(null != lxc){
                /*容器参数*/
                param.put("port", lxc.getContainerPort());
                if(examCode.equals(global.getJavaProject()))
                    param.put("sshPort", lxc.getContainerPort()+10000);
                else
                    param.put("sshPort", "");
                param.put("endTime",lxc.getShutdownDate().getTime());
            }
            /*开启容器*/
            else
                this.startLxc(param,lxc,examCode);
        }
        else
            this.startLxc(param,lxc,examCode);
        try{
            File simplePath = new File(path);
            BCodeFile simpleCodeFile = null;
            if(!simplePath.exists()) {
                simplePath.mkdirs();
                simpleCodeFile = new BCodeFile("0", "", null, simplePath.getName(),IdGen.uuid(),"","0");
                codeFile.add(simpleCodeFile);
            }else{
                boolean isFile = true;
                simpleCodeFile = new BCodeFile("0", "", null, simplePath.getName(),IdGen.uuid(),"0","");
                //codeFile.add(simpleCodeFile);
                getFileList(simplePath,codeFile,simpleCodeFile,isFile);
            }
            codeFile.get(0).setLxcParam(param);

        }catch (Exception e){
            log.debug(path + " is not exist");
            BCodeFile simpleCodeFile = new BCodeFile("0", "", null,"","","0", "");
        }

        return codeFile;
    }

    /**
     * 启动
     * @param param
     * @param lxc
     */
    private void startLxc(Map param,BCodeLxc lxc,String examCode){
        CommandResult result = bCodeLxcServiceImpl.startBCodeLxc(lxc);
                    /*启动失败*/
        if(null == result ||  !result.isSuccess()){
            log.error("启动失败" + result.getMessage());
            throw new ServiceException(400,result.getMessage());
        }
        param.put("port", lxc.getContainerPort());
        if(examCode.equals(global.getJavaProject()))
            param.put("sshPort", lxc.getContainerPort()+10000);
        else
            param.put("sshPort", "");
        param.put("endTime",lxc.getShutdownDate().getTime());
    }


    /**
     * 递归方法读取文件及子目录
     * @param files
     * @param codeFile
     * @throws IOException
     */
    private void getFileList(File files,List<BCodeFile> codeFile,BCodeFile parentFile,boolean isFile) throws IOException {

        BCodeFile bCodeFile = new BCodeFile();
        bCodeFile.setContent("");
        bCodeFile.setIsFirstFile("0");
        bCodeFile.setLastModify(new Date(files.lastModified()));
        bCodeFile.setName(files.getName());
        bCodeFile.setId(IdGen.uuid());
        bCodeFile.setPId(parentFile.getId());
        codeFile.add(bCodeFile);
        if(files.isFile()){
            if(isFile){
                bCodeFile.setContent(FileUtils.readFileToString(files));
                bCodeFile.setIsFirstFile("1");
            }

            bCodeFile.setFileType("1");
            isFile = false;
        }else{
            bCodeFile.setFileType("0");
            File[] ffs=files.listFiles();
            for (File ff : ffs) {
                getFileList(ff, codeFile, bCodeFile, isFile);
            }
        }







    }

    /**
     * 写文件
     * @param slId
     * @param chapId
     * @param examType
     * @param content
     * @param userInfo
     * @return
     */
    @Override
    public BCodeFile saveOwnExamFiles(String slId, String chapId, String examType, String content, String userInfo, String studentCode) {
        /*java 单文件*/
        if(examType.equals(global.getJavaSimple()))
            return this.saveOwnExamFiles(slId,chapId,CodeType.SIM_JAVA,content,userInfo,studentCode);
        /*java 工程*/
        if(examType.equals(global.getJavaProject()))
            return null;
        /*html 单文件*/
        if(examType.equals(global.getHtmlSimple()))
            return this.saveOwnExamFiles(slId,chapId,CodeType.SIM_HTML,content,userInfo,studentCode);
        /*html 工程*/
        if(examType.equals(global.getHtmlProject()))
            return null;
        return null;
    }


    /**
     * 写单文件
     * @param slId
     * @param chapId
     * @param examType
     * @param content
     * @param userInfo
     * @return
     */
    private BCodeFile saveOwnExamFiles(String slId, String chapId, CodeType examType, String content, String userInfo, String studentCode) {
        BCodeFile codeFile = null;
        String path = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getSL_LOCATION() + File.separator
                + slId + File.separator
                + chapId + File.separator
                + global.getCOURSE_CODE_LOCATION() + File.separator
                + userInfo + File.separator;
        String file = path + studentCode + examType.getPostfix();
        try{
            File simplePath = new File(path);
            if(!simplePath.exists()) {
                simplePath.mkdirs();
            }
            File simpleFile = new File(file);
            if(!simpleFile.exists()) {
                simpleFile.createNewFile();
            }
            FileUtils.write(simpleFile,content,"UTF-8");
            //List<String> fileName = new ArrayList<>();
            //fileName.add(simpleFile.getName());
            codeFile = new BCodeFile("0", content,new Date(simpleFile.lastModified()),simpleFile.getName(),IdGen.uuid(),"0", "");

        }catch (Exception e){
            log.debug(file + "创建失败");
            codeFile = new BCodeFile("0", "", null, "","","0","");
        }
        return codeFile;
    }


    /**
     * 导出作业
     * @param slId
     * @param studentId
     * @param type
     * @param slCode
     * @return
     */
    @Override
    public String exportFiles(String slId, String studentId, String type, String slCode) {
        String compressPath = "";
        String SlPath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getSL_LOCATION() +File.separator
                + slId + File.separator;
        //得到开课的编号和名称
        BSl sl = bSlRepository.queryBSlById(slId);
        List<BStudentDto> students;
        if(type.equals("1")) {//全部导出
            studentId="";
        }
        //根据slId获得章节及学生
        if(studentId.equals("")){//工程
            students = brSlStudentRepository.queryStudentBySlId1(slId,studentId);
        }else {
            students = brSlStudentRepository.queryStudentBySlId2(slId,studentId);
        }

        //查询该课程的所有章节
        //List<BChapter> chapterList = bChapterRepository.queryChapterBySlId(slId);
        if(students.size()>=1){
            for(BStudentDto student : students){
                String codeType = global.getCOURSE_CODE_LOCATION() + File.separator;//默认是单文件
                if(student.getExperimentType().equals("2")||student.getExperimentType().equals("4")){//工程
                    codeType = global.getCOURSE_PROJECT_LOCATION() + File.separator;
                }
                //得到节的排序 如 1.1   2.2
                String sort = student.getPSort()+"."+student.getSort();
                //将要导出的文件暂存入这个文件夹中
                String copyPath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                        + global.getTEMP_LOCATION() + File.separator
                        + global.getSTUDENT_CODE_LOCATION() + File.separator
                        + sl.getCode()+ "+" + sl.getLessonName() + File.separator
                        + sort + File.separator;
                //压缩后的文件
                compressPath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                        + global.getTEMP_LOCATION() + File.separator
                        + global.getSTUDENT_CODE_LOCATION() + File.separator
                        + sl.getCode()+ "+" + sl.getLessonName();
                //  以学号+姓名命名文件夹
                String finalPath = copyPath + student.getStudentCode()
                        + "+"
                        + student.getStudentName()
                        + File.separator;
                File copyFile = new File(finalPath);
                if(!copyFile.exists()){
                    copyFile.mkdirs();
                }
                //学生代码文件所在目录
                String codePath = SlPath + student.getChapterId() +File.separator
                        + codeType + student.getStudentId() + File.separator;
                File codeFile = new File(codePath);
                if(!codeFile.exists()){
                    codeFile.mkdirs();
                }
                try {
//                    if(student.getExperimentType().equals("2")||student.getExperimentType().equals("4")){
                        FileUtils.copyDirectory(codeFile,copyFile);
//                    }else {
//                        File javaFile[] = codeFile.listFiles();
//                        for(File simpleFile : javaFile){
//                            File finalFile = new File(finalPath + simpleFile.getName());
//                            FileUtils.copyFile(javaFile[0],finalFile);
//                        }
//
//                    }
                } catch (Exception e) {
                    System.out.println("复制逻辑出错，可能路径不对！");
                    e.printStackTrace();
                }


//                File userFile = new File(codePath);
//                //得到目录下的所有文件
//                File javaFile[] = userFile.listFiles();
//                if(javaFile!=null) {
//                    for (File simpleFile : javaFile) {
//                        //将文件复制出去
//                        try {
//                            int bytesum = 0;
//                            int byteread = 0;
//                            InputStream inStream = new FileInputStream(simpleFile.getPath()+File.separator+"42432"); //读入原文件
//                            FileOutputStream fs = new FileOutputStream(finalPath + simpleFile.getName());
//                            byte[] buffer = new byte[102400];
//                            int length;
//                            while ((byteread = inStream.read(buffer)) != -1) {
//                                bytesum += byteread; //字节数 文件大小
//                                fs.write(buffer, 0, byteread);
//                            }
//                            inStream.close();
//                            fs.close();
//                        } catch (Exception e) {
//                            System.out.println("复制时出错");
//                            e.printStackTrace();
//                        }
//                    }
//                }
            }
        }else{
            throw new ServiceException(ResultServiceEnums.NO_STUDENT_HOMEWORK);
        }
        try {
            //将文件打包
            this.compressFiles(compressPath);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return sl.getCode()+ "+" + sl.getLessonName() + ".zip";
    }


    /**
     * 压缩文件
     * @param compressPath
     * @return
     * @throws IOException
     */

    private String compressFiles(String compressPath) throws IOException {
        String zipFile = "";
        if (!compressPath.equals("")) {
            String copyPath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                    + global.getTEMP_LOCATION() + File.separator
                    + global.getSTUDENT_CODE_LOCATION();
            File compressFile = new File(compressPath);
            if(compressFile.exists()){
                compressFile.delete();
            }
            String zipFileName = compressPath.substring(compressPath.lastIndexOf(File.separator),compressPath.length());
            zipFile = copyPath+zipFileName+".zip";
            ZipCompress.compressExe(compressPath,zipFile);
            //压缩完之后删除
            /*File file = new File(compressPath);
            if(file.exists()){
                file.delete();
            }*/
        }
        return zipFile;
    }


    /**
     * 查看所有学生代码
     * @param slId
     * @param chapterId
     * @return
     */
    @Override
    public List<StudentFile> queryAllStudentCode(String slId, String chapterId){
        List<StudentFile> studentFiles = new ArrayList<>();
        BCodeFile codeFile = null;

        List<BChapter> bChapters = bChapterRepository.queryBchapterStudentById(chapterId);

        String codePath = "";
        String suffixName = "";
        for(BChapter chapter : bChapters) {
            if (chapter.getExperimentType().equals("1")) {//java单文件
                codePath = global.getCOURSE_CODE_LOCATION();
                suffixName = ".java";
            } else if (chapter.getExperimentType().equals("2")) {//java工程
                codePath = global.getCOURSE_PROJECT_LOCATION();
            } else if (chapter.getExperimentType().equals("3")) {//html单文件
                suffixName = ".html";
                codePath = global.getCOURSE_CODE_LOCATION();
            } else {//html工程
                codePath = global.getCOURSE_PROJECT_LOCATION();
            }
            String path = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                    + global.getCOURSE_LOCATION() + File.separator
                    + global.getSL_LOCATION() + File.separator
                    + slId + File.separator
                    + chapterId + File.separator
                    + codePath + File.separator
                    + chapter.getUserId() + File.separator
                    + chapter.getStuId()
                    + suffixName;
            String studentName = chapter.getStuId()+" "+chapter.getStudentName();
            try {
                File file = new File(path);
                if (file.exists()) {
                    if (file.isFile()) {//单文件的处理
                        String content = FileUtils.readFileToString(file);
                        codeFile = new BCodeFile("0", content, new Date(file.lastModified()), file.getName(), IdGen.uuid(), "0", "",null);
                        StudentFile studentFile = new StudentFile();
                        studentFile.setBCodeFile(codeFile);
                        studentFile.setStudentName(studentName);
                        studentFile.setName(chapter.getStudentName());
                        studentFile.setProjectOrFile("0");
                        studentFiles.add(studentFile);
                    }else if(file.isDirectory()){//工程的处理
                        List<BCodeFile> codeFiles = new ArrayList<>();
                        BCodeFile simpleCodeFile = null;
                        boolean isFile = true;
                        simpleCodeFile = new BCodeFile("0", "", null, file.getName(),IdGen.uuid(),"0","");
                        getFileList(file,codeFiles,simpleCodeFile,isFile);
                        StudentFile studentFile = new StudentFile();
                        studentFile.setProject(codeFiles);
                        studentFile.setName(chapter.getStudentName());
                        studentFile.setStudentName(studentName);
                        studentFile.setProjectOrFile("1");
                        studentFiles.add(studentFile);
                    }
                }
            }catch (Exception e){

            }
        }
        return studentFiles;
    }

    @Override
    public Integer isOpenLxcFromUser(String userId,String slId,String chapterId) {
        BCodeLxc lxc = null;
        /*容器是否已启动*/
        byte[] results = rcRedisServiceImpl.getBytes(global.getLxcPrefix() + "_" + userId);
        if(null != results) {
            lxc = SerializationUtils.deserializer(results, BCodeLxc.class);
            /*别的课或别的节已开启容器,返回错误*/
            if(null != lxc && (!slId.equals(lxc.getSl()) || !chapterId.equals(lxc.getChapterId()))){
                /*已开*/
                return 0;
            }
            /*再次打开已开启的容器*/
            return 1;
        }
        /*未开*/
        return 1;
    }

    @Override
    public CommandResult stopExam(String userId) {
        BCodeLxc lxc = null;
        /*容器是否已启动*/
        log.info("userId:+++++++++++++"+userId);
        byte[] results = rcRedisServiceImpl.getBytes(global.getLxcPrefix() + "_" + userId);
        if(null != results) {
            lxc = SerializationUtils.deserializer(results, BCodeLxc.class);
            /*别的课或别的节已开启容器,返回错误*/
            if(null == lxc)
                return new CommandResult(ResultCode.FAIL,"未开启实验",null);
            else {
                /*关闭容器*/
                return bCodeLxcServiceImpl.stopBCodeLxc(lxc);
            }
        }
        /*未开*/
        return new CommandResult(ResultCode.FAIL,"未开启实验",null);

    }


    /**
     * 获取智评分数
     * @param slId
     * @param chapterId
     * @param studentCode
     * @return
     */
    @Override
    public int getIntelligenceScore(String slId, String chapterId, String studentCode) {
        int score = 0;
        try {
            BChapter bChapter = bChapterRepository.selectOne(new BChapter(chapterId));
            String sutId = sysUserMapper.queryUserIdByLoginName(studentCode);

            String codeType = global.getCOURSE_CODE_LOCATION();
            if (bChapter.getExperimentType().equals("2") || bChapter.getExperimentType().equals("4")) {
                codeType = global.getCOURSE_PROJECT_LOCATION();
            }
            String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                    + global.getCOURSE_LOCATION() + File.separator
                    + global.getSL_LOCATION() + File.separator
                    + slId + File.separator
                    + chapterId + File.separator
                    + codeType + File.separator
                    + sutId + File.separator;
            score = IntellEvaluation.IntellEvaluation(filePath, studentCode, bChapter.getExperimentType());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return score;
    }



    @Override
    public CommandResult delayExam(String userInfoProp) {
        BCodeLxc lxc = null;
        /*容器是否已启动*/
        byte[] results = rcRedisServiceImpl.getBytes(global.getLxcPrefix() + "_" + userInfoProp);
        if(null != results) {
            lxc = SerializationUtils.deserializer(results, BCodeLxc.class);
            /*别的课或别的节已开启容器,返回错误*/
            if(null == lxc)
                return new CommandResult(ResultCode.FAIL,"未开启实验",null);
            else {
                /*延长*/
                if(lxc.getShutdownDate().after(new Date(new Date().getTime() + global.getLxcEndlineTime() * 60 * 1000)))
                    return new CommandResult(ResultCode.FAIL,"距实验结束时间结束大于" + global.getLxcEndlineTime() + "分钟,无需延长",null);
                /*可以延长*/
                /*1.延长时间*/
                Date endTime = new Date(new Date().getTime() + global.getLxcDelayTime() * 60 * 1000);
                lxc.setShutdownDate(endTime);
                rcRedisServiceImpl.setBytes(global.getLxcPrefix() + "_" + userInfoProp,SerializationUtils.serializer(lxc));
                /*2.延长调度任务*/
                try {
                    Trigger newTrigger = TriggerBuilder.newTrigger()
                            .withIdentity(global.getLxcPrefix() + lxc.getUserId())
                            .startAt(endTime).build();
                    // 触发器时间设定
                    schedulerFactoryBean.getScheduler().rescheduleJob(TriggerKey.triggerKey(global.getLxcPrefix() + userInfoProp),newTrigger);

                    return new CommandResult(ResultCode.SUCCESS,"延长成功",endTime);
                } catch (SchedulerException e) {
                    log.error(e.getMessage());
                    return new CommandResult(ResultCode.FAIL,"延长失败",null);
                }

            }
        }
        /*未开*/
        return new CommandResult(ResultCode.FAIL,"未开启实验",null);
    }

    @Override
    public List<BCodeFile> queryStudentExamFiles(String slId, String chapId, String examType, String studentCode) {
        /*java 单文件*/
        if(examType.equals(global.getJavaSimple()))
            return this.readStudentSimpleFile(slId,chapId,CodeType.SIM_JAVA,studentCode);
        /*java 工程*/
        if(examType.equals(global.getJavaProject()))
            return this.readStudentProjectFile(slId,chapId,studentCode);
        /*html 单文件*/
        if(examType.equals(global.getHtmlSimple()))
            return this.readStudentSimpleFile(slId,chapId,CodeType.SIM_HTML,studentCode);
        /*html 工程*/
        if(examType.equals(global.getHtmlProject()))
            return this.readStudentProjectFile(slId,chapId,studentCode);
        return null;
    }

    @Override
    public int checkStudentFiles(String slId) {
        int flag = 0;
        List<BStudentDto> students;
        //根据slId获得章节及学生
        students = brSlStudentRepository.queryStudentBySlId1(slId,"");
        if(students.size()>=1){
            flag = 1;
        }else {
            throw new ServiceException(ResultServiceEnums.NO_STUDENT_HOMEWORK);
        }
        return flag;
    }


    private List<BCodeFile> readStudentSimpleFile(String slId, String chapId, CodeType codeType, String studentCode) {
        List<BCodeFile> codeFileList = new ArrayList<>();
        BCodeFile codeFile = null;
        String userId = sysUserMapper.queryUserIdByLoginName(studentCode);
        String path = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getSL_LOCATION() + File.separator
                + slId + File.separator
                + chapId + File.separator
                + global.getCOURSE_CODE_LOCATION() + File.separator
                + userId + File.separator
                + studentCode + codeType.getPostfix();
        try{
            File simpleFile = new File(path);
            String content = FileUtils.readFileToString(simpleFile);
            codeFile = new BCodeFile("0", content,new Date(simpleFile.lastModified()), simpleFile.getName(), IdGen.uuid(),"1","");
            codeFileList.add(codeFile);
        }catch (Exception e){
            codeFile = new BCodeFile("0", "", null, "","", "","0");
            codeFileList.add(codeFile);
        }
        return codeFileList;
    }


    private List<BCodeFile> readStudentProjectFile(String slId, String chapId, String studentCode) {
        List<BCodeFile> codeFile = new ArrayList<>();
        String userId = sysUserMapper.queryUserIdByLoginName(studentCode);
        String path = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getSL_LOCATION() + File.separator
                + slId + File.separator
                + chapId +File.separator
                + global.getCOURSE_PROJECT_LOCATION() + File.separator
                + userId + File.separator
                + studentCode;

        try{
            File simplePath = new File(path);
            BCodeFile simpleCodeFile = null;
            boolean isFile = true;
            simpleCodeFile = new BCodeFile("0", "", null, simplePath.getName(),IdGen.uuid(),"0","");
            //codeFile.add(simpleCodeFile);
            getFileList(simplePath,codeFile,simpleCodeFile,isFile);

        }catch (Exception e){
            BCodeFile simpleCodeFile = new BCodeFile("0", "", null,"","","0", "");
            codeFile.add(simpleCodeFile);
        }

        return codeFile;
    }
}
