package org.rcisoft.business.bcode.task;

import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.rcisoft.business.bcode.model.BCodeLxc;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.command.LxcCommand;
import org.rcisoft.core.result.CommandResult;
import org.rcisoft.core.service.RcRedisService;
import org.rcisoft.core.service.SerializationUtils;
import org.rcisoft.core.util.TaskUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.SchedulerFactoryBean;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lcy on 18/1/10.
 *
 *
 * 项目启动时,清除已到期的容器
 *             解决因 服务器down 而造成的 stopLxcJob 未执行问题
 *
 *
 */
@Component
@Slf4j
public class ClearLxcJob implements Job {

    /*任务组名*/
    private static final String JOB_LXC_GROUP = "lxc_job_group";

    @Autowired
    private RcRedisService rcRedisServiceImpl;

    @Autowired
    private LxcCommand lxcCommand;

    @Autowired
    private Global global;

    @Autowired
    private SchedulerFactoryBean schedulerFactoryBean;


    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        /*所有启动的容器*/
        List<String> keys = rcRedisServiceImpl.getList("lxcKeys");
        if(null == keys || 0 == keys.size())
            return;
        /*name*/
        for (String key :
                keys) {
            this.stopLxc(key);
        }
    }

    /**
     * 关闭容器
     * @param key
     */
    private void  stopLxc(String key) {
        String containerPath = null;
        byte[] results = rcRedisServiceImpl.getBytes(key);
        BCodeLxc lxc = null;
        if (null != results) {
            lxc = SerializationUtils.deserializer(results, BCodeLxc.class);
            if (null == lxc)
                return;
            Date shutdown = lxc.getShutdownDate();
            /*未到点*/
            if(null != shutdown && shutdown.after(new Date())) {
                /*启动路径*/
                String dockerPath = global.getPHYSICAL_UPLOAD_SERVER_LOCATION()  +
                        global.getLxcDockerfilePath() + "/" + lxc.getUserId()  + "/";
                 /*任务参数*/
                Map jobParam = new HashMap();
                jobParam.put("redisKeyId",key);
                jobParam.put("containerName",this.getModelProject(lxc.getCode()) + "_" + lxc.getUserId());
                jobParam.put("containerPath",dockerPath);
                jobParam.put("containerPort",lxc.getContainerPort());
                JobDetail job = JobBuilder.newJob(StopLxcJob.class).
                        withIdentity(global.getLxcPrefix() + lxc.getUserId(), JOB_LXC_GROUP).
                        usingJobData(new JobDataMap(jobParam)).build();
                Trigger trigger = TriggerBuilder.newTrigger().forJob(job)
                        .usingJobData(new JobDataMap(jobParam))
                        .withIdentity(global.getLxcPrefix() + lxc.getUserId())
                        .startAt(lxc.getShutdownDate()).build();
                // 触发器时间设定
                /*触发器*/
                TaskUtil.startSchedule(schedulerFactoryBean.getScheduler(),trigger,job);

                return;
            }
            /*到点了*/
            containerPath = global.getPHYSICAL_UPLOAD_SERVER_LOCATION() + File.separator +
                    global.getLxcDockerfilePath() + File.separator + lxc.getUserId()  + File.separator;
            /*关闭容器*/
            /*2.docker-compose 起容器*/
            CommandResult commandResult = null;
            commandResult = lxcCommand.startOrDownLxc(containerPath,false);
            /*关闭失败 返回 不删除key*/
            if(!commandResult.isSuccess())
                return;
        }
        /*3.移除容器*/
        rcRedisServiceImpl.remove(key);
            /*4.移除端口*/
        rcRedisServiceImpl.removeList("lxcKeys",
                key);
        rcRedisServiceImpl.removeList("lxcPorts",lxc.getContainerPort() + "");
    }


    /**
     * 或许模型
     * @param code
     * @return
     */
    private String getModelProject(String code) {
        if(global.getJavaProject().equals(code))
            return "java";
        else if(global.getHtmlProject().equals(code))
            return "apache";
        else
            return "";
    }
}
