package org.rcisoft.business.bcode.task;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.command.LxcCommand;
import org.rcisoft.core.result.CommandResult;
import org.rcisoft.core.service.RcRedisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;

/**
 * Created by lcy on 18/1/10.
 */
@Component
@Slf4j
public class StopLxcJob implements Job {

    @Autowired
    private RcRedisService rcRedisServiceImpl;

    @Autowired
    private LxcCommand lxcCommand;

    @Autowired
    private Global global;

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        /*1.容器名称*/

        String redisKeyId = jobExecutionContext.getMergedJobDataMap().getString("redisKeyId");
        String containerName = jobExecutionContext.getMergedJobDataMap().getString("containerName");
        String containerPath = jobExecutionContext.getMergedJobDataMap().getString("containerPath");
        int containerPort = jobExecutionContext.getMergedJobDataMap().getInt("containerPort");
        if(StringUtils.isEmpty(containerName) || StringUtils.isEmpty(containerPath)){
            log.info("----任务异常----");
            return;
        }
        /*非远程*/
        if(!"1".equals(global.getLxcIsRemote())){
            File filePath = new File(containerPath);
            if (!filePath.exists() || !filePath.isDirectory()) {
                log.info("----任务路径异常----");
                return;
            }
        }
        /*关闭容器*/
        /*2.docker-compose 起容器*/
        CommandResult commandResult = null;
        commandResult = lxcCommand.startOrDownLxc(containerPath,false);
        if(!commandResult.isSuccess())
            return;
        /*3.移除容器*/
        rcRedisServiceImpl.remove(redisKeyId);
        /*4.移除端口*/
        rcRedisServiceImpl.removeList("lxcKeys",
                redisKeyId);
        rcRedisServiceImpl.removeList("lxcPorts",containerPort + "");
    }
}
