package org.rcisoft.business.bcompany.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bcompany.entity.BCompany;
import org.rcisoft.business.bcompany.service.BCompanyService;
import org.rcisoft.business.bstudent.entity.BStudent;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.UnsupportedEncodingException;
import java.util.List;

/**
 * Created by Administrator on 2018/5/2.
 */
@RestController
@RequestMapping("/BCompany")
public class BCompanyController extends PaginationController<BCompany> {

    @Autowired
    private BCompanyService bCompanyService;

    @Autowired
    private Global global;

    @ApiOperation(value="添加/编辑公司用户信息", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "登录名", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "姓名", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "agencyCode", value = "所属教学单位", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "roleId", value = "角色Id", required = true, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(BCompany bCompany, SysUser user) throws UnsupportedEncodingException {
        user.setLoginName(bCompany.getCode());
        PersistModel data = bCompanyService.persist(bCompany,user);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCompany);
    }


    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCompanyService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据条件分页查询公司用户信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "登录名", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "姓名", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "agencyCode", value = "教学单位Code", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryCompanysByPagination")
    public GridModel queryCompanysByPagination(BCompany param) {
        bCompanyService.queryCompanysByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }


    @ApiOperation(value="逻辑删除", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(String businessId) {
        PersistModel data = bCompanyService.removeBCompany(businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value="根据条件查询", notes="根据条件查询公司用户信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "登录名", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "姓名", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBStudents")
    public Result queryCompanys(BCompany param) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCompanyService.queryCompanys(param));
    }


    @ApiOperation(value="停用/启用用户", notes="停用/启用用户")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/stopOrStartUser")
    public Result stopOrStartUser(String businessId) {
        PersistModel data = bCompanyService.stopOrStartUser(businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

}
