package org.rcisoft.business.bcompany.service.impl;

import org.rcisoft.business.bcompany.dao.BCompanyRepository;
import org.rcisoft.business.bcompany.entity.BCompany;
import org.rcisoft.business.bcompany.service.BCompanyService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.util.TableUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.entity.DataEntity;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.role.dao.SysRoleRepository;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.entity.UserRole;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by Administrator on 2018/5/2.
 */
@Service
public class BCompanyServiceImpl implements BCompanyService{

    @Autowired
    private BCompanyRepository bCompanyRepository;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private Global global;

    @Autowired
    private SysUserService userService;


    @Autowired
    private SysRoleRepository sysRoleRepository;

    @Override
    public PersistModel persist(BCompany model, SysUser user) {
        int line = 0;
        BeanUtils.copyProperties((DataEntity) model, (DataEntity) user);
        if (model.getBusinessId() == null) {  //添加
            if (sysUserMapper.selectOne(new SysUser(model.getCode())) != null)
                throw new ServiceException(ResultServiceEnums.COMPANY_EXISTS);

            //1. 添加s_user
            UserUtil.setCurrentPersistOperation(user);
            user.setHeadPic(global.getDEFAULT_TEACHER_LOCATION());
            userService.persistUser(user);

            //2.添加b_company
            UserUtil.setCurrentPersistOperation(model);
            line = bCompanyRepository.insertSelective(model);

        } else {  //修改
            String uid = sysUserMapper.selectUserIdByStuOrTeaId(model.getBusinessId(), "b_company");
            if (uid == null)
                throw new ServiceException(ResultServiceEnums.COMPANY_NOT_EXISTS);
            UserUtil.setCurrentMergeOperation(model);
            line = bCompanyRepository.updateByPrimaryKeySelective(model);
            userService.updateUser(user,uid,line);
            //删除原来角色
            sysUserMapper.deleteUserRole(user.getBusinessId());
        }
        //添加新的角色
        UserRole userRole = new UserRole(user.getBusinessId(),user.getRoleId());
        userRole.setCommonBusinessId();
        sysUserMapper.insertUserRole(userRole);
        return new PersistModel(line);
    }

    @Override
    public BCompany selectOne(String businessId) {
        return bCompanyRepository.selectOne(new BCompany(businessId,"0","1"));
    }

    @Override
    public List<BCompany> queryCompanysByPagination(PageUtil pageUtil, BCompany param) {
        return bCompanyRepository.queryCompanys(param);
    }

    @Override
    public PersistModel removeBCompany(String businessId) {
        /*删除关联角色*/
        sysRoleRepository.deleteUserRoleByChildrenId(TableUtil.getTableByKey("1"),businessId);
        /*删除sys_user*/
        sysUserMapper.deleteByCompany(businessId);
        /*删除b_company*/
        int line = bCompanyRepository.deleteByPrimaryKey(businessId);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    public List<BCompany> queryCompanys(BCompany param) {
        return bCompanyRepository.queryCompanys(param);
    }

    @Override
    public PersistModel stopOrStartUser(String businessId) {
        String status = "1";
        BCompany nowStatus = bCompanyRepository.selectOne(new BCompany(businessId,"0"));
        if(nowStatus.getFlag().equals("1")){
            status="0";
        }
        BCompany company = new BCompany();
        company.setBusinessId(businessId);
        company.setFlag(status);

        SysUser user = new SysUser();
        user.setLoginName(nowStatus.getCode());
        String uId = sysUserMapper.selectByLoginName(user.getLoginName());
        user.setFlag(status);
        user.setBusinessId(uId);
//        sysUserMapper.stopOrStartUser(user);
        sysUserMapper.updateByPrimaryKeySelective(user);
        int line = bCompanyRepository.updateByPrimaryKeySelective(company);
        return new PersistModel(line);
    }






}
