package org.rcisoft.business.bcoursecode.service.impl;

import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bcoursecode.dao.BCourseCodeRepository;
import org.rcisoft.business.bcoursecode.entity.BCourseCode;
import org.rcisoft.business.bcoursecode.service.BCourseCodeService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2018-4-17 11:12:17.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BCourseCodeServiceImpl implements BCourseCodeService {

    @Autowired
    private BCourseCodeRepository bCourseCodeRepository;


    /**
    * 修改 bCourseCode
    * @param bCourseCode
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BCourseCode bCourseCode){
        UserUtil.setCurrentMergeOperation(bCourseCode);
        Date date1 = null;
        Date date2 = null;
        SimpleDateFormat format1 = new SimpleDateFormat("HH:mm");
        try {
            date1 = format1.parse(bCourseCode.getStartDate());
            date2 = format1.parse(bCourseCode.getEndDate());
        } catch (ParseException e) {
            e.printStackTrace();
        }
        if (date1.getTime() > date2.getTime()){
            throw new ServiceException(ResultServiceEnums.STARTDATE_AFTER_ENDDATE);
        }
        if (date1.getTime() == date2.getTime()) {
            throw new ServiceException(ResultServiceEnums.STARTDATE_EQUAL_ENDDATE);
        }
        int line = bCourseCodeRepository.updateByPrimaryKeySelective(bCourseCode);
        log.info(UserUtil.getUserInfoProp(bCourseCode.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            bCourseCode.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 分页查询 bCourseCode
    * @param bCourseCode
    * @return
    */
    @Override
    public List<BCourseCode> findAll(BCourseCode bCourseCode){
        bCourseCode.setNotDeleted();
        return bCourseCodeRepository.queryBCourseCodes(bCourseCode);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public PersistModel changeFlag(String businessId){
        BCourseCode bCourseCode = new BCourseCode();
        UserUtil.setCurrentMergeOperation(bCourseCode);
        int id = Integer.parseInt(businessId);
//        节点启用
        if (bCourseCodeRepository.querystatus(businessId).equals("0")){
            int num = bCourseCodeRepository.select_flag_start(id);
            if(num != 0){
                throw new ServiceException(ResultServiceEnums.PERVIOUS_NOT_ENABLED);
            }
            bCourseCodeRepository.startFlag(String.valueOf(id));
        }
//        节点停用
        else{
            int num  = bCourseCodeRepository.select_flag_stop(id);
            if(num != 0){
                throw new ServiceException(ResultServiceEnums.NEXT_NOT_DISABLED);
            }
            bCourseCodeRepository.stopFlag(String.valueOf(id));
        }
        return new PersistModel(1);
    }

}
