package org.rcisoft.business.bdirection.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bdirection.entity.BDirection;
import org.rcisoft.business.bdirection.service.BDirectionService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.model.SelectModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
* Created by  on 2017-12-19 13:45:42.
*/
@RestController
@RequestMapping("/BDirection")
public class BDirectionController extends PaginationController<BDirection> {
    @Autowired
    private BDirectionService bDirectionService;

    @ApiOperation(value="添加/编辑课程方向", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "编号(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "名称(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result add(BDirection bDirection) {
        PersistModel data = bDirectionService.persist(bDirection);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bDirection);
    }

    @ApiOperation(value="删除课程方向", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result remove(String id) {
        PersistModel data = bDirectionService.removeDirection(id);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping("/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bDirectionService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据URL中的参数分页查询")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping("/queryBLessonDirectionsByPagination")
    public GridModel queryBLessonDirectionsByPagination(BDirection param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bDirectionService.queryDirectionsByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件查询", notes="根据URL中的参数查询全部")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "名称", required = false, dataType = "varchar")})
    @GetMapping("/queryBLessonDirections")
    public Result queryBLessonDirections(BDirection param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bDirectionService.queryDirections(param));
    }


    @ApiOperation(value="查询方向列表", notes="查询方向列表赋值在下拉框中")
    @GetMapping(value = "/queryDirectionsList")
    public Result queryDirectionsList() {
        List<SelectModel> directions = bDirectionService.queryDirectionsList();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                directions);
    }
}
