package org.rcisoft.business.bdirection.service.impl;


import org.rcisoft.business.bclass.entity.BClass;
import org.rcisoft.business.bdirection.dao.BDirectionRepository;
import org.rcisoft.business.bdirection.entity.BDirection;
import org.rcisoft.business.bdirection.service.BDirectionService;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.model.SelectModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by  on 2017-12-19 13:45:42.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BDirectionServiceImpl implements BDirectionService {
    @Autowired
    private BDirectionRepository bDirectionRepository;

    @Override
    public BDirection selectOne(String businessId){
        return bDirectionRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BDirection> queryDirectionsByPagination(PageUtil pageUtil, BDirection model) {
        return bDirectionRepository.queryDirections(model);
    }

    @Override
    public List<BDirection> queryDirections(BDirection model) {
        return bDirectionRepository.queryDirections(model);
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel persist(BDirection model){
        int line = 0;
        if (bDirectionRepository.queryDirectionByCode(model.getCode()) != null)
            throw new ServiceException(ResultServiceEnums.INSERT_DATA_EXIST);
        if (bDirectionRepository.queryDirectionByName(model.getName()) != null)
            throw new ServiceException(ResultServiceEnums.INSERT_DATA_EXIST);
        if(model.getBusinessId()==null) {
            UserUtil.setCurrentPersistOperation(model);
            line = bDirectionRepository.insertSelective(model);
        }else
            UserUtil.setCurrentMergeOperation(model);
        line = bDirectionRepository.updateByPrimaryKeySelective(model);
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel removeDirection(String id) {
        if(bDirectionRepository.selectslNumById(id)>0) {
            throw new ServiceException(ResultServiceEnums.DIRECTION_IS_USED);
        }
        int line = bDirectionRepository.deleteByPrimaryKey(id);
        return new PersistModel(line);
    }

    @Override
    public List<SelectModel> queryDirectionsList() {
        List<BDirection> bDirection = bDirectionRepository.queryDirectionsList();
        ArrayList<SelectModel> comBoxModel = new ArrayList<SelectModel>();
        for(BDirection c : bDirection){
            SelectModel selectModel = new SelectModel();
            selectModel.setId(c.getBusinessId());
            selectModel.setText(c.getName());
            comBoxModel.add(selectModel);
        }
        return comBoxModel;
    }
}
