package org.rcisoft.business.beduclass.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.beduclass.entity.BEduClass;
import org.rcisoft.business.beduclass.service.BEduClassService;

import java.util.List;

/**
* Created by  on 2018-4-17 14:33:36.
*/
@RestController
@RequestMapping("/bEduClass")
public class BEduClassController extends PaginationController<BEduClass> {

    @Autowired
    private BEduClassService bEduClassServiceImpl;


    @ApiOperation(value="添加", notes="添加")
    @ApiImplicitParams({@ApiImplicitParam(name = "code", value = "教学班编码 length(1~40)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "className", value = "教学班名 length(1~150)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "agencyId", value = "企业ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "班级类型", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "classesId", value = "教学班code串", required = false, dataType = "varchar")})
        @PostMapping(value = "/add")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result add(BEduClass bEduClass) {
        bEduClass.setToken(getToken());
        PersistModel data = bEduClassServiceImpl.save(bEduClass);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            bEduClass);
    }

    @ApiOperation(value="逻辑删除", notes="逻辑删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = false, dataType = "varchar")})
    @PostMapping("/delete")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result delete(String id) {
        BEduClass bEduClass = new BEduClass();
        bEduClass.setBusinessId(id);
        bEduClass.setToken(getToken());
        PersistModel data = bEduClassServiceImpl.remove(bEduClass);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "教学班编码 length(1~40)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "className", value = "教学班名 length(1~150)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "agencyId", value = "企业ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "班级类型 length(1)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "classesId", value = "教学班code串 length(1~150)", required = false, dataType = "varchar")})
    @PostMapping("/update")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result update(BEduClass bEduClass) {
        bEduClass.setToken(getToken());
        PersistModel data = bEduClassServiceImpl.merge(bEduClass);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            bEduClass);
    }

    @ApiOperation(value="查看单 ", notes="查看单 ")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = false, dataType = "varchar")})
    @GetMapping("/selectOne")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result detail(String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            bEduClassServiceImpl.findById(id));
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @ApiImplicitParams({@ApiImplicitParam(name = "rows", value = "每页显示多少行（若不传默认为前10条）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "page", value = "第几页（若不传默认为前10条）", required = false, dataType = "varchar")
    })
    @GetMapping(value = "/queryBEduClassByPagination")
    @PreAuthorize("hasRole('ROLE_1001')")
    public GridModel listByPagination(BEduClass bEduClass) {
        bEduClass.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bEduClassServiceImpl.findAllByPagination(getPaginationUtility(), bEduClass);
        return getGridModelResponse();
    }



}
