package org.rcisoft.business.bgrade.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.bgrade.entity.BGrade;
import org.rcisoft.business.bgrade.service.BGradeService;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
* Created by  on 2018-4-17 15:09:18.
*/
@RestController
@RequestMapping("/bGrade")
public class BGradeController extends PaginationController<BGrade> {

    @Autowired
    private BGradeService bGradeServiceImpl;


    @ApiOperation(value="添加", notes="添加")
    @ApiImplicitParams({@ApiImplicitParam(name = "code", value = "年级编号 length(1~50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "年级名称 length(1~150)", required = true, dataType = "varchar")})
    @PostMapping(value = "/add")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result add(@ApiIgnore BGrade bGrade) {
        bGrade.setToken(getToken());
        PersistModel data = bGradeServiceImpl.save(bGrade);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            bGrade);
    }

    @ApiOperation(value="逻辑删除", notes="逻辑删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = true, dataType = "varchar")})
    @PostMapping("/delete")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result delete(@PathVariable String id) {
        BGrade bGrade = new BGrade();
        bGrade.setBusinessId(id);
        bGrade.setToken(getToken());
        PersistModel data = bGradeServiceImpl.remove(bGrade);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            id);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId 拼在地址栏", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "年级编号 length(1~50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "年级名称 length(1~150)", required = false, dataType = "varchar")})
    @PostMapping("/update")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result update(@ApiIgnore BGrade bGrade) {
        bGrade.setToken(getToken());
        PersistModel data = bGradeServiceImpl.merge(bGrade);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            bGrade);
    }

    @ApiOperation(value="查看单 ", notes="查看单 ")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId 拼在地址栏", required = true, dataType = "varchar")})
    @GetMapping("/selectOne")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result detail(String businessId) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            bGradeServiceImpl.findById(businessId));
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @ApiImplicitParams({@ApiImplicitParam(name = "rows", value = "每页显示多少行（若不传默认为前10条）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "page", value = "第几页（若不传默认为前10条）", required = false, dataType = "varchar")
    })
    @GetMapping(value = "/queryBGradeByPagination")
    @PreAuthorize("hasRole('ROLE_1001')")
    public GridModel listByPagination(@ApiIgnore BGrade bGrade) {
        bGrade.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bGradeServiceImpl.findAllByPagination(getPaginationUtility(), bGrade);
        return getGridModelResponse();
    }


    @ApiOperation(value="查询近5年的年级", notes="查询近5年的年级")
    @GetMapping(value = "/queryFirstFiveBGrades")
    @PreAuthorize("hasRole('ROLE_1001')")
    public Result queryFirstFiveBGrades(@ApiIgnore BGrade bGrade) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bGradeServiceImpl.queryFirstFiveBGrades(bGrade));
    }
}
