package org.rcisoft.business.bgrade.service.impl;

import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bgrade.dao.BGradeRepository;
import org.rcisoft.business.bgrade.entity.BGrade;
import org.rcisoft.business.bgrade.service.BGradeService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2018-4-17 15:09:18.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BGradeServiceImpl implements BGradeService {

    @Autowired
    private BGradeRepository bGradeRepository;


    /**
    * 保存 bGrade
    * @param bGrade
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BGrade bGrade){
        bGrade.setCommonBusinessId();
        //增加操作
        UserUtil.setCurrentPersistOperation(bGrade);
        int line = bGradeRepository.insertSelective(bGrade);
        log.info(UserUtil.getUserInfoProp(bGrade.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
            bGrade.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param bGrade
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BGrade bGrade){
        UserUtil.setCurrentMergeOperation(bGrade);
        bGrade.setDeleted();
        int line = bGradeRepository.logicalDelete(bGrade);
        log.info(UserUtil.getUserInfoProp(bGrade.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        bGrade.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 bGrade
    * @param bGrade
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BGrade bGrade){
        UserUtil.setCurrentMergeOperation(bGrade);
        int line = bGradeRepository.updateByPrimaryKeySelective(bGrade);
        log.info(UserUtil.getUserInfoProp(bGrade.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            bGrade.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bGrade
    * @param id
    * @return
    */
    public BGrade findById(String id){
        return bGradeRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bGrade
    * @param bGrade
    * @return
    */
    public List<BGrade> findAllByPagination(PageUtil<BGrade> paginationUtility, BGrade bGrade){
        bGrade.setStart();
        bGrade.setNotDeleted();
        return bGradeRepository.queryBGrades(bGrade);
    }

    /**
     * 查询近5年的年级
     * @param bGrade
     * @return
     */
    @Override
    public List<BGrade> queryFirstFiveBGrades(BGrade bGrade) {
        bGrade.setStart();
        bGrade.setNotDeleted();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
        Date date = new Date();
        String nowYear = sdf.format(date);
        int code = Integer.parseInt(nowYear);

        //是否满足5个年级，不满足的话删除全部，然后插入近5年的年级
        List<BGrade> grades;
        grades = bGradeRepository.queryFirstFiveBGrades(bGrade);
        if(grades.size()<5){
            bGradeRepository.deleteAll();
            List<BGrade> newGrades = new ArrayList<>();
            for(int i=0;i<5;i++){
                BGrade grade = new BGrade();
                grade.setName(String.valueOf(code-i)+"级");
                grade.setCode(String.valueOf(code-i));
                UserUtil.setCurrentPersistOperation(grade);
                newGrades.add(grade);
            }
            bGradeRepository.insertList(newGrades);
        }else{
            //查询是否有本年的年级
//        List<BGrade> bGrades = bGradeRepository.queryBGradeByNowYear(nowYear);
            String nowYearCode =  grades.get(0).getCode();
            if(!nowYearCode.equals(nowYear)){//没有就新增
                BGrade grade = new BGrade();
                grade.setCode(nowYear);
                grade.setName(nowYear+"级");
                UserUtil.setCurrentPersistOperation(grade);
                bGradeRepository.insertSelective(grade);
            }
        }


        grades = bGradeRepository.queryFirstFiveBGrades(bGrade);
        return grades;
    }

}
