package org.rcisoft.business.bimage.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bimage.entity.BImage;
import org.rcisoft.business.bimage.service.BImageService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

/**
* Created by gwf on 2017-8-10 11:05:34.
*/
@RestController
@RequestMapping("/BImage")
public class BImageController extends PaginationController<BImage> {
    @Autowired
    private BImageService bImageService;

    @ApiOperation(value="添加/编辑图片", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageName", value = "图片标题(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "imageCode", value = "图片code(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/add")
    public Result add(BImage bImage) {
        PersistModel data = bImageService.persist(bImage,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bImage);
    }

    @ApiOperation(value="删除图片", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bImageService.removeBImage(id,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bImageService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据URL中的参数分页查询")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageName", value = "图片标题", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageCode", value = "图片code", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBImagesByPagination")
    public GridModel queryBImagesByPagination(BImage param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bImageService.queryBImagesByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件查询", notes="根据URL中的参数查询全部")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageName", value = "图片标题", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "imageCode", value = "图片code", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBImages")
    public Result queryBImages(BImage param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bImageService.queryBImages(param));
    }
}
