package org.rcisoft.business.bimage.service.impl;

import org.rcisoft.business.bimage.dao.BImageRepository;
import org.rcisoft.business.bimage.entity.BImage;
import org.rcisoft.business.bimage.service.BImageService;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.DelStatus;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by gwf on 2017-8-10 11:05:33.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BImageServiceImpl implements BImageService {
    @Autowired
    private BImageRepository bImageRepository;

    @Override
    public BImage selectOne(String businessId){
        return bImageRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BImage> queryBImagesByPagination(PageUtil pageUtil, BImage model) {
        Map param = new HashMap<String,Object>();
        return bImageRepository.queryBImages(param);
    }

    @Override
    public List<BImage> queryBImages(BImage model) {
        Map param = new HashMap<String,Object>();
        return bImageRepository.queryBImages(param);
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel persist(BImage model,String token){
        int line = 0;
        if(StringUtils.isEmpty(model.getBusinessId())){
            UserUtil.setCurrentMergeOperation(model);
            line = bImageRepository.updateByPrimaryKeySelective(model);
        }else {
            if(bImageRepository.selectOne(new BImage(model.getImageCode()))!=null)
                throw new ServiceException(ResultServiceEnums.INSERT_DATA_EXIST);
            UserUtil.setCurrentPersistOperation(model);
            line = bImageRepository.insertSelective(model);
        }

        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel removeBImage(String id, String token) {
        BImage bImage = bImageRepository.selectByPrimaryKey(id);
        bImage.setDelFlag(DelStatus.DELETED.getStatus());
        UserUtil.setCurrentMergeOperation(bImage);
        int line = bImageRepository.updateByPrimaryKeySelective(bImage);
        return new PersistModel(line);
    }
}
