package org.rcisoft.business.blesson.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.bdirection.dao.BDirectionRepository;
import org.rcisoft.business.bdirection.entity.BDirection;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.entity.BLessonDirection;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.TestChinese;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BLessonServiceImpl implements BLessonService {
    @Autowired
    private BLessonRepository bLessonRepository;
    @Autowired
    private BSlRepository bSlRepository;
    @Autowired
    private Global global;
    @Autowired
    private BDirectionRepository bDirectionRepository;


    @Override
    public BLesson selectOne(String businessId){
        return bLessonRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public BLesson selectOne(BLesson bLesson) {
        return bLessonRepository.selectOne(bLesson);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public int  updateByPrimaryKeySelective(BLesson bLesson) {
        return bLessonRepository.updateByPrimaryKeySelective(bLesson);
    }

    @Override
    public List<BLesson> queryBLessonsByPagination(PageUtil pageUtil, BLesson model) {
        return bLessonRepository.queryBLessons(model);
    }

    @Override
    public List<BLesson> queryBLessons(BLesson model) {
        return bLessonRepository.queryBLessons(model);
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel persist(BLesson model,String token){
        int line = 0;
        BLessonDirection direction = new BLessonDirection();
        direction.setDirectionId(model.getDirectionId());
        if(model.getBusinessId()==null) {
            if (bLessonRepository.queryBLessonByCode(model.getCode()) != null)
                throw new ServiceException(ResultServiceEnums.INSERT_DATA_EXIST);
            model.setDefaultUrl(global.getDEFAULT_COURSE_LOCATION());
            UserUtil.setCurrentPersistOperation(model);
            line = bLessonRepository.insertSelective(model);
            //添加课程方向信息
            direction.setBusinessId(IdGen.uuid());
            direction.setLessionId(model.getBusinessId());
            bLessonRepository.insertLessonDirection(direction);
        }else {
            //查询当前已添加学时
            int classHour = bLessonRepository.queryClassHourByLessonId(model.getBusinessId());
            //如果小于当前已添加的学时  抛异常
            if(model.getClassHour()!=null&&Integer.parseInt(model.getClassHour())<classHour){
                throw new ServiceException(ResultServiceEnums.CLASSHOUR_ERROR);
            }
            UserUtil.setCurrentMergeOperation(model);
            line = bLessonRepository.updateByPrimaryKeySelective(model);
            direction.setLessionId(model.getBusinessId());
            bLessonRepository.updateDirectionByLessonId(direction);
        }
        return new PersistModel(line);
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel importExcel(HSSFWorkbook hwb, String token) throws IOException {
        ArrayList<BLesson> lessons = new ArrayList<BLesson>();
        ArrayList<String> repeatCode = new ArrayList<String>();
        List<String> errorCode = new ArrayList<>();
        ArrayList<String> direction = new ArrayList<String>();
        List<String> isChinese = new ArrayList<>();
        List<String> isNegative = new ArrayList<>();
        List<String> currentlyCode = new ArrayList<>();
        String innerRepeatResult = "";
        String[] headers = {"课程编号","课程名称","课程方向编号","学时","学分"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb,headers,false,1);  //获取excel数据


        if (values.size()<1){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        for(String[] value:values){   //将数据封装到entity
            BLesson bLesson = null;
            String classCode = value[0];

            //判断编号是否含有中文
            if(TestChinese.isChinese(classCode)){
                isChinese.add(value[0]);
                continue;
            };

            //判断1:班级编号是否满足<15位；
            if(classCode.length()>15){
                //该用户不满足要求，记入valuesShort中
                errorCode.add(value[0]);
                continue;
            }

            if(Integer.parseInt(value[4])<0||Integer.parseInt(value[3])<0){
                //该用户不满足要求，记入valuesShort中
                isNegative.add(value[0]);
                continue;
            }
            if((bLesson=bLessonRepository.queryBLessonByCode(value[0]))!=null) {
                repeatCode.add(value[0]);
                continue;
            }
            BDirection b_direction =  bDirectionRepository.selectOne(new BDirection(value[2]));
            if (b_direction == null) {
                if (!direction.contains(value[2]))
                    direction.add(value[2]);
                continue;
            }
            bLesson= new BLesson(value[0],value[1],value[3],value[4]);
            bLesson.setDefaultUrl(global.getDEFAULT_COURSE_LOCATION());
            UserUtil.setCurrentPersistOperation(bLesson);
            //表中去重
            if(currentlyCode.contains(bLesson.getCode())){
                innerRepeatResult += "编号 " + value[1] + " 在Excel中重复出现，除第一条以外的数据插入失败 \n";
                continue;
            }
            if(!lessons.contains(bLesson)){
                currentlyCode.add(bLesson.getCode());
                lessons.add(bLesson);

                BLessonDirection bDirection = new BLessonDirection();
                bDirection.setDirectionId(b_direction.getBusinessId());
                bDirection.setBusinessId(IdGen.uuid());
                bDirection.setLessionId(bLesson.getBusinessId());
                bDirectionRepository.insertBLessonDirection(bDirection);
            }
        }

        String result ="";
        if(lessons.size()>0){
            int line = bLessonRepository.insertList(lessons);

            if (line>0) {
                result+="成功导入"+lessons.size()+"门课程。";
            }else{
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);

            }
        }


        if(isNegative.size()>0){
            result+="以下课程的学时或学分不能为负数："+ JSON.toJSONString(isNegative)+"。";
        }
        if(isChinese.size()>0){
            result+="以下编号格式不正确："+ JSON.toJSONString(isChinese)+"。";
        }
        if(repeatCode.size()>0){
            result+="以下课程编号存在重复："+ JSON.toJSONString(repeatCode)+"。";
        }
        if(errorCode.size()>0){
            result+="以下课程编号超过15位："+ JSON.toJSONString(errorCode)+"。";
        }
        if(direction.size()>0){
            result+="以下课程方向编号不存在："+ JSON.toJSONString(direction)+"。";
        }
        if (lessons.size()<1){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST.getCode(),result);
        }

        return new PersistModel(1,result + innerRepeatResult);


    }
    public PersistModel importPic(HSSFWorkbook hwb) throws IOException
    {
        return null;
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel removeBLesson(String id) {
        if(bSlRepository.selectslNumByLesson(id)>0) {
            throw new ServiceException(ResultServiceEnums.LESSON_HAS_SL);
        }
        BLesson bLesson = new BLesson();
        bLesson.setBusinessId(id);
        bLesson.setDeleted();
        UserUtil.setCurrentMergeOperation(bLesson);
        int line = bLessonRepository.logicalDelete(bLesson);
        //int line = bLessonRepository.deleteByPrimaryKey(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);

    }

    private Map<String,Object> queryParamHandler(BLesson model){
        Map param = new HashMap<String, Object>();
        if(model.getLessonName()!=null)
            param.put("name","%"+model.getLessonName()+"%");
        else
            param.put("name","%%");
        if(model.getCode()!=null)
            param.put("code","%"+model.getCode()+"%");
        else
            param.put("code","%%");
        return param;
    }


}
