package org.rcisoft.business.bnews.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bnews.entity.BNews;
import org.rcisoft.business.bnews.service.BNewsService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
* Created by gwf on 2017-8-10 11:05:34.
*/
@RestController
@RequestMapping("/BNews")
public class BNewsController extends PaginationController<BNews> {
    @Autowired
    private BNewsService bNewsService;

    @ApiOperation(value="添加/编辑新闻", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "newsName", value = "新闻标题（1-50）", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "newsCode", value = "新闻code（1-50）", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "releaseTime", value = "发布时间", required = false, dataType = "datetime"),
            @ApiImplicitParam(name = "attribute", value = "0：非头条，1：头条", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "isRelease", value = "0未发布，1：发布", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "newsContent", value = "新闻内容", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/add")
    public Result add(@Valid BNews bNews, BindingResult bindingResult) {
        PersistModel data = bNewsService.persist(bNews,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bNews);
    }

    @ApiOperation(value="删除新闻", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bNewsService.removeBNews(id,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/oneWithoutAuth")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bNewsService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询新闻", notes="根据URL中的参数分页查询新闻")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "newsName", value = "新闻标题", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "newsCode", value = "新闻code", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "releaseTime", value = "发布时间", required = false, dataType = "datetime"),
            @ApiImplicitParam(name = "attribute", value = "0：非头条，1：头条", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isRelease", value = "0未发布，1：发布", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "newsContent", value = "新闻内容", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "delFlag", value = "删除标记（0：正常；1：删除；2：审核）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "启用标记（0：停用；1：启用）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBNewssByPaginationWithoutAuth")
    public GridModel queryBNewssByPagination(BNews param) {
        bNewsService.queryBNewssByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件查询新闻", notes="根据URL中的参数查询全部新闻")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "newsName", value = "新闻标题", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "newsCode", value = "新闻code", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "releaseTime", value = "发布时间", required = false, dataType = "datetime"),
            @ApiImplicitParam(name = "attribute", value = "0：非头条，1：头条", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isRelease", value = "0未发布，1：发布", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "newsContent", value = "新闻内容", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "delFlag", value = "删除标记（0：正常；1：删除；2：审核）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "启用标记（0：停用；1：启用）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBNewss")
    public Result queryBNewss(BNews param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bNewsService.queryBNewss(param));
    }
}
