package org.rcisoft.business.bnews.service.impl;

import org.rcisoft.business.bnews.dao.BNewsRepository;
import org.rcisoft.business.bnews.entity.BNews;
import org.rcisoft.business.bnews.enums.ReleaseEnum;
import org.rcisoft.business.bnews.service.BNewsService;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.DelStatus;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by gwf on 2017-8-10 11:05:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BNewsServiceImpl implements BNewsService {
    @Autowired
    private BNewsRepository bNewsRepository;

    @Override
    public BNews selectOne(String businessId){
        BNews bNews = bNewsRepository.selectByPrimaryKey(businessId);
        if(bNews==null)
            throw new ServiceException(ResultServiceEnums.NEWS_NOT_EXISTS);
        return bNews;
    }

    @Override
    public List<BNews> queryBNewssByPagination(PageUtil pageUtil, BNews model) {
        List<BNews> list = bNewsRepository.queryBNewss(getParames(model));
        return list;
    }

    @Override
    public List<BNews> queryBNewss(BNews model) {
        return bNewsRepository.queryBNewss(getParames(model));
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel persist(BNews model,String token){
        int line = 0;
        if(model.getBusinessId()!=null&&model.getBusinessId()!=""&&!model.getBusinessId().equals("")){
            if(!StringUtils.isEmpty(model.getIsRelease())&&model.getIsRelease().equals(ReleaseEnum.RELEASE.getCode())) //如果修改发布状态为发布，
            model.setReleaseTime(new Date());   // 则添加发布时间
            UserUtil.setCurrentMergeOperation(model);
            line = bNewsRepository.updateByPrimaryKeySelective(model);
        }
        else {
            model.autoSetApplyCode();
            UserUtil.setCurrentPersistOperation(model);
            line = bNewsRepository.insertSelective(model);
        }
        if(line!=1)
            throw new ServiceException(ResultServiceEnums.UPDATE_ERROR);
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel removeBNews(String id, String token) {
        BNews bNews = bNewsRepository.selectByPrimaryKey(id);
        if(bNews==null)
            throw new ServiceException(ResultServiceEnums.NEWS_NOT_EXISTS);
        bNews.setDelFlag(DelStatus.DELETED.getStatus());
        UserUtil.setCurrentMergeOperation(bNews);
        int line = bNewsRepository.updateByPrimaryKeySelective(bNews);
        return new PersistModel(line);
    }

    private Map<String,Object> getParames(BNews model){
        Map param = new HashMap<String,Object>();
        param.put("release", StringUtils.isEmpty(model.getIsRelease())?null:model.getIsRelease());
        param.put("attribute", StringUtils.isEmpty(model.getAttribute())?null:model.getAttribute());
        param.put("newsName","%"+(model.getNewsName()==null?"":model.getNewsName())+"%");
        return param;
    }
}
