package org.rcisoft.business.bopinion.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bopinion.entity.BOpinion;
import org.rcisoft.business.bopinion.service.BOpinionService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by Administrator on 2017/12/21.
 */
@RestController
@RequestMapping("/BOpinion")
public class BOpinionController extends PaginationController<BOpinion> {

    @Autowired
    private BOpinionService bOpinionService;

    @ApiOperation(value="查询历史审批意见", notes="根据开课ID查询历史审批意见")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "opinion", value = "意见(1-200)", required = false, dataType = "varchar"),})

    @GetMapping(value = "/queryOpinionBySlId")
    private Result queryOpinionBySlId(String slId){
        List<BOpinion> data = bOpinionService.queryOpinionBySlId(slId);
        return Result.builder(new PersistModel(1),
                            MessageConstant.MESSAGE_ALERT_SUCCESS,
                            MessageConstant.MESSAGE_ALERT_ERROR,
                            data);
    }

    @ApiOperation(value="审核通过", notes="根据排课ID通过审核")
    @ApiImplicitParams({@ApiImplicitParam(name = "subAgencyId", value = "子任务中对应的教学单位ID(1-50)", required = true, dataType = "varchar")})
    @PutMapping(value = "/auditPass")
    public Result auditPass(@RequestParam String subAgencyId){
        PersistModel data = bOpinionService.auditPass(subAgencyId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data);
    }

    @ApiOperation(value="审核驳回", notes="审核驳回，填写审核意见")
    @ApiImplicitParams({@ApiImplicitParam(name = "subAgencyId", value = "子任务ID(1-50)", required = true, dataType = "varchar"),
                        @ApiImplicitParam(name = "opinion", value = "审核意见(1-200)", required = true, dataType = "varchar")})
    @PostMapping("/auditFail")
    public Result auditFail(String opinion,String subAgencyId){
        PersistModel data = bOpinionService.save(opinion,subAgencyId);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data);
    }

}
