package org.rcisoft.business.bopinion.service.impl;

import org.rcisoft.business.barrange.dao.BArrangeRepository;
import org.rcisoft.business.barrange.dao.BSubAgencyRepository;
import org.rcisoft.business.barrange.dao.BSubtaskRepository;
import org.rcisoft.business.barrange.entity.BArrange;
import org.rcisoft.business.barrange.entity.BSubtask;
import org.rcisoft.business.bopinion.dao.BOpinionRepository;
import org.rcisoft.business.bopinion.entity.BOpinion;
import org.rcisoft.business.bopinion.service.BOpinionService;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * Created by Administrator on 2017/12/21.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BOpinionServiceImpl implements BOpinionService{

    @Autowired
    private BOpinionRepository bOpinionRepository;

    @Autowired
    private BSubAgencyRepository bSubAgencyRepository;

    @Autowired
    private BSubtaskRepository bSubtaskRepository;

    @Autowired
    private BArrangeRepository bArrangeRepository;

    @Override
    public List<BOpinion> queryOpinionBySlId(String slId) {
        return bOpinionRepository.queryOpinionBySlId(slId);
    }

    /**
     * 审核通过
     * @param subAgencyId
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel auditPass(String subAgencyId) {
        //修改subAgency状态为通过
        int line = bOpinionRepository.changeSubtaskStatus(subAgencyId);

        //获得子任务subTaskId
        String subTaskId = bSubAgencyRepository.querySubTaskIdBySubAgencyId(subAgencyId);


        //根据subAgencyId查询子任务下每个教学单位的审核状态
//        List<String> subAgencyStatus = bSubAgencyRepository.querySubAgencyStatusBySubAgencyId(subAgencyId);
        List<String> subAgencyStatus = bSubAgencyRepository.querySubAgencyStatusBySubAgencyId(subTaskId);
        if(subAgencyStatus.size()==1&&subAgencyStatus.get(0).equals("3")){//审核全部通过，将子任务状态置为通过
            BSubtask bSubtask = new BSubtask(subTaskId,"3");
            bSubtaskRepository.updateByPrimaryKeySelective(bSubtask);
            //查询该排课任务下所有子任务的状态
            String arrangeId = bSubtaskRepository.queryArrangeIdById(subTaskId);
            List<String> subTaskStatus = bSubtaskRepository.querySubTaskStatusById(arrangeId);
            boolean finish = true;
            for(String status : subTaskStatus){//子任务状态：0不能开课 3可以开课 5已开课
                if(status.equals("0")){//表示还有未通过的子任务，不能将排课任务完成
                    finish = false;
                }
            }
            if(finish){//审核全部通过，将排课任务状态置为完成
                BArrange bArrange = new BArrange(arrangeId,"3");
                bArrangeRepository.updateByPrimaryKeySelective(bArrange);
            }
        }
        return new PersistModel(line);
    }

    /**
     * 审核驳回
     * @param opinion
     * @param id
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(String opinion,String id) {
        BOpinion bOpinion = new BOpinion();
        bOpinion.setOpinion(opinion);
        bOpinion.setSubAgencyId(id);
        UserUtil.setCurrentPersistOperation(bOpinion);
        //插入意见表b_opinion
        int line = bOpinionRepository.insertSelective(bOpinion);
        //更新子任务状态b_sub_agency
        bSubAgencyRepository.changeStatus(id);
        return new PersistModel(line);
    }

}
