package org.rcisoft.business.brooms.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.rcisoft.business.brooms.utils.Upload2DataBase;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.FileController;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.brooms.entity.BRooms;
import org.rcisoft.business.brooms.service.BRoomsService;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
* Created by  on 2018-4-23 9:26:21.
*/
@RestController
@RequestMapping("brooms")
public class BRoomsController extends FileController<BRooms> {

    @Autowired
    private BRoomsService bRoomsServiceImpl;

    @Autowired
    private Global global;

    @ApiOperation(value="添加", notes="添加")
    @ApiImplicitParams({@ApiImplicitParam(name = "campus", value = "校区(1)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "building", value = "教学楼(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "category", value = "教室类型(1)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "classroomNo", value = "教室号(1-10)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "classroomName", value = "教室名(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "classSeat", value = "座位数(1-5)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "examSeat", value = "考试座位数(1-5)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "plan", value = "计划(1-50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "教室编号(1-50)", required = true, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add( @ApiIgnore @Valid BRooms bRooms, BindingResult bindingResult) {
        bRooms.setToken(getToken());
        PersistModel data = bRoomsServiceImpl.save(bRooms);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        bRooms);
    }

    @ApiOperation(value="物理删除", notes="物理删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{id:\\w+}")
    public Result delete(@PathVariable String id) {
        BRooms bRooms = new BRooms();
        bRooms.setBusinessId(id);
        bRooms.setToken(getToken());
        PersistModel data = bRoomsServiceImpl.remove(bRooms);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "campus", value = "校区(1)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "building", value = "教学楼(1-50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "category", value = "教室类型(1)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "classroomNo", value = "教室号(1-50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "classroomName", value = "教室名(1-50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "classSeat", value = "座位数(1-5)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "examSeat", value = "考试座位数(1-5)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "plan", value = "计划(1-50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "教室编号(1-50)", required = false, dataType = "varchar")})
    @PutMapping("/update")
    public Result update(@ApiIgnore BRooms bRooms, BindingResult bindingResult) {
        bRooms.setToken(getToken());
        PersistModel data = bRoomsServiceImpl.merge(bRooms);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        bRooms);
    }

    @ApiOperation(value="查看单 ", notes="查看单 ")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = true, dataType = "varchar")})
    @GetMapping("/detail/{id:\\w+}")
    public Result detail(@PathVariable String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        bRoomsServiceImpl.findById(id));
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "condition", value = "condition", required = false, dataType = "varchar")
    })
    @GetMapping(value = "/queryBRoomsByPagination")
    public GridModel listByPagination(BRooms bRooms) {
        bRooms.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        List<BRooms> list = bRoomsServiceImpl.findAllByPagination(getPaginationUtility(), bRooms);
        return getGridModelResponse();
    }

    @ApiOperation(value="Excel上传", notes="Excel上传")
    @PostMapping(value = "/uploadExcel")
    public Result uploadExcel(@RequestParam(value="uploadFile",required=true) MultipartFile uploadFile) throws IOException {
        PersistModel data = bRoomsServiceImpl.uploadExcel(MultipartFile2HSSFWorkbookConverter.convert(uploadFile));
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                data.getInfluenceReason(),
                null);
    }
}
