package org.rcisoft.business.brooms.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.barrange.dao.BArrangeRepository;
import org.rcisoft.business.brooms.dao.BRoomsRepository;
import org.rcisoft.business.brooms.entity.BRooms;
import org.rcisoft.business.brooms.service.BRoomsService;
import org.rcisoft.business.bslschedule.dao.BSlScheduleRepository;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by  on 2018-4-23 9:26:21.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BRoomsServiceImpl implements BRoomsService {

    @Autowired
    private BRoomsRepository bRoomsRepository;

    @Autowired
    private BSlScheduleRepository bSlScheduleRepository;

    @Autowired
    private BArrangeRepository bArrangeRepository;

    /**
    * 保存 bRooms
    * @param bRooms
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BRooms bRooms){
        //判断是否存在重复编码
        if(bRoomsRepository.avoidRepeatCode(bRooms.getCode()) != 0){
            throw new ServiceException(ResultServiceEnums.EXISE_ROOM_CODE);
        }else if(bRoomsRepository.avoidRepeatCode2(bRooms.getClassroomName()) != 0){
            throw new ServiceException(ResultServiceEnums.EXISE_ROOM_NAME);
        }else{
            bRooms.setCommonBusinessId();
            //增加操作
            UserUtil.setCurrentPersistOperation(bRooms);
            int line = bRoomsRepository.insertSelective(bRooms);
            log.info(UserUtil.getUserInfoProp(bRooms.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
                    bRooms.getBusinessId()+"的信息");
            return new PersistModel(line);
        }
    }

    /**
    * 物理删除
    * @param bRooms
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BRooms bRooms){
        if(bArrangeRepository.countThisRoomNum(bRooms.getBusinessId()) != 0
                || bSlScheduleRepository.countThisRoomNum(bRooms.getBusinessId()) != 0){
            throw new ServiceException(ResultServiceEnums.DATA_HAS_USED);
        }else{
            int line = bRoomsRepository.deleteByPrimaryKey(bRooms.getBusinessId());
            log.info(UserUtil.getUserInfoProp(bRooms.getToken(),UserUtil.USER_USERNAME)+"物理删除了ID为"+
                    bRooms.getBusinessId()+"的信息");
            return new PersistModel(line);
        }
    }

    /**
    * 修改 bRooms
    * @param bRooms
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BRooms bRooms){
//        if((bRooms.getCode() != null && !bRooms.getCode().equals("")) && bRooms.getCode().length() > 50){
//            throw new ServiceException(ResultServiceEnums.ROOM_CODE_TOO_LONG);
//        }
//        if((bRooms.getClassroomNo() != null && !bRooms.getClassroomNo().equals("")) && bRooms.getClassroomNo().length() > 5){
//            throw new ServiceException(ResultServiceEnums.ROOM_BUILDINGNO_TOO_LONG);
//        }
//        if((bRooms.getClassSeat() != null && bRooms.getClassSeat() > 10000)){
//            throw new ServiceException(ResultServiceEnums.ROOM_SEAT_TOO_MANY);
//        }
        UserUtil.setCurrentMergeOperation(bRooms);
        int line = bRoomsRepository.updateByPrimaryKeySelective(bRooms);
        log.info(UserUtil.getUserInfoProp(bRooms.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            bRooms.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bRooms
    * @param id
    * @return
    */
    public BRooms findById(String id){
        return bRoomsRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bRooms
    * @param bRooms
    * @return
    */
    public List<BRooms> findAllByPagination(PageUtil<BRooms> paginationUtility,
        BRooms bRooms){
        bRooms.setStart();
        bRooms.setNotDeleted();
        return bRoomsRepository.queryBRoomss(bRooms);
    }

    @Override
    public PersistModel uploadExcel(HSSFWorkbook hwb) {
        List<BRooms> list = new ArrayList<>();
        List<String> allCode = bRoomsRepository.queryAllCode();
        List<String> currentlyCode = new ArrayList<>();
        String[] headers = {"校区", "教室编号", "教学楼","教室","教室类型","座位数"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb, headers,true,1);  //获取excel数据
        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }
        String result = "";

        for(String[] value : values){
            int index = values.indexOf(value) + 1;
            if(value[0] == null || value[1] == null || value[2] == null || value[3] == null || value[4] == null || value[5] == null
                    || value[0].equals("") || value[1].equals("") || value[2].equals("") || value[3].equals("") || value[4].equals("") || value[5].equals("")){
                result += "表中第 " + index + " 行存在空数据 \n";
                continue;
            }

            if(allCode.contains(value[1])){
                result += "表中第 " + index  + " 行，编号 " + value[1] + " 已经存在于数据库 \n";
                continue;
            }
            if(currentlyCode.contains(value[1])){
                result += "表中第 " + index + " 行，编号 " + value[1] + " 在表中重复 \n";
                continue;
            }
            if(value[1].length() > 10){
                result += "表中第 " + index + " 行，编号 " + value[1] + " 太长 \n";
                continue;
            }
            if(value[3].length() > 15){
                result += "表中第 " + index + " 行，编号 " + value[3] + " 太长 \n";
                continue;
            }
            //Int最大值为2147483648，所以不能超过9位数
            if(value[5].length() >= 9){
                result += "表中第 " + index + " 行，编号 " + value[1] + " 的数据座位数不合法 \n";
                continue;
            }
            try{
                int seat = Integer.valueOf(value[5]);
            }catch (Exception e){
                result += "表中第 " + index + " 行，编号 " + value[1] + " 的数据座位数不合法 \n";
                continue;
            }
            BRooms bRooms = new BRooms();
            String classroomName = "";
            if(value[0].equals("校本部")){
                bRooms.setCampus("0");
                classroomName += "校本部";
            }else if(value[0].equals("新校区")){
                bRooms.setCampus("1");
                classroomName += "新校区";
            }

            bRooms.setCode(value[1]);
            if(value[2].equals("A区")){
                bRooms.setBuilding("0");
                classroomName += "A区";
            }else if(value[2].equals("B区")){
                bRooms.setBuilding("1");
                classroomName += "B区";
            }else if(value[2].equals("C区")){
                bRooms.setBuilding("2");
                classroomName += "C区";
            }else if(value[2].equals("D区")){
                bRooms.setBuilding("3");
                classroomName += "D区";
            }
            bRooms.setClassroomNo(value[3]);
            classroomName += value[3];
            switch (value[4]) {
                case "普通教室":
                    bRooms.setCategory("0");
                    break;
                case "听力教室":
                    bRooms.setCategory("1");
                    break;
                case "多媒体教室":
                    bRooms.setCategory("2");
                    break;
                case "实训机房":
                    bRooms.setCategory("3");
                    break;
                case "报告厅":
                    bRooms.setCategory("4");
                    break;
                case "多功能报告厅":
                    bRooms.setCategory("5");
                    break;
                case "阶梯教室":
                    bRooms.setCategory("6");
                    break;
            }
            bRooms.setClassSeat(Integer.valueOf(value[5]));
            bRooms.setClassroomName(classroomName);
            UserUtil.setCurrentPersistOperation(bRooms);
            currentlyCode.add(value[1]);
            list.add(bRooms);
        }
        int flag = 1;
        if(!result.equals("")){
            flag = 0;
        }
        if(list.size() != 0){
            int i = bRoomsRepository.addExcel(list);
            result += "成功插入了 " + i + "条数据 \n";
        }else {
            result += "没有输入被插入 \n";
            flag = 0;
        }

        return new PersistModel(flag, result);
    }

}
