package org.rcisoft.business.brslstudent.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.brslstudent.entity.BRSlStudent;
import org.rcisoft.business.brslstudent.service.BRSlStudentService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
* Created by CodeGenerator on 2017/07/12.
*/
@RestController
@RequestMapping("/BRSlStudent")
public class BRSlStudentController extends PaginationController<BRSlStudent> {
    @Autowired
    private BRSlStudentService bRSlStudentService;

    @ApiOperation(value="添加/编辑学生课程中间表信息", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId(1-50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "课程编号(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学生编号(1-50)", required = true, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/add")
    public Result add(BRSlStudent bRSlStudent) {
        PersistModel data = bRSlStudentService.persist(bRSlStudent);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bRSlStudent);
    }

    @ApiOperation(value="删除学生课程信息", notes="根课程级编号和学生编号删除一条记录")
    @ApiImplicitParams({@ApiImplicitParam(name = "slCode", value = "课程编号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学生编号", required = true, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(@RequestParam("studentCode") String studentCode,
                               @RequestParam("slCode") String slCode) {
        PersistModel data = bRSlStudentService.removeBRSlStudents(studentCode,slCode);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                studentCode);
    }

    @ApiOperation(value="excel导入", notes="excel导入")
    @ApiImplicitParam(name = "importFile", value = "excel文件", required = true, dataType = "MultipartFile")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "excelImport")
    public Result excelImport(MultipartFile importFile) throws Exception {
        PersistModel data = bRSlStudentService.importExcel(MultipartFile2HSSFWorkbookConverter.convert(importFile));
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data.getInfluenceReason());
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bRSlStudentService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据条件分页查询")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学生编号", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBRSlStudentsByPagination")
    public GridModel queryBRSlStudentsByPagination(BRSlStudent param) {
        bRSlStudentService.queryBRSlStudentsByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件查询", notes="根据条件查询全部")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学生编号", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBRSlStudents")
    public Result queryBRSlStudents(BRSlStudent param) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bRSlStudentService.queryBRSlStudents(param));
    }
}
