package org.rcisoft.business.bsl.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bsl.dto.SlDTO;
import org.rcisoft.business.bsl.dto.SlDetailDTO;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;
import java.util.List;
import java.util.Map;

/**
* Created with CodeGenerator on 2017/07/12.
*/
@Repository
public interface BSlRepository extends BaseMapper<BSl> {

    /**
    * 根据条件查找
    * @param param
    * @return
    */
    @Select("<script>SELECT sl.*, bd.name as directionName,lesson.lesson_name as lessonName," +
            "u.name as teacherName,bt.name as termName " +
            "FROM b_sl sl\n" +
            "LEFT JOIN b_lesson lesson ON sl.lesson_code=lesson.code \n" +
            "LEFT JOIN s_user u ON sl.teacher_code=u.login_name \n" +
            "left join b_lesson_direction bld on bld.sl_id = sl.business_id " +
            "left join b_direction bd on bd.business_id = bld.direction_id " +
            "left join b_term bt on bt.code = sl.term_code " +
            " <where>" +
            " sl.del_flag = '0' and sl.flag = '1' "+
            //" <if test=\"isOnline !=NULL and isOnline !=''\">AND sl.is_online like CONCAT('%',#{isOnline},'%')</if>" +
            " <if test=\"teacherName !=null and teacherName !=''\">AND u.name like CONCAT('%',#{teacherName},'%') </if>" +
            " <if test=\"lessonName !=null and lessonName !=''\">AND lesson.lesson_name like CONCAT('%',#{lessonName},'%') </if>" +
            " <if test=\"termCode !=null and termCode !=''\">AND sl.term_code = #{termCode} </if>" +
            " <if test=\"auditStatus !=null and auditStatus !=''\">AND sl.audit_status = #{auditStatus} </if>" +
            " </where> " +
            "GROUP BY sl.business_id</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BSl> queryBSls(Map<String, Object> param);

    /**
     * 获取精品课程
     * @return
     */
    @Select("<script>SELECT sl.business_id,sl.code,sl.lesson_code,lesson.lesson_name as lessonName,sl.teacher_code,u.name as teacherName,sl.sl_cover_url,sl.sl_notes,sl.is_recommend,sl.recommend_date,sl.recommend_order,sl.recommend_course, \n" +
            "(select COUNT(brs.student_code) from b_r_sl_student brs where brs.sl_code = sl.code) as studyNum FROM b_sl sl \n"+
            "LEFT JOIN b_lesson lesson ON sl.lesson_code=lesson.code \n" +
            "LEFT JOIN s_user u ON sl.teacher_code=u.login_name \n" +
            " <where>" +
            "sl.is_recommend = '1' and sl.audit_status != '4' and sl.del_flag = '0' and sl.flag = '1'"+
            " </where>" +
            " order by sl.recommend_order asc </script> ")
    @ResultMap(value = "BaseResultMap" )
    List<BSl> queryBSlsWithRecommend();

    /**
     * 批量插入
     * @param bSls
     * @return
     */
    @Insert("<script>INSERT INTO b_sl" +
            "(create_date,update_date,del_flag,flag,business_id,code,lesson_code,teacher_code,term_code,sl_cover_url,git_lab_project_id,git_lab_project_name,class_hour,credits)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "( #{item.createDate},#{item.updateDate},#{item.delFlag},#{item.flag},#{item.businessId},#{item.code},#{item.lessonCode},#{item.teacherCode},#{item.termCode},#{item.slCoverUrl},#{item.gitLabProjectId},#{item.gitLabProjectName},#{item.classHour},#{item.credits})" +
            "</foreach></script>")
    int insertList(List<BSl> bSls);

    /**
     * 更新状态
     * @param lessonCode
     * @param online
     * @return
     */
    /*@Update("<script>update b_sl sl set sl.is_online=#{online} where sl.lessonCode=#{lessonCode}</script>")
    int updateBSls(String lessonCode, String online);*/

    /**
     * 根据学号获取学习课程
     * @param studentCode
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tt2.`code` as code,\n" +
            "\tt2.sl_cover_url as slCoverUrl,\n" +
            "\tt1.is_study as isStudy,\n" +
            "\tt2.business_id as businessId,\n" +
            "\tt3.lesson_name as lessonName\n" +
            "FROM\n" +
            "\tb_r_sl_student t1\n" +
            "LEFT JOIN b_sl t2 ON t1.sl_code = t2.`code`\n" +
            "LEFT JOIN b_lesson t3 ON t2.lesson_code = t3.`code`\n" +
            "WHERE\n" +
            "\tt1.student_code = #{studentCode}\n" +
            "AND t2.del_flag = 0\n" +
            "AND t2.flag = 1\n" +
            "AND t3.del_flag = 0\n" +
            "AND t3.flag = 1\n" +
            "AND t2.audit_status = '2' "+
            "<if test=\"lessonName !=null and lessonName !=''\">AND (t3.lesson_name like CONCAT('%',#{lessonName},'%') or t2.`code` like CONCAT('%',#{lessonName},'%') ) </if>" +
            "<if test=\"termCode !=null and termCode !=''\">AND t2.term_code = #{termCode} </if></script>")
    List<SlDTO> selectStudentSLByCode(@Param("studentCode") String studentCode,
                                      @Param("lessonName") String lessonName,
                                      @Param("termCode") String termCode);

    /**
     * 根据工号获取在教课程
     * @param teacherCode
     * @param lessonName
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tt2.`code` as code,\n" +
            "t2.audit_status AS auditStatus, " +
            "\tt2.sl_cover_url as slCoverUrl,\n" +
            "\tt2.business_id as businessId,\n" +
            "\tt3.lesson_name as lessonName\n" +
            "FROM\n" +
            "\tb_sl t2 \n" +
            "LEFT JOIN b_lesson t3 ON t2.lesson_code = t3.`code`\n" +
            "WHERE\n" +
            "\tt2.teacher_code = #{teacherCode}\n" +
            "AND t2.del_flag = 0\n" +
            "AND t2.flag = 1\n" +
            "AND t3.del_flag = 0\n" +
            "AND t3.flag = 1\n"+
            "AND t2.audit_status != '4' "+
            "<if test=\"lessonName !=null and lessonName !=''\">AND (t3.lesson_name like CONCAT('%',#{lessonName},'%') or t2.`code` like CONCAT('%',#{lessonName},'%') ) </if>" +
            "<if test=\"auditStatus !=null and auditStatus !=''\">AND t2.audit_status = #{auditStatus} </if>" +
            " <if test=\"termCode !=null and termCode !=''\">AND t2.term_code = #{termCode} </if></script>")
    List<SlDTO> selectTeacherSLByCode(@Param("teacherCode") String teacherCode,
                                      @Param("lessonName") String lessonName,
                                      @Param("auditStatus") String auditStatus,
                                      @Param("termCode") String termCode);

    /**
     * 根据课程id获取开课详情
     * @param id
     * @return
     */
    @Select("<script>SELECT DISTINCT(t1.business_id) as businessId,t1.credits AS credits,t1.class_hour AS classHour,\n" +
            "t1.audit_status AS auditStatus,t8.`name` AS directionName,t8.`code` AS directionCode,t9.`name` AS termName,\n" +
            "t1.`code` AS slCode,t1.sl_notes as slNotes,t1.sl_cover_url as coverUrl,\n" +
            "t6.lesson_name as lessonName,t5.head_pic as teaPic,t5.`name` AS teaName,\n" +
            "t5.t_num as teaSlNum,t5.`code` AS teaCode,\n" +
            "(select count(*) from b_chapter bc where bc.sl_id=t1.business_id and bc.is_test like CONCAT('%','1','%' )) AS experiment,\n" +
            "(select count(*) from b_chapter bc where bc.sl_id=t1.business_id and bc.is_test like CONCAT('%','2','%' )) AS video,\n" +
            "(select count(*) from b_chapter bc where bc.sl_id=t1.business_id and bc.is_test like CONCAT('%','3','%' )) AS ppt,\n" +
            "(select count(*) from b_chapter bc where bc.sl_id=t1.business_id and bc.is_test like CONCAT('%','4','%' )) AS pdf,\n" +
            "(select count(*) from b_chapter bc where bc.sl_id=t1.business_id and bc.is_test like CONCAT('%','5','%' )) AS file,\n" +
            "(select count(*) from b_chapter bc where bc.sl_id=t1.business_id and bc.is_test like CONCAT('%','6','%' )) AS questions\n" +
            "FROM\n" +
            "b_sl t1\n" +
            "LEFT JOIN b_lesson_direction t7 ON t1.business_id = t7.sl_id\n" +
            "LEFT JOIN b_direction t8 ON t7.direction_id = t8.business_id\n" +
            "LEFT JOIN b_term t9 ON t1.term_code = t9.`code`\n" +
            "LEFT JOIN b_lesson t6 ON t1.lesson_code = t6.`code`\n" +
            "LEFT JOIN (SELECT count(t3.business_id) AS t_num,t4.`name`,t2.`code`,t4.head_pic FROM b_teacher t2\n" +
            "LEFT JOIN b_sl t3 ON t2.`code` = t3.teacher_code AND (t3.audit_status='2'|| t3.audit_status='4')\n" +
            "LEFT JOIN s_user t4 ON t4.login_name = t2.`code`\n" +
            "GROUP BY t3.teacher_code) t5 ON t5.`code` = t1.teacher_code\n" +
            "WHERE\n" +
            "t1.business_id = #{id} and t1.del_flag != 1 and t1.flag = 1 </script>")
    SlDetailDTO selectSlDetailDTOById(@Param("id") String id);

    /**
     * 根据教师id获取在教课程数
     * @param teaId
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tCOUNT(b1.`code`)\n" +
            "FROM\n" +
            "\tb_sl b1\n" +
            "LEFT JOIN b_teacher b2 ON b1.teacher_code = b2.`code`\n" +
            "WHERE\n" +
            "\tb2.business_id =#{teaId} and b1.del_flag != 1 and b1.flag = 1 </script>")
    int selectSlNumByStuent(String teaId);

    /**
     * 根据课程id获取开课数
     * @param lessonId
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tCOUNT(b1.`code`)\n" +
            "FROM\n" +
            "\tb_sl b1\n" +
            "LEFT JOIN b_lesson b2 ON b1.lesson_code = b2.`code`\n" +
            "WHERE\n" +
            "\tb2.business_id = #{lessonId} and b1.del_flag != 1 and b1.flag = 1 </script>")
    int selectslNumByLesson(String lessonId);

    /**
     * 根据课程id获取开课数
     * @param termId
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tCOUNT(b1.`code`)\n" +
            "FROM\n" +
            "\tb_sl b1\n" +
            "LEFT JOIN b_term b2 ON b1.term_code = b2.`code`\n" +
            "WHERE"+
            "\tb2.business_id = #{lessonId} and b1.del_flag != 1 and b1.flag = 1</script>")
    int selectslNumByTermId(String termId);


    @Select("<script>SELECT `code` from b_sl where lesson_code = #{lessionCode} and del_flag != 1 and flag = 1</script>")
    String queryCodeByLessionCode(String lessionCode);

    @Select("<script>SELECT count(*) from b_r_sl_student where sl_code = #{code} and student_code = #{studentCode} </script>")
    int queryStuInLessionOrNot(String studentCode, String code);

    @Select("<script>SELECT count(*) from b_sl where teacher_code = #{0} and business_id = #{1} and del_flag != 1 and flag = 1</script>")
    int queryBslTeacher(String teacherCode, String businessId);

    @Select("<script>select *, bl.lesson_name as lessonName from b_sl sl " +
            "<if test=\"studentCode !=null and studentCode !=''\"> " +
            "left join b_r_sl_student stu on sl.code = stu.sl_code " +
            "</if>" +
            "left join b_term bt on sl.term_code = bt.code " +
            "left join b_lesson bl on sl.lesson_code = bl.code " +
            "where sl.del_flag != '1' and sl.flag = '1' and sl.audit_status = '4' " +
            "<if test=\"lessonName!=null and lessonName != ''\">" +
            "   AND (bl.lesson_name LIKE CONCAT('%',#{lessonName},'%') or sl.code LIKE CONCAT('%',#{lessonName},'%')) " +
            "</if>" +
            "<if test=\"termCode!=null and termCode != ''\">" +
            "   AND  sl.term_code = #{termCode} " +
            "</if>" +
            "<if test=\"studentCode!=null and studentCode != ''\">" +
            "   AND  stu.student_code = #{studentCode} " +
            "</if>" +
            "<if test=\"teacherCode!=null and teacherCode != ''\">" +
            "   AND  sl.teacher_code = #{teacherCode} " +
            "</if>" +
            "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BSl> historySlByCodeByPagination(@Param("lessonName") String lessonName,
                                          @Param("termCode") String termCode,
                                          @Param("studentCode")String studentCode,
                                          @Param("teacherCode")String teacherCode);

    @Update("<script>update b_sl set audit_status=#{code} where business_id=#{slId}</script>")
    int updateAuditStatus(@Param("slId") String slId, @Param("code") String code);

    @Select("select audit_status from b_sl where business_id = #{slId} and del_flag != 1 and flag = 1")
    String queryStateById(@Param("slId") String slId);

    @Select("select sl.code, le.lesson_name as lessonName from b_sl sl " +
            "left join b_lesson le on sl.lesson_code = le.code " +
            "where sl.business_id = #{slId} and sl.del_flag != 1 and sl.flag = 1")
    @ResultMap(value = "BaseResultMap" )
    BSl queryBSlById(String slId);


    @Select("select count(brs.student_code) AS studentNum " +
            "from b_sl sl " +
            "LEFT JOIN b_r_sl_student brs on sl.`code` = brs.sl_code " +
            "where sl.business_id = #{slId}")
    int queryStudentNumById(String slId);

    @Update("<script>update b_sl set teacher_code=#{teacherCode} where business_id=#{businessId}</script>")
    int updateTeacher(@Param("businessId") String businessId, @Param("teacherCode") String teacherCode);

    //List<BStudent> querySlStudentByPagination(String slId);
}

