package org.rcisoft.business.bslapply.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bslapply.dto.ApplyDTO;
import org.rcisoft.business.bslapply.dto.ApplyFormDTO;
import org.rcisoft.business.bslapply.entity.BSlApply;
import org.rcisoft.business.bslapply.enums.ApplyStatuEnum;
import org.rcisoft.business.bslapply.service.BSlApplyService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.io.UnsupportedEncodingException;

/**
* Created by gwf on 2017-8-10 11:05:34.
*/
@RestController
@RequestMapping("/BSlApply")
public class BSlApplyController extends PaginationController<BSlApply> {
    @Autowired
    private BSlApplyService bSlApplyService;

    @ApiOperation(value="添加/编辑开课申请", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "termCode", value = "学期标号(1-50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slName", value = "课程名称(1-50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyCourse", value = "申请理由", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyState", value = "0申请中，1已通过，2已打回", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyCode", value = "申请单号(1-50)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherCode", value = "教师编号(1-30)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "refuseCourse", value = "拒绝原因(1-150)", required = false, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/add")
    public Result add(@Valid ApplyFormDTO applyFormDTO,
                            BindingResult bindingResult) {
        BSlApply bSlApply = new BSlApply();
        BeanUtils.copyProperties(applyFormDTO,bSlApply);
        PersistModel data = bSlApplyService.persist(bSlApply,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                applyFormDTO);
    }

    @ApiOperation(value="删除开课申请", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bSlApplyService.removeBSlApply(id,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="根据主键唯一查找", notes="根据主键唯一查找")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bSlApplyService.selectOne(businessId));
    }

    @ApiOperation(value="通过开课申请", notes="通过开课申请")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "courseCode", value = "课程编号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "课序号", required = true, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/agree")
    public Result agree(@Valid ApplyDTO applyDTO,
                              BindingResult bindingResult) throws UnsupportedEncodingException {
        bSlApplyService.agreeApply(applyDTO,getToken());
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                applyDTO);
    }

    @ApiOperation(value="拒绝开课申请", notes="拒绝开课申请")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "termCode", value = "学期标号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slName", value = "课程名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyCourse", value = "申请理由", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyState", value = "0申请中，1已通过，2已打回", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyCode", value = "申请单号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherCode", value = "教师编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "refuseCourse", value = "拒绝原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/refuse")
    public Result refuse(BSlApply bSlApply) {
        bSlApply.setApplyState(ApplyStatuEnum.REFUSE.getCode());
        PersistModel data = bSlApplyService.persist(bSlApply,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bSlApply);
    }


    @ApiOperation(value="根据条件分页查询开课申请", notes="根据条件分页查询开课申请")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "termCode", value = "学期标号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slName", value = "课程名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyCourse", value = "申请理由", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyState", value = "0申请中，1已通过，2已打回", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyCode", value = "申请单号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherCode", value = "教师编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "refuseCourse", value = "拒绝原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBSlApplysByPagination")
    public GridModel queryBSlApplysByPagination(BSlApply param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bSlApplyService.queryBSlApplysByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件查询开课申请", notes="根据条件查询开课申请")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "termCode", value = "学期标号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slName", value = "课程名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyCourse", value = "申请理由", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyState", value = "0申请中，1已通过，2已打回", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "applyCode", value = "申请单号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherCode", value = "教师编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "refuseCourse", value = "拒绝原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBSlApplys")
    public Result queryBSlApplys(BSlApply param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bSlApplyService.queryBSlApplys(param));
    }
}
