package org.rcisoft.business.bslapply.service.impl;

import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.business.bsl.service.BSlService;
import org.rcisoft.business.bslapply.dao.BSlApplyRepository;
import org.rcisoft.business.bslapply.dto.ApplyDTO;
import org.rcisoft.business.bslapply.entity.BSlApply;
import org.rcisoft.business.bslapply.enums.ApplyStatuEnum;
import org.rcisoft.business.bslapply.service.BSlApplyService;
import org.rcisoft.business.bteacher.dao.BTeacherRepository;
import org.rcisoft.business.bteacher.entity.BTeacher;
import org.rcisoft.business.bterm.dao.BTermRepository;
import org.rcisoft.business.bterm.entity.BTerm;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.DelStatus;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by gwf on 2017-8-10 11:05:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BSlApplyServiceImpl implements BSlApplyService {
    @Autowired
    private BSlApplyRepository bSlApplyRepository;

    @Autowired
    private BSlRepository bSlRepository;

    @Autowired
    private BLessonRepository bLessonRepository;

    @Autowired
    private BTermRepository bTermRepository;

    @Autowired
    private BTeacherRepository bTeacherRepository;

    @Autowired
    private BSlService bSlService;

    @Autowired
    private BLessonService bLessonService;

    @Override
    public BSlApply selectOne(String businessId){
        return bSlApplyRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BSlApply> queryBSlApplysByPagination(PageUtil pageUtil, BSlApply model) {
        return bSlApplyRepository.queryBSlApplys(getParames(model));
    }

    @Override
    public List<BSlApply> queryBSlApplys(BSlApply model) {
        return bSlApplyRepository.queryBSlApplys(getParames(model));
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel persist(BSlApply model, String token){
        int line = 0;
        if(model.getTermCode()!=null&&bTermRepository.selectOne(new BTerm(model.getTermCode(),null))==null)
            throw new ServiceException(ResultServiceEnums.TERM_NOT_EXISTS);
        if(model.getTeacherCode()!=null&&bTeacherRepository.selectOne(new BTeacher(model.getTeacherCode()))==null)
            throw new ServiceException(ResultServiceEnums.TEACHER_NOT_EXISTS);
        if(model.getBusinessId()!=null) {
            UserUtil.setCurrentMergeOperation(model);
            line = bSlApplyRepository.updateByPrimaryKeySelective(model);
        }else {
            model.autoSetApplyCode();
            UserUtil.setCurrentPersistOperation(model);
            line = bSlApplyRepository.insertSelective(model);
        }
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel removeBSlApply(String id, String token) {
        BSlApply bSlApply = bSlApplyRepository.selectByPrimaryKey(id);
        bSlApply.setDelFlag(DelStatus.DELETED.getStatus());
        UserUtil.setCurrentMergeOperation(bSlApply);
        int line = bSlApplyRepository.updateByPrimaryKeySelective(bSlApply);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public void agreeApply(ApplyDTO applyDTO,String token) throws UnsupportedEncodingException {
        if(bSlRepository.selectOne(new BSl(applyDTO.getSlCode()))!=null)
            throw new ServiceException(ResultServiceEnums.SL_EXISTS);
        if(bLessonRepository.selectOne(new BLesson(applyDTO.getCourseCode()))!=null)
            throw new ServiceException(ResultServiceEnums.LESSON_EXISTS);
        //1.修改开课申请状态
        BSlApply bSlApply = bSlApplyRepository.selectByPrimaryKey(applyDTO.getBusinessId());
        if(bSlApply==null)
            throw new ServiceException(ResultServiceEnums.SL_APPLY_NOT_EXISTS);
        bSlApply.setApplyState(ApplyStatuEnum.PROCESS.getCode());
        UserUtil.setCurrentMergeOperation(bSlApply);
        int l1 = bSlApplyRepository.updateByPrimaryKeySelective(bSlApply);

        //2.添加课程信息
        BLesson bLesson = new BLesson(applyDTO.getCourseCode());
        bLesson.setLessonName(bSlApply.getSlName());
        PersistModel l2 = bLessonService.persist(bLesson,token);

        //3.添加开课信心
        BSl bSl = new BSl(
                applyDTO.getSlCode(),
                applyDTO.getCourseCode(),
                bSlApply.getTeacherCode(),
                bSlApply.getTermCode());
        PersistModel l3= bSlService.persist(bSl,token);

        if(l1!=1||!l2.isSuccessBySinglePersist()||!l3.isSuccessBySinglePersist())
            throw new ServiceException(ResultServiceEnums.UPDATE_ERROR);
    }

    private Map<String,Object> getParames(BSlApply model){
        Map param = new HashMap<String,Object>();
        param.put("termCode", StringUtils.isEmpty(model.getTermCode())?"%%":model.getTermCode());
        param.put("state", model.getApplyState());
        return param;
    }
}
