package org.rcisoft.business.bstudent.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bstudent.entity.BStudent;
import org.rcisoft.business.bstudent.service.BStudentService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.UnsupportedEncodingException;
import java.util.List;

/**
* Created by CodeGenerator on 2017/07/12.
*/
@RestController
@RequestMapping("/BStudent")
public class BStudentController extends PaginationController<BStudent> {
    @Autowired
    private BStudentService bStudentService;
    @Autowired
    private SysUserService userService;
    @Autowired
    private Global global;

    @ApiOperation(value="添加/编辑学生信息", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "学号（8-11）", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "isRecommend", value = "0:未推荐，1:推荐", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendDate", value = "推荐时间，排序依据", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendOrder", value = "推荐排序", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendCourse", value = "推荐原因(1-200)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "gradeCode", value = "年级(1-64)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "姓名（1-15）", required = false, dataType = "varchar")})
@PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/add")
    public Result add(BStudent bStudent, SysUser user) throws UnsupportedEncodingException {
        if(bStudent.getCode()!=null&&bStudent.getCode().length()>
                Integer.parseInt(global.getMAX_CODE_LENGTH()))
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
        user.setLoginName(bStudent.getCode());
        PersistModel data = bStudentService.persist(bStudent,user,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bStudent);
    }
    @ApiOperation(value="excel导入", notes="excel导入学生信息")
    @ApiImplicitParam(name = "importFile", value = "excel文件", required = true, dataType = "MultipartFile")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "excelImport")
    public Result excelImport(MultipartFile importFile) throws Exception {
        PersistModel data = bStudentService.importExcel(MultipartFile2HSSFWorkbookConverter.convert(importFile),getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data.getInfluenceReason());
    }


    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bStudentService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据条件分页查询学生信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "condition", value = "模糊查询条件", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isRecommend", value = "0:未推荐，1:推荐", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendDate", value = "推荐时间，排序依据", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendOrder", value = "推荐排序", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendCourse", value = "推荐原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBStudentsByPagination")
    public GridModel queryBStudentsByPagination(BStudent param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bStudentService.queryBStudentsByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件分页查询", notes="根据班级编号分页查询学生信息")
    @ApiImplicitParam(name = "classCode", value = "班级编号", required = true, dataType = "varchar")
    @GetMapping(value = "/queryStudentByClassCode")
    public GridModel queryStudentByClassCode(String classCode) {
        List<BStudent> bStudents = bStudentService.queryStudentByClassCodeByPagination(getPaginationUtility(),classCode);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件分页查询", notes="根据开课编号分页查询学生信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "slCode", value = "开课编号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "classCode", value = "班级编号", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryStudentBySlCode")
    public GridModel queryStudentBySlCode(String slCode, String classCode) {
        List<BStudent> bStudents = bStudentService.queryStudentBySlByPagination(getPaginationUtility(),slCode,classCode);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }


    @ApiOperation(value="物理删除", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bStudentService.removeBBStudent(id);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="根据条件查询", notes="根据条件查询学生信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "学号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isRecommend", value = "0:未推荐，1:推荐", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendDate", value = "推荐时间，排序依据", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendOrder", value = "推荐排序", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendCourse", value = "推荐原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBStudents")
    public Result queryBStudents(BStudent param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bStudentService.queryBStudents(param));
    }

    @ApiOperation(value="获取推荐学生", notes="分页获取推荐学生")
    @GetMapping(value = "/queryStudentWithRecommendWithoutAuth")
    public GridModel queryStudentWithRecommend(){
        List<BStudent> students = bStudentService.queryStudentsWithIsRecommendByPagination(getPaginationUtility());
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }


    @ApiOperation(value="查询本课学生", notes="根据开课编号分页查询本课学生")
    @ApiImplicitParams({@ApiImplicitParam(name = "slCode", value = "开课编号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "keyWord", value = "学号或关键字", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "classCode", value = "班级编号", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryStudentInSlByPagination")
    public GridModel queryStudentInSlByPagination(String slCode, String classCode, String keyWord, String slId) {
        List<BStudent> bStudents = bStudentService.queryStudentInSlByPagination(getPaginationUtility(),slCode,classCode,keyWord,slId);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }


    @ApiOperation(value="根据班级code查询班组所属学生", notes="根据班级code查询班组所属学生")
    @ApiImplicitParam(name = "classesCode", value = "班级编号  逗号隔开", required = true, dataType = "varchar")
    @GetMapping(value = "/queryStudentByClassesCode")
    public GridModel queryStudentByClassesCode(String classesCode) {
        List<BStudent> bStudents = bStudentService.queryStudentByPagination(getPaginationUtility(),classesCode);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }
}
