package org.rcisoft.business.bstudent.dao;

import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.business.bclass.entity.BClass;
import org.rcisoft.business.bstudent.entity.BStudent;
import org.rcisoft.business.bstudent.entity.BStudentDto;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
* Created with CodeGenerator on .
*/
@Repository
public interface BStudentRepository extends BaseMapper<BStudent> {

    /**
    * 根据条件查找
    * @param param
    * @return
    */
    @Select("<script>SELECT\n" +
            "b.`business_id`,b.`code`, b.`is_recommend`, b.`recommend_order`,b.`recommend_course`,b.`recommend_date`,\n" +
            "b.create_by,b.create_date,b.del_flag,b.flag,b.remarks,b.update_by,b.update_date,u.`name`,\n" +
            "u.email,u.phone,u.sex,u.head_pic,b.grade_code,bg.`name` as gradeName\n" +
            "FROM b_student b\n" +
            "LEFT JOIN s_user u ON b.`code` = u.login_name\n" +
            "LEFT JOIN b_grade bg ON b.grade_code = bg.`code`\n" +
            "where b.del_flag != 1 and b.flag = 1  "+
            "<if test=\"condition!=null and condition != ''\">and u.`name` like CONCAT('%',#{condition},'%') or b.`code` like CONCAT('%',#{condition},'%') </if>"+
            "order by b.create_date desc" +
            "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BStudent> queryBStudents(BStudent param);

    @Select("<script>SELECT\n" +
            "\tt3.`code`,t4.`name`\n" +
            "FROM\n" +
            "b_r_class_student t2\n" +
            "LEFT JOIN b_student t3 ON t2.student_code = t3.`code`\n" +
            "LEFT JOIN s_user t4 ON t3.`code` = t4.login_name\n" +
            "WHERE\n" +
            "\tt2.class_code = #{classCode} and t3.del_flag != 1 and t3.flag = 1 order by t3.grade_code desc</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BStudent> queryStudentByClassCode(@Param("classCode") String classCode);

    @Select("<script>SELECT\n" +
            "\tt3.*,t4.`name`,t6.class_name as className\n" +
            "FROM\n" +
            "b_r_sl_student t2\n" +
            "LEFT JOIN b_student t3 ON t2.student_code = t3.`code`\n" +
            "LEFT JOIN s_user t4 ON t3.`code` = t4.login_name\n" +
            "LEFT JOIN b_r_class_student t5 on t5.student_code = t3.code " +
            "LEFT JOIN b_class t6 on t6.code = t5.class_code " +
            "WHERE\n" +
            "\tt2.sl_code = #{slCode} " +
            " and t3.del_flag != 1 and t3.flag = 1 " +
            "<if test=\"classCode!=null and classCode != ''\">\n" +
            "   AND t6.code = #{classCode}\n" +
            "</if>" +
            "group by t3.code " +
            "order by className asc,code asc</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BStudent> queryStudentBySlCode(@Param("slCode") String slCode, @Param("classCode") String classCode);

    @Insert("<script>INSERT INTO b_student" +
            "(create_date,update_date,create_by,update_by,remarks,del_flag,flag,business_id,code,grade_code)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "( #{item.createDate},#{item.updateDate},#{item.createBy},#{item.updateBy},#{item.remarks},#{item.delFlag},#{item.flag},#{item.businessId},#{item.code},#{item.gradeCode})" +
            "</foreach></script>")
    int insertList(List<BStudent> bStudents);

    /**
     * 查询推荐学生
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tb.`business_id`,\n" +
            "\tb.`code`,\n" +
            "\tb.`is_recommend`,\n" +
            "\tb.`recommend_date`,\n" +
            "\tb.`recommend_order`,\n" +
            "\tb.`recommend_course`,\n" +
            "\tb.create_by,\n" +
            "\tb.create_date,\n" +
            "\tb.del_flag,\n" +
            "\tb.flag,\n" +
            "\tb.remarks,\n" +
            "\tb.update_by,\n" +
            "\tb.update_date,\n" +
            "\tu.`name`,\n" +
            "\tu.email,\n" +
            "\tu.phone,\n" +
            "\tu.sex,\n" +
            "\tu.head_pic\n" +
            "FROM\n" +
            "\tb_student b\n" +
            "LEFT JOIN s_user u ON b.`code` = u.login_name\n" +
            "where b.del_flag = '0' " +
            "and b.flag = '1' "+
            "and b.is_recommend = '1' "+
            "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BStudent> queryStudentWithReCommend();

    @Select("<script>select b.business_id from b_student b left join s_user s on b.code = s.login_name where s.business_id = #{userId} and s.del_flag != 1 and s.flag = 1</script>")
    String queryStudentByUserId(String userId);


    @Select("<script>SELECT t3.business_id as businessId, t3.code,t4.`name`,t6.class_name as className," +
            "IFNULL(bsc.is_complete,0) as submit, " +
            "(CONCAT((SELECT COUNT(*) from b_r_student_chapter " +
            "where student_id = t3.business_id and sl_id = #{slId} and is_complete = '1')," +
            "'/'," +
            "(SELECT COUNT(*) from b_chapter where is_test like CONCAT('%','1','%') and sl_id = #{slId})) \n" +
            ") AS finishNum " +
            "FROM\n" +
            "b_r_sl_student t2\n" +
            "LEFT JOIN b_student t3 ON t2.student_code = t3.`code` " +
            "LEFT JOIN s_user t4 ON t3.`code` = t4.login_name " +
            "LEFT JOIN b_r_class_student t5 on t5.student_code = t3.code " +
            "LEFT JOIN b_class t6 on t6.code = t5.class_code " +
            "LEFT JOIN b_r_student_chapter bsc on bsc.student_id = t3.business_id " +
            "WHERE " +
            "t2.sl_code = #{slCode} " +
            " and t3.del_flag != 1 and t3.flag = 1" +
            "<if test=\"classCode!=null and classCode != ''\"> " +
            "   AND t6.code = #{classCode} " +
            "</if>" +
            "<if test=\"keyWord!=null and keyWord != ''\"> " +
            "   AND (t3.`code` like CONCAT('%',#{keyWord},'%') or t4.`name` like CONCAT('%',#{keyWord},'%')) " +
            "</if>" +
            "GROUP BY t3.`code` " +
            "order by className asc,code asc</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BStudent> queryStudentInSl(@Param("slCode") String slCode,
                                    @Param("classCode")String classCode,
                                    @Param("keyWord")String keyWord,
                                    @Param("slId")String slId);

    @Select("<script>select su.`name` AS studentName, stu.`code` AS studentCode " +
            "from b_student stu LEFT JOIN s_user su on stu.`code` = su.login_name \n" +
            "where FIND_IN_SET(su.business_id,#{studentId}) " +
            "and su.del_flag != 1 and su.glag = 1</script>")
    List<BStudentDto> queryStudentByBusinessId(String studentId);


    @Select("<script>select business_id from b_student where code = #{studentCode} and del_flag != 1 and flag = 1</script>")
    String queryStudentIdByCode(String studentCode);

    /**
    @Select("<script>select * from b_student bs " +
            "LEFT JOIN b_r_class_student cs on bs.`code` = cs.student_code " +
            "where cs.class_code = #{classCode} and del_flag != 1 and flag = 1</script>")
    @ResultMap(value = "BaseResultMap")
    List<BStudent> queryStudentsByClassCode(String classCode);
    */

    @Select("SELECT * FROM b_student \n" +
            "LEFT JOIN b_r_class_student ON b_student.`code` = b_r_class_student.student_code\n" +
            "LEFT JOIN b_class ON b_r_class_student.class_code = b_class.`code`\n" +
            "LEFT JOIN s_user ON b_student.`code` = s_user.login_name\n" +
            "WHERE b_r_class_student.class_code = #{classCode}\n" +
            "and b_student.del_flag != 1 \n" +
            "and b_student.flag = 1")
    @ResultMap(value = "BaseResultMap")
    List<BStudent> queryStudentsByClassCode(String classCode);

    @Select("select * from b_grade where code = #{gradeCode} and del_flag = 0 and flag = 1")
    List<BClass> checkStudentGrade(BStudent bStudent);

    @Select("SELECT bs.* FROM b_student bs " +
            "LEFT JOIN b_r_class_student bcs on bcs.student_code = bs.`code` " +
            "LEFT JOIN b_class bc on bc.`code` = bcs.class_code " +
            "WHERE FIND_IN_SET(bcs.class_code,( " +
            "SELECT bec.classes_id FROM b_edu_class bec WHERE bec.`code` = #{classCode})) " +
            "GROUP BY bs.`code` " +
            "ORDER BY bs.`code`")
    @ResultMap(value = "BaseResultMap")
    List<BStudent> queryStudentByEduClassCode(String classCode);

    @Select("SELECT bcs.class_code from b_student bs \n" +
            "LEFT JOIN b_r_class_student bcs on bcs.student_code = bs.`code`\n" +
            "WHERE bs.business_id = #{id}")
    List<String> queryClassCodesById(String id);
}

