package org.rcisoft.business.bstudent.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSON;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.bclass.dao.BClassRepository;
import org.rcisoft.business.brclassstudent.dao.BRClassStudentRepository;
import org.rcisoft.business.brclassstudent.entity.BRClassStudent;
import org.rcisoft.business.brslstudent.dao.BRSlStudentRepository;
import org.rcisoft.business.bstudent.dao.BStudentRepository;
import org.rcisoft.business.bstudent.entity.BStudent;
import org.rcisoft.business.bstudent.service.BStudentService;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.constant.UserConstant;
import org.rcisoft.core.entity.DataEntity;
import org.rcisoft.core.enums.UserSexEnum;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.RegexValidateUtil;
import org.rcisoft.core.util.TestChinese;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.entity.UserRole;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class BStudentServiceImpl implements BStudentService {
    @Autowired
    private BStudentRepository bStudentRepository;
    @Autowired
    private BRClassStudentRepository brClassStudentRepository;
    @Autowired
    private SysUserService userService;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private Global global;
    @Autowired
    private BRSlStudentRepository brSlStudentRepository;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private BClassRepository bClassRepository;

    @Override
    public BStudent selectOne(String businessId) {
        return bStudentRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BStudent> queryBStudentsByPagination(PageUtil pageUtil, BStudent model) {
        return bStudentRepository.queryBStudents(model);
    }

    @Override
    public List<BStudent> queryBStudents(BStudent model) {
        return bStudentRepository.queryBStudents(model);
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel persist(BStudent model, SysUser user, String token) throws UnsupportedEncodingException {
        int line = 0;
        BeanUtils.copyProperties((DataEntity) model, (DataEntity) user);
        if (model.getBusinessId() == null) {  //添加
            if (sysUserMapper.selectOne(new SysUser(model.getCode())) != null)
                throw new ServiceException(ResultServiceEnums.STUDENT_EXISTS);
            /*SysUser user1 = new SysUser();
            user1.setEmail(model.getEmail());
            user1.setDelFlag("0");
            user1.setFlag("1");
            if(sysUserMapper.selectOne(user1)!=null)
                throw new ServiceException(ResultServiceEnums.EMAIL_EXISTS);*/

            //1. 添加用户
            UserUtil.setCurrentPersistOperation(user);
            user.setHeadPic(global.getDEFAULT_STUDENT_LOCATION());
            userService.persistUser(user);

            //2.添加学生
            UserUtil.setCurrentPersistOperation(model);
            line = bStudentRepository.insertSelective(model);

            //3.添加权限
            UserRole userRole = new UserRole(user.getBusinessId(), "cccccccc");
            userRole.setCommonBusinessId();

            int line1 = sysUserMapper.insertUserRole(userRole);
            if (line1 != line)
                throw new ServiceException(ResultServiceEnums.INSERT_ERROR);
        } else {  //修改
            String uid = sysUserMapper.selectUserIdByStuOrTeaId(model.getBusinessId(), "b_student");
            if (uid == null)
                throw new ServiceException(ResultServiceEnums.STUDENT_NOT_EXISTS);
            UserUtil.setCurrentMergeOperation(model);
            line = bStudentRepository.updateByPrimaryKeySelective(model);
            userService.updateUser(user,uid,line);
        }
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel importExcel(HSSFWorkbook hwb,String token) throws IOException {
        ArrayList<BStudent> students = new ArrayList<BStudent>();
        List<String> currentlyCode = new ArrayList<>();
        String innerRepeatResult = "";
        ArrayList<SysUser> users = new ArrayList<SysUser>();
        ArrayList<UserRole> userRoles = new ArrayList<UserRole>();
        List<String> valuesIncomplete = new ArrayList<>();
        String[] headers = {"学号", "学生姓名", "性别","年级","备注"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb, headers,true,1);  //获取excel数据
        
        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        List<String[]> valuesOK = new ArrayList<>();
        List<String> valuesRepeat = new ArrayList<>();
        List<String> valuesShort = new ArrayList<>();
        List<String> valuesEmailExist = new ArrayList<>();
        List<String> isChinese = new ArrayList<>();
        List<String> nameTooLong = new ArrayList<>();
        List<String> valuesGrade = new ArrayList<>();

        int i = 0;
        for (String[] value : values) {
            i++;
            String stuCode = value[0];

            //判断学号是否含有中文
            if(TestChinese.isChinese(stuCode)){
                isChinese.add(value[0]);
                continue;
            };

            //判断1:学号是否满足8~15位；
            if(stuCode.length()<8||stuCode.length()>15){
                //该用户不满足要求，记入valuesShort中
                valuesShort.add(value[0]);
                continue;
            }

            value[1].trim();
            if(value[1].length()<1||value[1].length()>15){
                //该用户不满足要求，记入valuesShort中
                nameTooLong.add(value[0]);
                continue;
            }

            //判断2:MySQL数据库中是否存在该用户；
            SysUser user1 =new SysUser();
            user1.setLoginName(value[0]);
            if (sysUserMapper.selectOne(user1) != null) {
                //该用户已存在，记入codeRepeat中
                valuesRepeat.add(value[0]);
                continue;
            }

            //判断3:邮箱是否存在、格式是否正确；
//            SysUser user2 = new SysUser();
//            user2.setEmail(value[3]);
//            boolean nullEmail = StringUtils.isEmpty(value[3]);
//            boolean emailCheck = RegexValidateUtil.checkEmail(value[3]);
//            boolean emailRepeat = sysUserMapper.selectOne(user2)!=null;
//            if(nullEmail||!emailCheck||emailRepeat){
//                valuesEmailExist.add(value[3]);
//                continue;
//            }

            //判断4:学年是否存在，存在才能录入
            BStudent bs = new BStudent();
            bs.setGradeCode(value[3]);
            if (bStudentRepository.checkStudentGrade(bs).size() == 0){
                valuesGrade.add(value[3]);
                continue;
            }

            if(value[0].equals("")||value[1].equals("")||value[2].equals("")||value[3].equals("")){
                valuesIncomplete.add(String.valueOf(i));
                continue;
            }

            //合法:通过了三次判断，证明可建立该用户
            valuesOK.add(value);
            //为合法的用户建立Gitlab账户，并批量存储Student、User和UserRole;
            if (value[2].equals("女"))
                value[2] = UserSexEnum.WOMEN.getCode();
            else if(value[2].equals("男"))
                value[2] = UserSexEnum.MAN.getCode();
            else
                value[2] = "";
            //1. 添加user
            SysUser user = new SysUser(value[0], passwordEncoder.encode(global.getDEFAULT_PASSWORD()), value[1], value[2]);  //学号，密码，姓名，性别;
//            user.setEmail(value[3]);
            UserUtil.setCurrentPersistOperation(user);
            //2 创建gitlab账户
            /*GitUser gitUser = new GitUser();
            gitUser.setUsername(global.getGIT_LAB_STU_PREFIX()+value[0]);
            gitUser.setName(value[1]);
            gitUser.setPassword(value[0]);
            gitUser.setEmail(value[3]);
            AccountDTO accountDTO = action.createGitLabAccount(gitUser);
            //3. 将gitlab账户信息保存进user里
            user.setGitLabId(accountDTO.getId());
            user.setGitLabUsername(accountDTO.getUsername());
            user.setGitLabImpressionToken(accountDTO.getImpersonationToken());*/
            user.setHeadPic(global.getDEFAULT_STUDENT_LOCATION());

            //4.添加用户权限
            UserRole userRole = new UserRole(user.getBusinessId(), "cccccccc");
            userRole.setCommonBusinessId();

            //创建完Gitlab账户后，将完整的User信息和UserRole计入users和userRoles中等待插入数据库
            BStudent bStudent = new BStudent(value[0]);
            bStudent.setGradeCode(value[3]);
            bStudent.setRemarks(value[4]);
            UserUtil.setCurrentPersistOperation(bStudent);
            if(currentlyCode.contains(bStudent.getCode())){
                innerRepeatResult += "编号 " + value[0] + " 在Excel中重复出现，除第一条以外的数据插入失败 \n";
                continue;
            }
            if (!students.contains(bStudent)) {
                currentlyCode.add(bStudent.getCode());
                students.add(bStudent);
                users.add(user);
                userRoles.add(userRole);
            }
        }


        String result = "";
        if (valuesOK.size()>0){
            int line1 = bStudentRepository.insertList(students);
            int line2 = sysUserMapper.insertList(users);
            int line3 = sysUserMapper.insertUserRoleList(userRoles);
            if (line1 != line2 || line1 != students.size() || line2 != users.size() || line1 != line3 || line3 != userRoles.size()) {
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);
            }else{
                result+="成功导入"+users.size()+"名学生。";
            }
        }

        if(isChinese.size()>0){
            result+="以下学生的学号格式不正确："+ JSON.toJSONString(isChinese)+"。";
        }
        if(valuesRepeat.size()>0){
            result+="以下学生的学号存在重复："+ JSON.toJSONString(valuesRepeat)+"。";
        }
        if(nameTooLong.size()>0){
            result+="以下学生的姓名格式不正确："+ JSON.toJSONString(nameTooLong)+"。";
        }
        if(valuesShort.size()>0){
            result+="以下学生的学号位数不在8~15位之间："+ JSON.toJSONString(valuesShort)+"。";
        }
        if(valuesEmailExist.size()>0){
            result+="以下邮箱已注册："+ JSON.toJSONString(valuesEmailExist)+"。";
        }
        if(valuesGrade.size()>0){
            result+="以下学年不存在："+ JSON.toJSONString(valuesGrade)+"。";
        }
        if(valuesIncomplete.size()>0){
            result+="第"+ JSON.toJSONString(valuesIncomplete)+"行数据不完整";
        }


        if (valuesOK.size()<1){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR.getCode(),result);
        }
        return new PersistModel(1, result + innerRepeatResult);



    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel removeBBStudent(String id) {
        if(brSlStudentRepository.selectSlNumByStuId(id)>0) {
            throw new ServiceException(ResultServiceEnums.DATA_HAS_USED);
        }
        BStudent bStudent = new BStudent();
        bStudent.setBusinessId(id);
        BStudent student = bStudentRepository.selectOne(bStudent);
        sysUserMapper.deleteByCode(student.getCode());

        List<String> classCodes = bStudentRepository.queryClassCodesById(id);
        String clCode = "";
        for(String classCode : classCodes){
            clCode += classCode + ",";
        }
        bClassRepository.updateStudentNumByClassCodes(clCode);//更新班级人数-1

        int line = bStudentRepository.deleteByPrimaryKey(bStudent);
        brClassStudentRepository.deleteCSByStuId(student.getCode());

        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    public List<BStudent> queryStudentByClassCodeByPagination(PageUtil pageUtil, String code) {
        List<BStudent> list = bStudentRepository.queryStudentByClassCode(code);
        return list;
    }

    @Override
    public List<BStudent> queryStudentBySlByPagination(PageUtil pageUtil, String code, String classCode) {
        return bStudentRepository.queryStudentBySlCode(code,classCode);
    }

    @Override
    public List<BStudent> queryStudentsWithIsRecommendByPagination(PageUtil pageUtil) {
        return bStudentRepository.queryStudentWithReCommend();
    }

    /**
     *
     * @param pageUtil
     * @param slCode
     * @param classCode
     * @param keyWord
     * @return
     */
    @Override
    public List<BStudent> queryStudentInSlByPagination(PageUtil pageUtil, String slCode, String classCode, String keyWord, String slId) {
        return bStudentRepository.queryStudentInSl(slCode,classCode,keyWord,slId);
    }

    @Override
    public List<BStudent> queryStudentByPagination(PageUtil pageUtil, String classesCode) {
        List<BStudent> students = new ArrayList<>();
        for(String classCode : classesCode.split(",")){
            List<BStudent> studentList = bStudentRepository.queryStudentsByClassCode(classCode);
            students.addAll(studentList);
        }
        return students;
    }

    private Map<String, Object> queryParamHandler(BStudent model) {
        Map param = new HashMap<String, Object>();
        if (model.getName() != null)
            param.put("name", "%" + model.getName() + "%");
        else
            param.put("name", "%%");
        if (model.getCode() != null)
            param.put("code", "%" + model.getCode() + "%");
        else
            param.put("code", "%%");
        return param;
    }
}
