package org.rcisoft.business.bteacher.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSON;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bslschedule.dao.BSlScheduleRepository;
import org.rcisoft.business.bslschedule.entity.BSlSchedule;
import org.rcisoft.business.bteacher.dao.BTeacherRepository;
import org.rcisoft.business.bteacher.entity.BTeacher;
import org.rcisoft.business.bteacher.service.BTeacherService;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.constant.UserConstant;
import org.rcisoft.core.entity.DataEntity;
import org.rcisoft.core.enums.UserSexEnum;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.RegexValidateUtil;
import org.rcisoft.core.util.TestChinese;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.entity.UserRole;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class BTeacherServiceImpl implements BTeacherService {
    @Autowired
    private BTeacherRepository bTeacherRepository;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private SysUserService userService;
    /*@Autowired
    private IAction action;*/
    @Autowired
    private Global global;
    @Autowired
    private BSlRepository bSlRepository;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private BSlScheduleRepository bSlScheduleRepository;

    @Override
    public BTeacher selectOne(String businessId) {
        return bTeacherRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BTeacher> queryBTeachersByPagination(PageUtil pageUtil, BTeacher model) {
        return bTeacherRepository.queryBTeachers(model);
    }

    @Override
    public List<BTeacher> queryBTeachers(BTeacher model) {
        return bTeacherRepository.queryBTeachers(model);
//        return bTeacherRepository.queryBTeachers(queryParamHandler(model));
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel importExcel(HSSFWorkbook hwb, String token) throws IOException, InterruptedException {
        ArrayList<BTeacher> teachers = new ArrayList<BTeacher>();
        ArrayList<SysUser> users = new ArrayList<SysUser>();
        String innerRepeatResult = "";
        List<String> currentlyCode = new ArrayList<>();
        ArrayList<UserRole> userRoles = new ArrayList<UserRole>();
        String[] headers = {"教师工号", "教师姓名", "性别","教学单位编号","备注"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb, headers,true,1);  //获取excel数据
        if (values.size() == 0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        List<String[]> valuesOK = new ArrayList<>();
        List<String> valuesRepeat = new ArrayList<>();
        List<String> valuesShort = new ArrayList<>();
        List<String> valuesEmailExist = new ArrayList<>();
        List<String> isChinese = new ArrayList<>();
        List<String> nameTooLong = new ArrayList<>();

        List<String> valuesIncomplete = new ArrayList<>();
        List<String> valuesAgencyExist = new ArrayList<>();
        int i = 0;
        for (String[] value : values) {
            i++;
            String teaCode = value[0];
            //判断工号是否含有中文
            if(TestChinese.isChinese(teaCode)){
                isChinese.add(value[0]);
                continue;
            };

            //判断1:学号是否满足8~15位；
            if(teaCode.length()<8||teaCode.length()>15){
                //该用户不满足要求，记入valuesShort中
                valuesShort.add(value[0]);
                continue;
            }

            value[1].trim();
            if(value[1].length()<1||value[1].length()>15){
                //该用户不满足要求，记入valuesShort中
                nameTooLong.add(value[0]);
                continue;
            }

            //判断2:MySQL数据库中是否存在该用户；
            SysUser user1 =new SysUser();
            user1.setLoginName(value[0]);
            if (sysUserMapper.selectOne(user1) != null) {
                //该用户已存在，记入codeRepeat中
                valuesRepeat.add(value[0]);
                continue;
            }

            //判断3:邮箱是否存在、格式是否正确；
//            SysUser user2 = new SysUser();
//            user2.setEmail(value[3]);
//            boolean nullEmail = StringUtils.isEmpty(value[3]);
//            boolean emailCheck = RegexValidateUtil.checkEmail(value[3]);
//            boolean emailRepeat = sysUserMapper.selectOne(user2)!=null;
//            if(nullEmail||!emailCheck||emailRepeat){
//                valuesEmailExist.add(value[3]);
//                continue;
//            }

            //判断4:教师的所属教学单位是否存在，存在才能添加；
            if(!value[3].equals("")&&value[3]!=null){
                if (sysUserMapper.checkTeacherAgency(value[3]).size() == 0){
                    valuesAgencyExist.add(value[3]);
                    continue;
                }
            }
            if(value[0].equals("")||value[1].equals("")||value[2].equals("")){
                valuesIncomplete.add(String.valueOf(i));
                continue;
            }
            //合法:通过了三次判断，证明可建立该用户
            valuesOK.add(value);

            if (value[2].equals("女"))
                value[2] = UserSexEnum.WOMEN.getCode();
            else if(value[2].equals("男"))
                value[2] = UserSexEnum.MAN.getCode();
            else
                value[2] = "";

            //1. 添加user
            SysUser user = new SysUser(value[0], passwordEncoder.encode(global.getDEFAULT_PASSWORD()), value[1], value[2]);  //教师工号，密码，姓名，性别;
//          user.setEmail(value[3]);
            UserUtil.setCurrentPersistOperation(user);
            //2 创建gitlab账户
            /*GitUser gitUser = new GitUser();
            gitUser.setUsername(global.getGIT_LAB_STU_PREFIX()+value[0]);
            gitUser.setName(value[1]);
            gitUser.setPassword(value[0]);
            gitUser.setEmail(value[3]);
            AccountDTO accountDTO = action.createGitLabAccount(gitUser);
            //3. 将gitlab账户信息保存进user里
            user.setGitLabId(accountDTO.getId());
            user.setGitLabUsername(accountDTO.getUsername());
            user.setGitLabImpressionToken(accountDTO.getImpersonationToken());*/
            user.setHeadPic(global.getDEFAULT_TEACHER_LOCATION());

            //4.添加用户权限
            UserRole userRole = new UserRole(user.getBusinessId(), "bbbbbbbb");
            userRole.setCommonBusinessId();

            //创建完Gitlab账户后，将完整的User信息和UserRole计入users和userRoles中等待插入数据库
            BTeacher bTeacher= new BTeacher(value[0],value[3]);
            bTeacher.setRemarks(value[4]);
            UserUtil.setCurrentPersistOperation(bTeacher);
            if(currentlyCode.contains(bTeacher.getCode())){
                innerRepeatResult += "编号 " + bTeacher.getCode() + " 在Excel中重复出现，除第一条以外的数据插入失败 ";
                continue;
            }
            if (!teachers.contains(bTeacher)) {
                currentlyCode.add(bTeacher.getCode());
                teachers.add(bTeacher);
                users.add(user);
                userRoles.add(userRole);
            }
        }
        String result = "";
        if (teachers.size()>0){
            int line1 = bTeacherRepository.insertList(teachers);
            int line2 = sysUserMapper.insertList(users);
            int line3 = sysUserMapper.insertUserRoleList(userRoles);
            if (line1 != line2 || line1 != teachers.size() || line2 != users.size() || line1 != line3 || line3 != userRoles.size()) {
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);
            }else{
                result+="成功导入"+users.size()+"名教师。";
            }
        }

        if(isChinese.size()>0){
            result+="以下教师的工号格式不正确："+ JSON.toJSONString(isChinese)+"。";
        }
        if(valuesRepeat.size()>0){
            result+="以下教师的工号存在重复："+ JSON.toJSONString(valuesRepeat)+"。";
        }
        if(nameTooLong.size()>0){
            result+="以下教师的姓名格式不正确："+ JSON.toJSONString(nameTooLong)+"。";
        }
        if(valuesShort.size()>0){
            result+="以下教师的工号位数不在8~15位之间："+ JSON.toJSONString(valuesShort)+"。";
        }
        if(valuesIncomplete.size()>0){
            result+="第"+ JSON.toJSONString(valuesIncomplete)+"行数据不完整";
        }
        if (valuesAgencyExist.size()>0){
            result+="以下教师的所属教学单位不存在："+JSON.toJSONString(valuesAgencyExist)+"。";
        }

        if (valuesOK.size()<1){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR.getCode(),result);
        }


        return new PersistModel(1, result + innerRepeatResult);

    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel removeBTeacher(String id) {
        if(bSlRepository.selectSlNumByStuent(id)>0 ||
                bSlScheduleRepository.querySlScheduleByTeaCode(id)>0 ||
                bSlScheduleRepository.querySlScheduleDtoByTeaCode(id)>0) {
            throw new ServiceException(ResultServiceEnums.DATA_HAS_USED);
        }
        BTeacher bTeacher = new BTeacher();
        bTeacher.setBusinessId(id);
        BTeacher teacher = bTeacherRepository.selectOne(bTeacher);
        sysUserMapper.deleteByCode(teacher.getCode());
//        bTeacher.setDeleted();
        int line = bTeacherRepository.deleteByPrimaryKey(bTeacher);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    public List<BTeacher> queryTeachersWithRecommendByPagination(PageUtil pageUtil) {
        return bTeacherRepository.queryTeacherWithReCommend();
    }


    @Override
    @Transactional(readOnly = false,propagation = Propagation.REQUIRED,rollbackFor = Exception.class)
    public PersistModel persist(BTeacher model, SysUser user, String token) {
        int line = 0;
        BeanUtils.copyProperties((DataEntity) model, (DataEntity) user);
        if (model.getBusinessId() == null) {
            if (sysUserMapper.selectOne(new SysUser(model.getCode())) != null)
            {throw new ServiceException(ResultServiceEnums.TEACHER_EXISTS);}
            /*SysUser user1 = new SysUser();
            user1.setEmail(model.getEmail());
            user1.setDelFlag("0");
            user1.setFlag("1");
            if(user1.getEmail()!=null) {
                if (sysUserMapper.selectOne(user1) != null) {
                    throw new ServiceException(ResultServiceEnums.EMAIL_EXISTS);
                }
            }*/
            //1添加用户
            user.setHeadPic(global.getDEFAULT_TEACHER_LOCATION());
            UserUtil.setCurrentPersistOperation(user);
            userService.persistUser(user);

            //2添加用户权限
            UserRole userRole = new UserRole(user.getBusinessId(), "bbbbbbbb");
            userRole.setCommonBusinessId();
            int line1 = sysUserMapper.insertUserRole(userRole);

            //添加教师
            UserUtil.setCurrentPersistOperation(model);
            line = bTeacherRepository.insertSelective(model);

            if (line1 != line)
            {throw new ServiceException(ResultServiceEnums.INSERT_ERROR);}
        } else {
            UserUtil.setCurrentMergeOperation(model);
            line = bTeacherRepository.updateByPrimaryKeySelective(model);
            String uid = sysUserMapper.selectUserIdByStuOrTeaId(model.getBusinessId(), "b_teacher");
            if (uid == null)
            {throw new ServiceException(ResultServiceEnums.STUDENT_NOT_EXISTS);}
            userService.updateUser(user,uid,line);
        }
        return new PersistModel(line);
    }

    private Map<String, Object> queryParamHandler(BTeacher model) {
        Map param = new HashMap<String, Object>();
        if (model.getName() != null)
        {param.put("name", "%" + model.getName() + "%");}
        else
        {param.put("name", "%%");}
        if (model.getCode() != null)
        {param.put("code", "%" + model.getCode() + "%");}
        else
            {param.put("code", "%%");}
        param.put("teacherCode", model.getTeacherCode());
        return param;
    }
}
