package org.rcisoft.business.bterm.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.aspectj.weaver.loadtime.Aj;
import org.rcisoft.business.bterm.dto.BTermDTO;
import org.rcisoft.business.bterm.entity.BTerm;
import org.rcisoft.business.bterm.service.BTermService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.converter.BTermDTO2BTermConverter;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.model.SelectModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.beans.factory.annotation.Autowired;

import javax.validation.Valid;
import java.util.List;

/**
* Created by CodeGenerator on 2017/07/12.
*/
@RestController
@RequestMapping("/BTerm")
public class BTermController extends PaginationController<BTerm> {
    @Autowired
    private BTermService bTermService;
    @Autowired
    private Global global;

    @ApiOperation(value="添加/编辑学期", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "startDate", value = "学期开始时间", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "endDate", value = "学期结束时间", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "isStart", value = "是否启动(0：未启动，1：启动)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "学期编号(1-15)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "学期名称(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/add")
    public Result add(@Valid BTermDTO bTermDTO, BindingResult bindingResult) {
        /* if(bTermDTO.getCode().length() >
                Integer.parseInt(global.getMAX_CODE_LENGTH()))
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);*/
        BTerm bTerm = BTermDTO2BTermConverter.convert(bTermDTO);
        PersistModel data = bTermService.persist(bTerm,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bTermDTO);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bTermService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据URL中的参数分页查询学期")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "startDate", value = "学期开始时间", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "endDate", value = "学期结束时间", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isStart", value = "是否启动(0：未启动，1：启动)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "学期编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "学期名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBTermsByPagination")
    public GridModel queryBTermsByPagination(BTerm param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bTermService.queryBTermsByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件查询", notes="根据URL中的参数查询学期")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "startDate", value = "学期开始时间", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "endDate", value = "学期结束时间", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isStart", value = "是否启动(0：未启动，1：启动)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "学期编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "学期名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBTerms")
    public Result queryBTerms(BTerm param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bTermService.queryBTerms(param));

    }

    @ApiOperation(value="逻辑删除", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bTermService.removeBTerm(id);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="修改 is_start值", notes="根据学期编号修改 is_start值")
    @ApiImplicitParam(name = "code", value = "学期编号", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/changeIS")
    public Result changeIS(@RequestParam("code") String code)
    {
        PersistModel data=bTermService.changeIS(code,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                code);
    }


    @ApiOperation(value="查询学期列表", notes="查询学期列表赋值在下拉框中")
    @ApiImplicitParams({@ApiImplicitParam(name = "current", value = "是否将当前学期排在最前 0否1是", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "arrange", value = "是否是排课任务的下拉 0否1是", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBTermList")
    public Result queryBTermList(String current, String arrange, String flag) {
        List<SelectModel> bTerm = bTermService.queryBTermList(current,arrange,flag);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bTerm);
    }


    @ApiOperation(value="更改当前学期", notes="更改当前学期")
    @ApiImplicitParam(name = "code", value = "code", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "/changeCurrent")
    public Result changeCurrent(@RequestParam("code") String code){
        PersistModel data=bTermService.changeCurrent(code);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                code);
    }

    @ApiOperation(value="获取校历", notes="获取校历")
    @ApiImplicitParam(name = "termCode", value = "termCode", required = true, dataType = "varchar")
    @GetMapping(value = "/getSchoolWeek")
    public Result getSchoolWeek(String termCode){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bTermService.getSchoolWeek(termCode));
    }
}
