package org.rcisoft.business.bterm.service.impl;

import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bterm.dao.BTermRepository;
import org.rcisoft.business.bterm.entity.BTerm;
import org.rcisoft.business.bterm.enums.BTermEnum;
import org.rcisoft.business.bterm.service.BTermService;
import org.rcisoft.business.bterm.utils.SchoolCalendarWeek;
import org.rcisoft.business.bterm.utils.Week;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.model.SelectModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BTermServiceImpl implements BTermService {

    @Autowired
    private BTermRepository bTermRepository;
    @Autowired
    private BSlRepository bSlRepository;

    @Override
    public BTerm selectOne(String businessId){
        return bTermRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BTerm> queryBTermsByPagination(PageUtil pageUtil, BTerm model) {
        Map param = new HashMap<String,Object>();
        return bTermRepository.queryBTerms(param);
    }

    @Override
    public List<BTerm> queryBTerms(BTerm model) {
        Map param = new HashMap<String,Object>();
        return bTermRepository.queryBTerms(param);
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel persist(BTerm model, String token){
        //验证时间区间是否重叠
        List<BTerm> bTerms = bTermRepository.queryBTermsByStartDateOrEndDate(model);
        if(bTerms.size()>0){
            throw new ServiceException(ResultServiceEnums.CAN_NOT_OVERLAP);
        }
        //验证名称是否重复
        if(bTermRepository.selectOne(new BTerm(null,model.getName()))!=null)
            throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
        int line = 0;
        model.setWeek(String.valueOf(Week.buildList(model.getStartDate(),model.getEndDate()).size() + 1));
        if(model.getBusinessId()!=null) {
            UserUtil.setCurrentMergeOperation(model);
            line = bTermRepository.updateByPrimaryKeySelective(model);
        }else {
            //验证编号是否重复
            if(bTermRepository.selectOne(new BTerm(model.getCode(),null))!=null)
                throw new ServiceException(ResultServiceEnums.TERM_EXISTS);
            Map<String,Object> map = Week.buildWeek(model.getStartDate(),model.getEndDate());
            model.setWeek(String.valueOf(((List<Week>) map.get("weeks")).size()));
            UserUtil.setCurrentPersistOperation(model);
            line = bTermRepository.insertSelective(model);
        }
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel removeBTerm(String id) {
        if(bSlRepository.selectslNumByTermId(id)>0) {
            throw new ServiceException(ResultServiceEnums.TERM_HAS_SL);
        }
        //验证是否是当前学期
        BTerm bTerm = bTermRepository.selectByPrimaryKey(id);
        if(bTerm.getCurrent().equals("1")){
            throw new ServiceException(ResultServiceEnums.CURRENT_TERM);
        }
        int line = bTermRepository.deleteByPrimaryKey(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel changeIS(String code, String token)
    {
        //这里没有直接使用  0  1 ，而是自定义了枚举类，来使用枚举类型
        BTerm bt=new BTerm();
        bt.setCode(code);
        List<BTerm> bTerm = bTermRepository.queryBtermByCode(code);
        //BTerm bTerm=bTermRepository.selectOne(bt);
        if(bTerm!=null)
        {
            if(bTerm.get(0).getIsStart().equals(BTermEnum.UN_START.getCode())) {
                bTerm.get(0).setIsStart(BTermEnum.START.getCode());
                //bTermRepository.updateUnStart(bTerm.get(0).getBusinessId());
            }
            else {
                bTerm.get(0).setIsStart(BTermEnum.UN_START.getCode());
            }
            //如果是用updateByPrimaryKey的话，那么在数据库中将没赋值的项 置空
            UserUtil.setCurrentMergeOperation(bTerm.get(0));
            int line = bTermRepository.updateByPrimaryKeySelective(bTerm.get(0));
            return new PersistModel(line);
        }
        else
        {
            throw new ServiceException(ResultServiceEnums.UPDATE_PASSWORD_ERROR);
        }
    }

    /**
     *
     * @param current 是否将当前学期放在最上面
     * @param arrange 是否是新建排课任务的学期下拉列表
     * @return
     */
    @Override
    public List<SelectModel> queryBTermList(String current, String arrange, String flag) {
        List<BTerm> bTerm;
        if(arrange!=null&&arrange.equals("1")){
            bTerm = bTermRepository.queryTermNotInArrange(current);
        }else{
            bTerm = bTermRepository.queryBTermList(current,flag);
        }

        ArrayList<SelectModel> comBoxModel = new ArrayList<SelectModel>();
        for(BTerm c : bTerm){
            SelectModel selectModel = new SelectModel();
            selectModel.setId(c.getCode());
            selectModel.setText(c.getName());
            comBoxModel.add(selectModel);
        }
        return comBoxModel;
    }

    @Override
    public PersistModel changeCurrent(String code) {
        //先把学期全部置为否
        bTermRepository.changeAllTermNotCurrent();
        //把所选的学期置为是
        int result = bTermRepository.changeCurrent(code);
        return new PersistModel(result);
    }

    @Override
    public Map<String, Object> getSchoolWeek(String termCode){
        Map<String,Object> map = new HashMap<>();
        BTerm bTerm = bTermRepository.queryCurrentTerm(termCode);
        List<SchoolCalendarWeek> list = new ArrayList<>();
        //list = Week.buildList(bTerm.getStartDate(),bTerm.getEndDate());
        list = SchoolCalendarWeek.buildList(bTerm.getStartDate(),bTerm.getEndDate());
        map.put("weekList",list);
        map.put("termName",bTerm.getName());
        return map;
    }

}
