package org.rcisoft.business.component;

import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.rcisoft.business.bcode.task.ClearLxcJob;
import org.rcisoft.core.component.RcVerify;
import org.rcisoft.core.util.TaskUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.scheduling.quartz.SchedulerFactoryBean;
import org.springframework.stereotype.Component;

import java.util.HashMap;

/**
 * Created by lcy on 18/1/20.
 *
 *  容器所有bean都被defined 之后触发
 */
@Slf4j
@Component
public class RcApplicationRunner implements ApplicationRunner {


    /*任务组名*/
    private static final String JOB_LXC_GROUP = "lxc_clear_job_group";
    /*触发器组名*/
    private static final String TRIGGER_LXC_GROUP = "lxc_clear_trigger_group";
    /*任务 identity*/
    private static final String IDENTITY_LXC_GROUP = "lxc_clear_job";

    @Autowired
    private SchedulerFactoryBean schedulerFactoryBean;

    @Autowired
    private RcVerify rcVerify;

    @Override
    public void run(ApplicationArguments args) throws Exception {
        //验证
        rcVerify.verify();
        JobDetail job = JobBuilder.newJob(ClearLxcJob.class).
                withIdentity(IDENTITY_LXC_GROUP, JOB_LXC_GROUP).
                usingJobData(new JobDataMap(new HashMap())).build();
        Trigger trigger = TriggerBuilder.newTrigger().forJob(job)
                .usingJobData(new JobDataMap(new HashMap()))
                .withIdentity(IDENTITY_LXC_GROUP)
                .startNow().build();
        // 触发器时间设定
        /*触发器*/
        TaskUtil.startSchedule(schedulerFactoryBean.getScheduler(),trigger,job);

    }
}
