package org.rcisoft.business.texaminfo.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.texaminfo.OptionsDto;
import org.rcisoft.business.tquestion.entity.TQuestion;
import org.rcisoft.business.tquestion.entity.TQuestionOptions;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Repository;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.exception.ServiceException;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;


import org.rcisoft.business.texaminfo.entity.TExamInfo;
import org.rcisoft.business.texaminfo.service.TExamInfoService;

import java.util.List;

/**
* Created by yangzhaojun on 2018-1-15 19:49:07.
*/
@RestController
@RequestMapping("/examInfo")
public class TExamInfoController extends PaginationController<TExamInfo> {

    @Autowired
    private TExamInfoService tExamInfoServiceImpl;


    @ApiOperation(value="添加", notes="添加")
    @ApiImplicitParams({@ApiImplicitParam(name = "chapterId", value = "chapterId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "ip", value = "ip(1-30)", required = false, dataType = "varchar")})
    @PostMapping("/add")
    public Result add(String chapterId, HttpServletRequest request) {
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        TExamInfo tExamInfo = new TExamInfo();
        UserUtil.setCurrentPersistOperation(tExamInfo);
        PersistModel data = tExamInfoServiceImpl.save(chapterId,userId,tExamInfo,request);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                chapterId);
    }

    @ApiOperation(value="逻辑删除", notes="逻辑删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = false, dataType = "varchar")})
    @DeleteMapping("/{id:\\d+}")
    public Result delete(@PathVariable String id) {
        TExamInfo tExamInfo = new TExamInfo();
        tExamInfo.setBusinessId(id);
        tExamInfo.setToken(getToken());
        PersistModel data = tExamInfoServiceImpl.remove(tExamInfo);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/{id:\\d+}")
    public Result update(@Valid TExamInfo tExamInfo, BindingResult bindingResult) {
        tExamInfo.setToken(getToken());
        PersistModel data = tExamInfoServiceImpl.merge(tExamInfo);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tExamInfo);
    }

    @ApiOperation(value="查看单 ", notes="查看单 ")
    @GetMapping("/{id:\\d+}")
    public Result detail(@PathVariable String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tExamInfoServiceImpl.findById(id));
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @GetMapping(value = "/queryTExamInfoByPagination")
    public GridModel listByPagination(TExamInfo tExamInfo) {
        tExamInfo.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        tExamInfoServiceImpl.findAllByPagination(getPaginationUtility(), tExamInfo);
        return getGridModelResponse();
    }



    @ApiOperation(value="学生完成练习", notes="完成练习")
    @ApiImplicitParams({@ApiImplicitParam(name = "choiceOptions", value = "单选题数组", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "multiSelectOptions", value = "多选题数组", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "judgmentOptions", value = "判断题数组", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "examInfoId", value = "考试信息表Id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterId", value = "章节ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar")})
    @PostMapping("/finishPractice")
    @PreAuthorize("hasRole('ROLE_1003')")
    public Result finishPractice(OptionsDto options) {
        PersistModel data = tExamInfoServiceImpl.finishPractice(options,UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                options);
    }
}
