package org.rcisoft.business.tpaper.service.impl;

import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.tpaper.dao.TPaperRepository;
import org.rcisoft.business.tpaper.entity.TPaper;
import org.rcisoft.business.tpaper.service.TPaperService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by yangzhaojun on 2018-1-15 19:49:07.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class TPaperServiceImpl implements TPaperService {

    @Autowired
    private TPaperRepository tPaperRepository;


    /**
    * 保存 tPaper
    * @param tPaper
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(TPaper tPaper){
        //增加操作
        int line = tPaperRepository.insertSelective(tPaper);
        log.info(UserUtil.getUserInfoProp(tPaper.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
            tPaper.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param tPaper
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(TPaper tPaper){
        int line = tPaperRepository.logicalDelete(tPaper);
        log.info(UserUtil.getUserInfoProp(tPaper.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        tPaper.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 tPaper
    * @param tPaper
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(TPaper tPaper){
        int line = tPaperRepository.updateByPrimaryKeySelective(tPaper);
        log.info(UserUtil.getUserInfoProp(tPaper.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            tPaper.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 tPaper
    * @param id
    * @return
    */
    public TPaper findById(String id){
        return tPaperRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 tPaper
    * @param tPaper
    * @return
    */
    public List<TPaper> findAllByPagination(PageUtil<TPaper> paginationUtility,
        TPaper tPaper){
        return tPaperRepository.queryTPapers(tPaper);
    }

}
