package org.rcisoft.business.tpaperchapter.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.tpaperchapter.entity.TPaperChapter;
import org.rcisoft.business.tpaperchapter.service.TPaperChapterService;

import java.util.List;

/**
* Created by yangzhaojun on 2018-1-15 19:49:07.
*/
@RestController
@RequestMapping("/tPaperChapter")
public class TPaperChapterController extends PaginationController<TPaperChapter> {

    @Autowired
    private TPaperChapterService tPaperChapterServiceImpl;


    @ApiOperation(value="添加练习", notes="添加练习")
    @ApiImplicitParams({@ApiImplicitParam(name = "pdesc", value = "说明", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课ID(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节id(1-50)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "poption", value = "1 随机 2 自主", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pscope", value = "1 全部 2 本节", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pcompose", value = "{'1':3,'2':0} 3道选择,2道判断(1-30)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "choiceIds", value = "选择题ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "judgmentIds", value = "判断题ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "multiSelectIds", value = "多选题ID", required = false, dataType = "varchar")})
    @PostMapping("/add")
    public Result add(TPaperChapter tPaperChapter) {
        PersistModel data = tPaperChapterServiceImpl.save(tPaperChapter);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tPaperChapter);
    }

    @ApiOperation(value="逻辑删除", notes="逻辑删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = false, dataType = "varchar")})
    @DeleteMapping("/{id:\\d+}")
    public Result delete(@PathVariable String id) {
        TPaperChapter tPaperChapter = new TPaperChapter();
        tPaperChapter.setBusinessId(id);
        tPaperChapter.setToken(getToken());
        PersistModel data = tPaperChapterServiceImpl.remove(tPaperChapter);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="修改练习", notes="修改练习")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pdesc", value = "说明", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "poption", value = "1 随机 2 自主", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pscope", value = "1 全部 2 本节", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pcompose", value = "{'1':3,'2':0} 3道选择,2道判断", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "choiceIds", value = "选择题ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "judgmentIds", value = "判断题ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "multiSelectIds", value = "多选题ID", required = false, dataType = "varchar")})
    @PostMapping("/update")
    public Result update(TPaperChapter tPaperChapter) {
        PersistModel data = tPaperChapterServiceImpl.merge(tPaperChapter);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            tPaperChapter);
    }

    @ApiOperation(value="查看单 ", notes="查看单 ")
    @GetMapping("/{id:\\d+}")
    public Result detail(@PathVariable String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            tPaperChapterServiceImpl.findById(id));
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @GetMapping(value = "/queryTPaperChapterByPagination")
    public GridModel listByPagination(TPaperChapter tPaperChapter) {
        tPaperChapter.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        tPaperChapterServiceImpl.findAllByPagination(getPaginationUtility(), tPaperChapter);
        return getGridModelResponse();
    }


    @ApiOperation(value="进入创建练习页面时执行此查询", notes="如果此节已有练习，将练习信息返回")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterId", value = "章节id", required = true, dataType = "varchar")})
    @GetMapping(value = "/queryTPaperChapterInfo")
    public Result queryTPaperChapterInfo(String slId, String chapterId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                tPaperChapterServiceImpl.queryTPaperChapterInfo(slId,chapterId));
    }
}
