package org.rcisoft.business.tpaperchapter.service.impl;

import com.google.gson.Gson;
import org.apache.commons.collections.map.HashedMap;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.texaminfo.dao.TExamInfoRepository;
import org.rcisoft.business.texaminfo.entity.TExamInfo;
import org.rcisoft.business.tpaper.dao.TPaperDetailRepository;
import org.rcisoft.business.tpaper.dao.TPaperRepository;
import org.rcisoft.business.tpaper.dao.TPaperSectionRepository;
import org.rcisoft.business.tpaper.entity.TPaper;
import org.rcisoft.business.tpaper.entity.TPaperDetail;
import org.rcisoft.business.tpaper.entity.TPaperSection;
import org.rcisoft.business.tpaperchapter.dao.TPaperChapterSectionRepository;
import org.rcisoft.business.tpaperchapter.dto.SectionIdDto;
import org.rcisoft.business.tpaperchapter.entity.TPaperChapterSection;
import org.rcisoft.business.tquestion.dao.TQuestionRepository;
import org.rcisoft.business.tquestion.entity.QuestionNumDto;
import org.rcisoft.business.tquestion.entity.TQuestion;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.tpaperchapter.dao.TPaperChapterRepository;
import org.rcisoft.business.tpaperchapter.entity.TPaperChapter;
import org.rcisoft.business.tpaperchapter.service.TPaperChapterService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by yangzhaojun on 2018-1-15 19:49:07.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class TPaperChapterServiceImpl implements TPaperChapterService {

    @Autowired
    private TPaperChapterRepository tPaperChapterRepository;

    @Autowired
    private TPaperRepository tPaperRepository;

    @Autowired
    private TPaperSectionRepository tPaperSectionRepository;

    @Autowired
    private TPaperChapterSectionRepository tPaperChapterSectionRepository;

    @Autowired
    private TPaperDetailRepository tPaperDetailRepository;

    @Autowired
    private BSlRepository bSlRepository;

    @Autowired
    private TQuestionRepository tQuestionRepository;

    @Autowired
    private TExamInfoRepository tExamInfoRepository;


    /**
    * 保存 tPaperChapter
    * @param tPaperChapter
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(TPaperChapter tPaperChapter){
        //增加操作
        UserUtil.setCurrentPersistOperation(tPaperChapter);
        int line = tPaperChapterRepository.insertSelective(tPaperChapter);

        this.persist(tPaperChapter);

        return new PersistModel(line);
    }

    private void persist(TPaperChapter tPaperChapter) {
        List<TPaperChapterSection> tPaperChapterSections = new ArrayList<>();
        Gson gson = new Gson();
        Map<String, Object> pcompose = new HashMap<String, Object>();
        pcompose = gson.fromJson(tPaperChapter.getPcompose(), pcompose.getClass());
        for(int i=0; i<pcompose.size();i++){
            String key = String.valueOf(i+1);
            //记录题型及数量
            TPaperChapterSection tPaperChapterSection = new TPaperChapterSection();
            tPaperChapterSection.setPcId(tPaperChapter.getBusinessId());
            tPaperChapterSection.setQType(key);
            tPaperChapterSection.setQNum(Double.valueOf(String.valueOf(pcompose.get(key))).intValue());
            tPaperChapterSections.add(tPaperChapterSection);
        }
        tPaperChapterSectionRepository.insertList(tPaperChapterSections);

        if(tPaperChapter.getPoption().equals("2")){
            //自主选题
            //1.添加试卷表信息
            TPaper tPaper = new TPaper();
            tPaper.setChapId(tPaperChapter.getChapId());
            tPaper.setPcId(tPaperChapter.getBusinessId());
            //tPaper.setStatus("1");
            tPaperRepository.insertSelective(tPaper);

            //2.添加大题表信息  记录题型及数量
            List<TPaperSection> tPaperSections = new ArrayList<>();

            //得到试卷ID
            TPaper pid = tPaperRepository.selectOne(tPaper);
            //数据转为map
            for(int i=0; i<pcompose.size();i++){
                String key = String.valueOf(i+1);
                TPaperSection tPaperSection = new TPaperSection();
                tPaperSection.setPId(String.valueOf(pid.getBusinessId()));
                tPaperSection.setQType(key);
                tPaperSection.setSorder(i+1);
                tPaperSections.add(tPaperSection);
            }
            //插入数据库
            tPaperSectionRepository.insertList(tPaperSections);

            //3.添加小题表信息
            List<TPaperDetail> tPaperDetails = new ArrayList<>();
            List<SectionIdDto> sectionIdDto = tPaperSectionRepository.querySectionIdByPid(String.valueOf(pid.getBusinessId()));
            if(sectionIdDto.size()>0){
                int i = 0;
                for(String questionId : tPaperChapter.getChoiceIds().split(",")){
                    if(questionId.equals(""))
                        continue;
                    TPaperDetail tPaperDetail = new TPaperDetail();
                    tPaperDetail.setPId(String.valueOf(pid.getBusinessId()));
                    tPaperDetail.setPsId(sectionIdDto.get(0).getChoiceId());
                    tPaperDetail.setQId(questionId);
                    tPaperDetail.setSorder(i);
                    tPaperDetails.add(tPaperDetail);
                    i++;

                }
                int j = 0;
                for(String questionId : tPaperChapter.getJudgmentIds().split(",")){
                    if(questionId.equals(""))
                        continue;
                    TPaperDetail tPaperDetail = new TPaperDetail();
                    tPaperDetail.setPId(String.valueOf(pid.getBusinessId()));
                    tPaperDetail.setPsId(sectionIdDto.get(0).getJudgmentId());
                    tPaperDetail.setQId(questionId);
                    tPaperDetail.setSorder(j);
                    tPaperDetails.add(tPaperDetail);
                    j++;
                }
                int k = 0;
                for(String questionId : tPaperChapter.getMultiSelectIds().split(",")){
                    if(questionId.equals(""))
                        continue;
                    TPaperDetail tPaperDetail = new TPaperDetail();
                    tPaperDetail.setPId(String.valueOf(pid.getBusinessId()));
                    tPaperDetail.setPsId(sectionIdDto.get(0).getMultiSelectId());
                    tPaperDetail.setQId(questionId);
                    tPaperDetail.setSorder(k);
                    tPaperDetails.add(tPaperDetail);
                    k++;
                }
                if(tPaperDetails.size()<1){
                    throw new ServiceException(ResultServiceEnums.PARAM_NO_QUESTION);
                }
                tPaperDetailRepository.insertList(tPaperDetails);
            }else{
                throw new ServiceException(ResultServiceEnums.INSERT_ERROR);
            }

        }else{
            //随机生成
            int plan_count = bSlRepository.queryStudentNumById(tPaperChapter.getSlId());
            Map<String,Object> map = new HashedMap();
            map.put("planId",tPaperChapter.getBusinessId());
            map.put("plan_type",Integer.parseInt(tPaperChapter.getPscope()));
            map.put("slId",tPaperChapter.getSlId());
            map.put("chid",tPaperChapter.getChapId());
            map.put("plan_count",plan_count);
            tPaperChapterRepository.logicPaperProcedure(map);
        }
    }

    /**
    * 逻辑删除
    * @param tPaperChapter
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(TPaperChapter tPaperChapter){
        UserUtil.setCurrentMergeOperation(tPaperChapter);
        tPaperChapter.setDeleted();
        int line = tPaperChapterRepository.logicalDelete(tPaperChapter);
        log.info(UserUtil.getUserInfoProp(tPaperChapter.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        tPaperChapter.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 tPaperChapter
    * @param tPaperChapter
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(TPaperChapter tPaperChapter){
        List<String> paperIds = tPaperRepository.queryIdByPcId(tPaperChapter.getBusinessId());
        for(String paperId : paperIds){
            TExamInfo tExamInfo = tExamInfoRepository.selectExamInfoByPid(paperId);
            if(tExamInfo!=null){
                throw new ServiceException(ResultServiceEnums.PAPER_IN_USE);
            }
            tPaperDetailRepository.delete(new TPaperDetail(paperId));//删除小题表信息
            tPaperSectionRepository.delete(new TPaperSection(paperId));//删除大题表信息
            tPaperRepository.delete(new TPaper(tPaperChapter.getBusinessId()));//删除试卷表信息
            tPaperChapterSectionRepository.delete(new TPaperChapterSection(tPaperChapter.getBusinessId()));//删除记录试题数量表信息
        }

        UserUtil.setCurrentMergeOperation(tPaperChapter);
        int line = tPaperChapterRepository.updateByPrimaryKeySelective(tPaperChapter);

        this.persist(tPaperChapter);

        return new PersistModel(line);
    }

    /**
    * 根据id查询 tPaperChapter
    * @param id
    * @return
    */
    public TPaperChapter findById(String id){
        return tPaperChapterRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 tPaperChapter
    * @param tPaperChapter
    * @return
    */
    public List<TPaperChapter> findAllByPagination(PageUtil<TPaperChapter> paginationUtility,
        TPaperChapter tPaperChapter){
        tPaperChapter.setStart();
        tPaperChapter.setNotDeleted();
        return tPaperChapterRepository.queryTPaperChapters(tPaperChapter);
    }

    @Override
    public TPaperChapter queryTPaperChapterInfo(String slId, String chapterId) {
        TPaperChapter tPaperChapter = new TPaperChapter();
        List<TPaperChapter> tPaperChapters = tPaperChapterRepository.queryTPaperChapterInfo(slId,chapterId);
        //查询全部的题型数量
        QuestionNumDto questionNumDto = tQuestionRepository.queryQuestionNum(slId,chapterId);
        if(questionNumDto==null){
            questionNumDto = new QuestionNumDto();
            questionNumDto.setChapChoiceNum("0");
            questionNumDto.setChapMultiSelectNum("0");
            questionNumDto.setChapJudgmentNum("0");
            questionNumDto.setSlChoiceNum("0");
            questionNumDto.setSlMultiSelectNum("0");
            questionNumDto.setSlJudgmentNum("0");
        }
        if(tPaperChapters.size()>0){
            tPaperChapter = tPaperChapters.get(0);
            //查询已添加的题型数量
            QuestionNumDto nowNum = tPaperChapterSectionRepository.queryNumByPcId(tPaperChapter.getBusinessId());
            questionNumDto.setNumOfChoice(nowNum.getNumOfChoice());
            questionNumDto.setNumOfMultiSelect(nowNum.getNumOfMultiSelect());
            questionNumDto.setNumOfJudgment(nowNum.getNumOfJudgment());
        }else{
            questionNumDto.setNumOfChoice("0");
            questionNumDto.setNumOfMultiSelect("0");
            questionNumDto.setNumOfJudgment("0");
        }
        tPaperChapter.setQuestionNumDto(questionNumDto);
        return tPaperChapter;
    }

}
