package org.rcisoft.business.tquestion.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.tquestion.dto.QuestionListAndNumDto;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;

import javax.servlet.http.HttpServletRequest;


import org.rcisoft.business.tquestion.entity.TQuestion;
import org.rcisoft.business.tquestion.service.TQuestionService;

/**
* Created by yangzhaojun on 2018-1-15 19:49:07.
*/
@RestController
@RequestMapping("/tQuestion")
public class TQuestionController extends PaginationController<TQuestion> {

    @Autowired
    private TQuestionService tQuestionServiceImpl;


    @ApiOperation(value="新增试题", notes="新增试题")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "课程ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "qtype", value = "1 选择题 2 多选题 3 判断题", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "qtitle", value = "题干(1-180)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "qdesc", value = "解析(1-250)", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "qanswer", value = "单选从0开始；判断：0错误 1正确；多选 0,1,2(1-15)", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "questionOptions", value = "选项", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "represent", value = "列描述（1-40）", required = true, dataType = "varchar")})
    @PostMapping("/add")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result add(TQuestion tQuestion) {
        PersistModel data = tQuestionServiceImpl.save(tQuestion);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            tQuestion);
    }


    @ApiOperation(value="删除题目", notes="根据ID删除一个题")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping("/remove")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result delete(String id) {
        return Result.builder(new PersistModel(tQuestionServiceImpl.remove(id,getToken())),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="批量删除", notes="根据ID批量删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "idInfoList", value = "businessId数组", required = true, dataType = "varchar")})
    @PostMapping("/removeByIds")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result deletes( String idInfoList ) {
        PersistModel data = new PersistModel(tQuestionServiceImpl.removeByIds(idInfoList));
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                idInfoList);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "章ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "课程ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "qtype", value = "1 选择题 2 多选题 3 判断题", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "qtitle", value = "题干", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "qdesc", value = "解析", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "qanswer", value = "单选从0开始；判断：0错误 1正确；多选 0,1,2", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "questionOptions", value = "选项", required = false, dataType = "varchar")})
    @PostMapping("/update")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result update(TQuestion tQuestion) {
        PersistModel data = tQuestionServiceImpl.merge(tQuestion);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            tQuestion);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @GetMapping("/one")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result detail(@PathVariable String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            tQuestionServiceImpl.findById(id));
    }

    @ApiOperation(value="根据条件分页查询", notes="根据URL中的参数分页查询")
    @ApiImplicitParams({@ApiImplicitParam(name = "qtype" , value = "题目类型" , required = false , dataType = "varchar"),
            @ApiImplicitParam(name = "qtitle" , value = "关键字" , required = false , dataType = "varchar"),
            @ApiImplicitParam(name = "chapId" , value = "章节Id" , required = false , dataType = "varchar"),
            @ApiImplicitParam(name = "slId" , value = "课程ID" , required = true , dataType = "varchar")})
    @GetMapping(value = "/queryTQuestionByPagination")
    @PreAuthorize("hasRole('ROLE_1002')")
    public GridModel listByPagination(TQuestion tQuestion) {
        //tQuestion.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        tQuestionServiceImpl.findAllByPagination(getPaginationUtility(), tQuestion);
        return getGridModelResponse();
    }

    @ApiOperation(value="根据条件查询(不分页)", notes="根据URL中的参数查询全部")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "课程ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapId", value = "章节ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pcompose" , value = "1全部   2本节" , required = true , dataType = "varchar"),
            @ApiImplicitParam(name = "composeBeforeUpdate" , value = "是否查询inUse 1是 2否" , required = false , dataType = "varchar"),
            @ApiImplicitParam(name = "stateBeforeUpdate" , value = "修改之前的状态 1 随机 2 自主" , required = false , dataType = "varchar"),
            @ApiImplicitParam(name = "qtype", value = "题型", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "qtitle", value = "题干关键字", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryTQuestions")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result queryTQuestion(TQuestion tQuestion) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                tQuestionServiceImpl.queryTQuestions(tQuestion));
    }


    @ApiOperation(value="根据开课ID查询全部试题", notes="根据开课ID查询全部试题")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "课程ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "qtype", value = "题型", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "qtitle", value = "题干关键字", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryTQuestionBySlId")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result queryTQuestionBySlId(TQuestion tQuestion) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                tQuestionServiceImpl.queryTQuestionBySlId(tQuestion));
    }


    @ApiOperation(value="题目详情", notes="根据题目ID获取题目详情")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/queryTQestionById")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result queryTQestionById(String BusinessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                tQuestionServiceImpl.queryTQestionById(BusinessId));
    }

    @ApiOperation(value="学生进入练习页面查询试题", notes="学生进入练习页面查询试题")
    @GetMapping(value = "/queryQuestionsByChap_IdAndQtype")
    public Result queryBChapters(String chapterId, HttpServletRequest request) {
        QuestionListAndNumDto questionListAndNum = tQuestionServiceImpl.queryQuestionsByChap_IdAndQtype(chapterId,UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID),request);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                questionListAndNum);
    }



}
