package org.rcisoft.core.aop;

import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.rcisoft.common.controller.ValidatedController;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.validate.ValidatedResult;
import org.springframework.validation.BindingResult;

/**
 * Created by lcy on 18/1/21.
 */
@Aspect
@Slf4j
public class EntityParamAspect {


    /**
     * 环绕aop   service.*ByPagination    分页拦截
     * @param proceedingJoinPoint
     * @return
     */
    @Around("execution(* org.rcisoft..*.controller.*Controller.add*(..))")
    public Object beforeAddValidate(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {

        return dealWithParameterValidate(proceedingJoinPoint);
    }

    @Around("execution(* org.rcisoft..*.controller.*Controller.update*(..))")
    public Object beforeUpdateValidate(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        return dealWithParameterValidate(proceedingJoinPoint);
    }

    /**
     * 处理  增加/修改  方法参数验证
     * @param proceedingJoinPoint
     * @return
     */
    private Object dealWithParameterValidate(ProceedingJoinPoint proceedingJoinPoint) throws Throwable{
        try {
            /*验证 长度必须大于1*/
            if(proceedingJoinPoint.getArgs().length < 2)
                return proceedingJoinPoint.proceed();
            /*BindingResult 必须写在第二个参数里*/
            Object bindResult = proceedingJoinPoint.getArgs()[1];
            if(!(bindResult instanceof BindingResult))
                return proceedingJoinPoint.proceed();
            BindingResult br = (BindingResult)bindResult;
            /*有无异常*/
            if(ValidatedController.hasErrorsByStatic(br))
                throw new ServiceException(4004, ValidatedResult.builder(br).getValidatedMessage().getResponse());
                /*

                不能直接返回
                return Result.builder(new PersistModel(0),
                                        MessageConstant.MESSAGE_ALERT_SUCCESS,
                        MessageConstant.MESSAGE_ALERT_INFO_INVALID,
                        ValidatedResult.builder(br));

                com.fasterxml.jackson.databind.JsonMappingException: No serializer found for class org.springframework.validation.DefaultMessageCodesResolver
                 and no properties discovered to create BeanSerializer (to avoid exception, disable SerializationFeature.FAIL_ON_EMPTY_BEANS)
                 (through reference chain: org.rcisoft.core.result.Result["data"]->org.rcisoft.core.validate.ValidatedResult["br"]->
                 org.springframework.validation.BeanPropertyBindingResult["messageCodesResolver"])

                 getOutputStream() has already been called for this response

                */
            else
                return proceedingJoinPoint.proceed();
        } catch (Throwable var5) {
            log.error(var5.getMessage());
            throw var5;
        }
    }


}
