package org.rcisoft.core.command;

import ch.ethz.ssh2.Connection;
import ch.ethz.ssh2.Session;
import ch.ethz.ssh2.StreamGobbler;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.*;

/**
 * Created by lcy on 18/1/11.
 */
@Data
@NoArgsConstructor
@Slf4j
@Component
public class RemoteExecuteCommand {
    //字符编码默认是utf-8
    private static String  DEFAULT_CHART="UTF-8";
    private Connection conn;

    @Value("${serverLxc.ip}")
    private String ip;
    @Value("${serverLxc.username}")
    private String userName;
    @Value("${serverLxc.password}")
    private String userPwd;



    /**
     * 远程登录linux的主机
     * @author Ickes
     * @since  V0.1
     * @return
     *      登录成功返回true，否则返回false
     */
    private Boolean login(){
        boolean flg=false;
        try {
            conn = new Connection(ip);
            conn.connect();//连接
            flg=conn.authenticateWithPassword(userName, userPwd);//认证
        } catch (IOException e) {
            e.printStackTrace();
        }
        return flg;
    }
    /**
     * @author Ickes
     * 远程执行shll脚本或者命令
     * @param cmd
     *      即将执行的命令
     * @return
     *      命令执行完后返回的结果值
     * @since V0.1
     */
    public String execute(String cmd,int execTime){
        log.info("执行:{}",cmd);
        String result="";
        try {
            if(login()){  //这里偶尔会由于网络原因抛出异常
                Session session= conn.openSession();//打开一个会话
                session.execCommand(cmd);//执行命令
                result=processStdout(session.getStdout(),DEFAULT_CHART);
                //如果为得到标准输出为空，说明脚本执行出错了
                if(StringUtils.isBlank(result)){
                    result=processStdout(session.getStderr(),DEFAULT_CHART);
                }
                session.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
            if(execTime<10) {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e1) {
                    e1.printStackTrace();
                }
                execTime++;
                return execute(cmd,execTime);   //异常了就循环执行方法，直到正常了才返回
            }else{
                //throw new BusinessException(ResultEnums.NETWORK_ERROR);
            }
        }finally {
            conn.close();
        }
        log.info("返回:{}",result);
        return result;
    }

    /**
     * @author Ickes
     * 远程执行shll脚本或者命令
     * @param cmd
     *      即将执行的命令
     * @param location
     *      即将执行命令的所在位置
     * @return
     *      命令执行完后返回的结果值
     * @since V0.1
     */
    public String execute(String cmd,String location){
        return execute("cd "+location+" && "+cmd,1);
    }


    /**
     * @author Ickes
     * 远程执行shll脚本或者命令
     * @param cmd
     *      即将执行的命令
     * @return
     *      命令执行成功后返回的结果值，如果命令执行失败，返回空字符串，不是null
     * @since V0.1
     */
    public String executeSuccess(String cmd){
        String result="";
        try {
            if(login()){
                Session session= conn.openSession();//打开一个会话
                session.execCommand(cmd);//执行命令
                result=processStdout(session.getStdout(),DEFAULT_CHART);
                session.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }finally {
            conn.close();
        }
        return result;
    }

    /**
     * 解析脚本执行返回的结果集
     * @author Ickes
     * @param in 输入流对象
     * @param charset 编码
     * @since V0.1
     * @return
     *       以纯文本的格式返回
     */
    private String processStdout(InputStream in, String charset){
        InputStream   stdout = new StreamGobbler(in);
        StringBuffer buffer = new StringBuffer();;
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(stdout,charset));
            String line=null;
            while((line=br.readLine()) != null){
                buffer.append(line+"\n");
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return buffer.toString();
    }

    public static void main(String[] args) {
        //RemoteExecuteCommand remoteExecuteCommand = new RemoteExecuteCommand("106.2.3.134","root","91isoft_xunda");
        String branch = "master";
        String localUrl = "/working/dockervolume/chedir/project/p_201809004";
        /*String branchs = remoteExecuteCommand.execute(GitCommands.GIT_BRANCH,localUrl);
        String[] branchArr = branchs.split("\n");
        for(String item:branchArr){
            if(item.startsWith("*")){
                System.out.println(item+" "+branch);
                System.out.println(item.replace("* ",""));
                System.out.println(item.replace("* ","").equals(branch));
                if(!item.replace("* ","").equals(branch)){
                    break;
                }
                return;
            }
        }
        System.out.println("aaaa");*/
        //remoteExecuteCommand.execute(GitCommands.getGitCheckoutCommand(branch),localUrl);
    }

}
