package org.rcisoft.core.controller;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.component.RcVerify;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.model.UserAuthDTO;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.service.AuthenticationService;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by lcy on 17/11/24.
 */
@RestController
@RequestMapping("/auth")
@Slf4j
public class AuthenticationController {

    @Value("${jwt.header}")
    private String tokenHeader;

    @Autowired
    private AuthenticationService authenticationServiceImpl;

    @Autowired
    private RcVerify rcVerify;

    /**
     * 登录
     * @param username
     * @param password
     * @return
     */
    @PostMapping(value = "${jwt.route.authentication.path}")
    public Result login(@RequestParam("username")String username,
                        @RequestParam("password")String password,
                        @RequestParam("userType")String userType){
        final UserAuthDTO userAuthDTO = authenticationServiceImpl.login(username, password, userType);
        //Result result = ResultGenerator.genSuccessResult(token);
        return Result.builder(new PersistModel(1),
                                MessageConstant.MESSAGE_ALERT_SUCCESS,
                                MessageConstant.MESSAGE_ALERT_ERROR,
                                userAuthDTO);
    }

    @PostMapping(value = "lk")
    public Result lk(){
        String mess = "fai";
        try {
            if(rcVerify.verify())
                mess = "true";
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                mess);
    }

    /**
     * 注册
     * @param addedUser
     * @return
     */
    @PostMapping(value = "${jwt.route.authentication.register}")
    public Result register(SysUser addedUser ,String roleId){
        authenticationServiceImpl.register(addedUser,roleId);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                null);
    }

    @GetMapping(value = "${jwt.route.authentication.refresh}")
    public ResponseEntity<?> refreshAndGetAuthenticationToken(
            HttpServletRequest request){
        String token = request.getHeader(tokenHeader);
        String refreshedToken = authenticationServiceImpl.refresh(token);
        if(refreshedToken == null) {
            return ResponseEntity.badRequest().body(null);
        } else {
            return ResponseEntity.ok(refreshedToken);
        }
    }


}
