package org.rcisoft.core.lk;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.bean.RcLkConfigBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import zlicense.de.schlichtherle.license.*;

import java.io.File;
import java.io.IOException;
import java.util.prefs.Preferences;

/**
 * Created by lcy on 18/3/19.
 */
@Slf4j
@Component
public class VerifyLi {

    @Autowired
    private Global global;

    //common param
    private static String PUBLICALIAS = "";
    private static String STOREPWD = "";
    private static String SUBJECT = "";
    private static String licPath = "";
    private static String pubPath = "";

    private void setParam(RcLkConfigBean lk) throws IOException {
        PUBLICALIAS = lk.getPublicalias();
        STOREPWD = lk.getStorePwd();
        SUBJECT = lk.getSubject();
        licPath = lk.getLicPath();
        pubPath = lk.getPubPath();
    }

    public boolean verify(RcLkConfigBean lk) throws Exception  {
        this.setParam(lk);
        LicenseManager licenseManager = LkManagerHolder
                .getLicenseManager(this.initLicenseParams());
        // install license file
        try {
            licenseManager.install(new File(global.getBASE_UPLOAD_SERVER_LOCATION() + licPath));
            log.info("License file instal successfully!");
        } catch (Exception e) {
            log.error("License file instal failure");
            //throw  e ;
            return false;
        }
        // verify license file
        try {
            licenseManager.verify();
            log.info("License file instal successfully!");
        } catch (Exception e) {
            log.error("License file verify failure");
            //throw  e ;
            return false;
        }
        return true;
    }

    private LicenseParam initLicenseParams() {
        Preferences preference = Preferences
                .userNodeForPackage(VerifyLi.class);
        CipherParam cipherParam = new DefaultCipherParam(STOREPWD);
        KeyStoreParam privateStoreParam = new DefaultKeyStoreParam(
                VerifyLi.class, global.getBASE_UPLOAD_SERVER_LOCATION() + pubPath, PUBLICALIAS, STOREPWD, null);
        LicenseParam licenseParams = new DefaultLicenseParam(SUBJECT,
                preference, privateStoreParam, cipherParam);
        return licenseParams;
    }
}
