package org.rcisoft.core.model;

import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.dlsw.Column;
import org.rcisoft.core.model.dlsw.Condition;
import org.rcisoft.core.model.dlsw.Sort;

import java.util.List;
import java.util.Map;

/**
 * service dlsw grid
 * Created by lcy on 18/3/13.
 */
@Data
public class DlswPageGridModel implements GridModel {

    private static final long serialVersionUID = -13317137312873392L;

    /**
     * 是否出错
     */
    @JsonProperty(value = "isSuccess")
    private boolean isSuccess;

    /**
     * 每页显示条数
     */
    private int pageSize;

    /**
     * 开始记录数
     */
    private int startRecord;

    /**
     * 当前页数
     */
    private int nowPage;

    /**
     * 记录总数
     */
    private long recordCount;

    /**
     * 总页数
     */
    private int pageCount;

    /**
     * 参数列表
     */
    private Map<String, Object> parameters;

    /**
     * 快速查询参数列表
     */
    private Map<String, Object> fastQueryParameters;

    /**
     * 高级查询列表
     */
    private List<Condition> advanceQueryConditions;

    /**
     * 高级排序列表
     */
    private List<Sort> advanceQuerySorts;

    /**
     * 显示数据集
     */
    //private List<Map<String, Object>> exhibitDatas;
    private List<Object> exhibitDatas;

    /**
     * 是否导出：1-是，0-否
     */
    @JsonProperty(value = "isExport")
    private boolean isExport;

    /**
     * 导出类型，支持excel、pdf、txt、cvs
     */
    private String exportType;

    /**
     * 导出文件名
     */
    private String exportFileName;

    /**
     * 导出列
     */
    private List<Column> exportColumns;

    /**
     * 全部数据导出
     */
    private boolean exportAllData;

    /**
     * 导出数据是否已被加工
     */
    private boolean exportDataIsProcessed;

    /**
     * 导出数据
     */
    private List<Map<String, Object>> exportDatas;

    /**
     * 复制数据
     * @param paginationUtility
     */
    public void setPageData(PageUtil paginationUtility) {
        this.exhibitDatas = paginationUtility.getList();
        this.isSuccess = true;
        this.nowPage = paginationUtility.getPageNum();
        this.pageSize = paginationUtility.getPageSize();
        this.pageCount = paginationUtility.getPages();
        this.recordCount = paginationUtility.getTotal();
        this.startRecord = paginationUtility.getStartRow();
    }
}
