package org.rcisoft.core.service.impl;

import org.rcisoft.business.bcompany.dao.BCompanyRepository;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.UserAuthDTO;
import org.rcisoft.core.result.ResultExceptionEnum;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.service.AuthenticationService;
import org.rcisoft.core.util.JwtUtil;
import org.rcisoft.sys.menu.dao.SysMenuRepository;
import org.rcisoft.sys.menu.entity.MenuDto;
import org.rcisoft.sys.role.dao.SysRoleRepository;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.entity.UserRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;

/**
 * Created by lcy on 17/11/24.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class AuthenticationServiceImpl implements AuthenticationService {

    @Autowired
    private AuthenticationManager authenticationManager;
    @Autowired
    private UserDetailsService userDetailsService;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private SysRoleRepository sysRoleRepository;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private SysMenuRepository sysMenuRepository;
    @Autowired
    private BCompanyRepository bCompanyRepository;

    @Autowired
    private Global global;



    @Value("${jwt.tokenHead}")
    private String tokenHead;

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public void register(SysUser userToAdd , String roleId) {
        Example example = new Example(SysUser.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("loginName", userToAdd.getLoginName());
        List<SysUser> sysUser = sysUserMapper.selectByExample(example);
        if (sysUser.size() > 0)
            throw new ServiceException(ResultExceptionEnum.USER_EXISTS);
        userToAdd.setPassword(passwordEncoder.encode(userToAdd.getPassword()));
        userToAdd.setCommonBusinessId();
        int result = sysUserMapper.insertSelective(userToAdd);
        UserRole userRole = new UserRole(userToAdd.getBusinessId(),roleId);
        userRole.setCommonBusinessId();
        sysUserMapper.insertUserRole(userRole);
        if(result==0)
            throw new ServiceException(ResultExceptionEnum.SAVE_ERROR);
    }

    @Override
    public UserAuthDTO login(String username, String password, String userType) {
        UsernamePasswordAuthenticationToken upToken = new UsernamePasswordAuthenticationToken(username, password);
        /*进入到 UserDetailsService(JwtUserDetailServiceImpl)  loadUserByUsername 方法*/

        UserAuthDTO userAuthDTO = new UserAuthDTO();
        final Authentication authentication = authenticationManager.authenticate(upToken);
        UserDetails userDetails = (UserDetails)authentication.getPrincipal();

        List<SysUser> sysUserList = sysUserMapper.queryUserByName(username);
        SysUser sysUser = sysUserList.get(0);

        List<SysRole> role = sysRoleRepository.queryCodeByUsername(username);
        int rolePeration = 0;
        if(role.size()!=0){
            //判断用户有没有登陆运维平台的权限
            //rolePeration = sysRoleRepository.queryRoleByRoleCode(role.get(0).getCode(),"19");//19代表的是“登录运维平台权限”的ID

            rolePeration = sysRoleRepository.queryRoleByRoleCode("10",role.get(0).getBusinessId());
            //如果用户请求登录后台，并且但是其并没有任何后台权限，抛出异常
            if (userType.equals("1")&&rolePeration == 0)
                throw new ServiceException(ResultServiceEnums.ROLE_ERROR);
            else if(userType.equals("0")&&
                    !global.getTeacherCode().equals(role.get(0).getCode())&&
                    !global.getStudentCode().equals(role.get(0).getCode()))
                //代表请求前台，教师和学生可以请求，管理员不可以
                throw new ServiceException(ResultServiceEnums.ROLE_ERROR);
        }else
            throw new ServiceException(ResultServiceEnums.ROLE_ERROR);

        //查询菜单权限
        List<MenuDto> menus = sysMenuRepository.queryMeunsByRoleId(role.get(0).getBusinessId());
        final String token = JwtUtil.generateToken(userDetails);
        SecurityContextHolder.getContext().setAuthentication(authentication);

        String agencyCode = "";
        //请求登陆运维平台并且拥有登录运维平台权限并且不是Admin的用户，查询他的所属教学单位
        if (userType.equals("1")&&rolePeration!=0&&!global.getAdminCode().equals(role.get(0).getCode())){
            //查询教学单位code
            agencyCode = bCompanyRepository.queryAgencyCodeByCode(username);
        }
        userAuthDTO.setUserId(sysUser.getBusinessId());
        userAuthDTO.setName(sysUser.getName());
        userAuthDTO.setHeadPic(sysUser.getHeadPic());
        userAuthDTO.setRoleCode(role.get(0).getCode());
        userAuthDTO.setToken(token);
        userAuthDTO.setMenus(menus);
        userAuthDTO.setAgencyCode(agencyCode);
        return userAuthDTO;
    }

    @Override
    public String refresh(String oldToken) {
        return null;
    }
}
