package org.rcisoft.core.service.impl;

import org.apache.commons.collections4.CollectionUtils;
import org.rcisoft.core.constant.FlagStatus;
import org.rcisoft.core.model.JwtUser;
import org.rcisoft.core.util.JwtUserFactory;
import org.rcisoft.sys.role.dao.SysRoleRepository;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

/**
 * Created by lcy on 17/11/21.
 */
@Service
public class JwtUserDetailServiceImpl implements UserDetailsService {

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysRoleRepository sysRoleRepository;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        //调用持久层从数据库获取用户信息

        List<SysUser> sysUserList = sysUserMapper.queryUserByName(username);

        if (sysUserList == null || sysUserList.size() == 0)
            throw new UsernameNotFoundException("用户名不存在");
        SysUser sysUser = sysUserList.get(0);
        /*根据用户查询用户权限*/
        List<SysRole> roles = sysRoleRepository.findRolesByUserId(sysUser.getBusinessId());
        if (CollectionUtils.isEmpty(roles))
            roles = Collections.emptyList();
        sysUser.setRoleList(roles);
        JwtUser user = JwtUserFactory.create(sysUser);
        /*停用*/
        if(FlagStatus.ABNORMAL.getStatus().equals(sysUser.getFlag()))
            user.setEnable(false);
        return user;
    }
}
