package org.rcisoft.core.util;

import java.io.*;

public class CompileUtil {
	/**
     * @param code 代码
     * @param fileName 文件名
     * @param fileUrl  文件所在目录
     * @return
     */
    public static String executeJavaCode(String code,String fileName,String fileUrl) throws Exception {
    	writeCodeToFile(code,fileName,fileUrl);//写文件
        File file = new File(fileUrl+fileName+".java");//java文件
        //等待文件写完
        while(!file.exists()) {
            Thread.sleep(10);
        }
        File dir = new File(fileUrl);
        String os = System.getProperty("os.name");
		String java,java_class;
		//判断操作系统
		if (os.toLowerCase().startsWith("win")) {
			java = "c:\\Windows\\System32\\cmd.exe /c javac -encoding utf-8 "+fileName+".java";
            java_class = "c:\\Windows\\System32\\cmd.exe /c java "+fileName;
		} else {
            java = "javac -encoding utf-8 "+fileName+".java";
            java_class = "java "+fileName;
		}

        Runtime runJavac = Runtime.getRuntime();
        String result="";
        try {
            //先编译java文件  java文件编译成功时不会有信息报出，所以此处只取错误信息
            StringBuilder message = new StringBuilder();
            Process javac = runJavac.exec(java, null, dir);
            InputStreamReader error = new InputStreamReader(javac.getErrorStream(),"GBK");
            LineNumberReader errorInput = new LineNumberReader(error);
            boolean firstLine = true;
            String line = "";

            //取出输出结果
            while ((line = errorInput.readLine()) != null) {
                if(!firstLine){
                    message.append(System.getProperty("line.separator"));
                }else{
                    firstLine = false;
                }
                message.append(line);
            }
            message.append(line);
            error.close();
            errorInput.close();
            result = message.toString();


            if(result.equals("null")) {
                //java文件编译成功，开始编译class文件
                StringBuilder classMessage = new StringBuilder();
                firstLine = true;
                boolean wrong = true;
                String classUrl = fileUrl + fileName + ".class";
                File classFile = new File(classUrl);
                while (!classFile.exists()) {
                    Thread.sleep(10);
                }
                Runtime runClass = Runtime.getRuntime();
                Process Class = runClass.exec(java_class, null, dir);
                InputStreamReader successIsr = new InputStreamReader(Class.getInputStream(), "GBK");//成功信息
                InputStreamReader errorIsr = new InputStreamReader(Class.getErrorStream(), "GBK");//失败信息
                LineNumberReader successLnr = new LineNumberReader(successIsr);
                LineNumberReader errorLnr = new LineNumberReader(errorIsr);
                while ((line = successLnr.readLine()) != null) {
                    wrong = false;
                    if (!firstLine) {
                        classMessage.append(System.getProperty("line.separator"));
                    } else {
                        firstLine = false;
                    }
                    classMessage.append(line);
                }
                if(wrong){
                    while ((line = errorLnr.readLine()) != null) {
                        if (!firstLine) {
                            classMessage.append(System.getProperty("line.separator"));
                        } else {
                            firstLine = false;
                        }
                        classMessage.append(line);
                    }
                }
                result = classMessage.toString();
                successLnr.close();
                successIsr.close();
                errorLnr.close();
                errorIsr.close();
                classFile.delete();
                Class.waitFor();
                Class.destroy();
            }
            file.delete();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    public static void writeCodeToFile(String code,String fileName,String fileUrl) throws Exception{
        File file = new File(fileUrl+fileName+".java");
        if(!file.getParentFile().exists()){
            file.getParentFile().mkdirs();
        }

        Writer out = new FileWriter(file);
        out.write(code);
        out.close();

    }
}
