package org.rcisoft.core.util;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.util.CellRangeAddress;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by gaowenfeng on 2017/7/12.
 * excel导入导出工具类
 */
public class ExcelUtil {
    static DecimalFormat df = new DecimalFormat("0");  //动态转换

    /**
     * 获取excel模板信息
     * @param hwb 源数据
     * @param headers  excel模板头
     * @param allowEmpty 是否允许单元格有空数据
     * @return  整理好的ArrayList<String[]>
     */
    public static ArrayList<String[]> importExcel(HSSFWorkbook hwb, String[] headers, boolean allowEmpty , int headNum){
        ArrayList<String[]> values = new ArrayList<>();
        for (int i = 0; i < hwb.getNumberOfSheets(); i++) {
            HSSFSheet hs = hwb.getSheetAt(i);
            if (hs == null)
                continue;
            else {
                int firstrownum = hs.getFirstRowNum();  //第一行
                int lastrownum = hs.getLastRowNum();    //最后一行
                List<Integer> emptyColList=new ArrayList<>();
                for (int j = firstrownum + headNum; j <= lastrownum; j++) {
                    HSSFRow hr = hs.getRow(j);
                    if(hr==null||hr.getLastCellNum()<0){
                        continue;
                    }
                    int firstcolumnnum = hr.getFirstCellNum();
                    int lastcolumnnum = hr.getLastCellNum();
                    String[] value = new String[lastcolumnnum];
                    boolean hasEmpty = true;
                    //修改过！！！！！！！！！！！！
                    for (int k = 0; k < lastcolumnnum; k++) {
                        HSSFCell hc = hr.getCell(k);
                        if(hc == null){
                            value[k] = "";
                            continue;
                        }
                        if (j==firstrownum + headNum) {
                            //判断表头那一列为空数据并记录。
                            if(parseDB(hc).trim().equals("")) {
                                emptyColList.add(-1);
                            }else{
                                emptyColList.add(k);
                            }
                        }
                        else {
                            if(allowEmpty){//允许单元格为空，但不允许整行为空时
                                //hc != null &&
                                if (hc != null && !parseDB(hc).trim().equals("")&&emptyColList.get(k)!=-1&&hasEmpty) {
                                    //只要一行中有一个值不为空，即为正确数据
                                    hasEmpty = false;

                                }
                            }else{//不允许单元格为空，只要一行中有一个值为空，即为错误数据
                                hasEmpty = false;
                                //hc != null &&
                                if (hc != null && parseDB(hc) != null && parseDB(hc).trim().equals("")&&emptyColList.get(k)!=-1) {
                                    //在第j行第k列有空数据，且该列表头不是空的，则舍弃本行数据。
                                    hasEmpty = true;

                                    break;
                                }
                            }

                        }
                        //如果此单元格为空则插入""
                        if(hc == null){
                            value[k] = "";
                        }else {
                            value[k] = parseDB(hc);
                        }

                                //parseDB(hc.getCellType(), hc);
                    }
                    if(j==firstrownum + headNum){
                        for(int h=0;h<headers.length;h++){
                            if(!value[h].trim().equals(headers[h]))
                                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_HEADER_ERROR);
                        }
                    }else{
                        if (!hasEmpty) {
                            values.add(value);
                        }
                    }
                }
            }
        }
        return values;
    }

//    public static boolean isRepeat(List<String[]> totalValue,String[] value){
//        for(int i = 0 ; i < totalValue.size() ; i++){
//            for(int j = 0 ; j < value.length ; j++){
//                if(((String)value[j]).equals((String)totalValue.get(i)[j])){
//                    return true;
//                }
//            }
//        }
//    }

    /**
     * 重复数据检查
     * @param repeatCode
     * @return
     */
    public static String checkRepeat(List<String> repeatCode){
        StringBuilder repeatMessage = new StringBuilder();
        if(repeatCode.size()>0){
            repeatMessage.append("以下编号的数据导入重复:");
            for(int i=0;i<repeatCode.size();i++){
                if(!repeatCode.get(i).equals("")&&repeatCode.get(i)!=null) {
                    if (i == 0)
                        repeatMessage.append(repeatCode.get(i));
                    else
                        repeatMessage.append(",").append(repeatCode.get(i));
                }
            }
        }else {
            repeatMessage.append("全部数据已经导入");
        }
        return repeatMessage.toString();
    }

    public static ArrayList<String[]> importExcelv2(HSSFWorkbook hwb, String[] headers, int headNum){
        String result = "";
        ArrayList<String[]> values = new ArrayList<>();
        for (int i = 0; i < hwb.getNumberOfSheets(); i++) {
            HSSFSheet hs = hwb.getSheetAt(i);
            if (hs == null)
                continue;
            else {
                List<CellRangeAddress> cellRangeAddressList = hs.getMergedRegions();
                int firstrownum = hs.getFirstRowNum();  //第一行
                int lastrownum = hs.getLastRowNum();    //最后一行
                for (int j = firstrownum + headNum; j <= lastrownum; j++) {
                    HSSFRow hr = hs.getRow(j);
                    if(hr==null||hr.getLastCellNum()<0){
                        continue;
                    }
                    int firstcolumnnum = hr.getFirstCellNum();
                    int lastcolumnnum = hr.getLastCellNum();
                    String[] value = new String[lastcolumnnum + 6];
                    String lessonValue = null;
                    String teacherValue = null;
                    String classValue = null;
                    String roomValue = null;
                    for (int k = 0; k < lastcolumnnum; k++) {
                        HSSFCell hc = hr.getCell(k);
                        //如果此单元格为空则插入""
                        if(hc == null){
                            value[k] = "";
                        }else {
                            value[k] = parseDB(hc);
                        }
                    }
                    if(j==firstrownum + headNum){
                        for(int h=0;h<headers.length;h++){
                            if(!value[h].trim().equals(headers[h])){
                                result += "导入头出现错误";
                                return null;
                            }
                        }
                    }else{
                        //判断是否是空数据并且存在空数据不是合并单元格
                        if(containSunday(hwb)){
                            CellRangeAddress value3 = belongBigCell(cellRangeAddressList,j,3,hs);
                            CellRangeAddress value14 = belongBigCell(cellRangeAddressList,j,14,hs);
                            CellRangeAddress value16 = belongBigCell(cellRangeAddressList,j,16,hs);
                            CellRangeAddress value27 = belongBigCell(cellRangeAddressList,j,27,hs);
                            if((value[3] == null || value[3].equals("")) && value3 == null
                                    || (value[14] == null || value[14].equals("")) && value14 == null
                                    || (value[16] == null || value[16].equals("")) && value16 == null
                                    || (value[27] == null || value[27].equals("")) && value27 == null){
                                //如果存在，跳过插入
                                if(j != hs.getLastRowNum()){
                                    result += "第 "+ Integer.valueOf(j + 1) +" 行存在空数据";
                                }
                                continue;
                            }
                            if((value[3] == null || value[3].equals("")) && value3 != null){
                                value[3] = parseDB(hs.getRow(value3.getFirstRow()).getCell(value3.getFirstColumn()));
                            }
                            if((value[14] == null || value[14].equals("")) && value3 != null){
                                value[14] = parseDB(hs.getRow(value14.getFirstRow()).getCell(value14.getFirstColumn()));
                            }
                            if((value[16] == null || value[16].equals("")) && value3 != null){
                                value[16] = parseDB(hs.getRow(value16.getFirstRow()).getCell(value16.getFirstColumn()));
                            }
                            if((value[27] == null || value[27].equals("")) && value3 != null){
                                value[27] = parseDB(hs.getRow(value27.getFirstRow()).getCell(value27.getFirstColumn()));
                            }
                            value[28] = value3 == belongBigCell(cellRangeAddressList,j - 1,3,hs) ? "true" : "false";
                            value[29] = value14 == belongBigCell(cellRangeAddressList,j - 1,14,hs) ? "true" : "false";
                            value[30] = value16 == belongBigCell(cellRangeAddressList,j - 1,16,hs) ? "true" : "false";
                            value[31] = value27 == belongBigCell(cellRangeAddressList,j - 1,27,hs) ? "true" : "false";
                        }else{
                            CellRangeAddress value3 = belongBigCell(cellRangeAddressList,j,3,hs);
                            CellRangeAddress value14 = belongBigCell(cellRangeAddressList,j,14,hs);
                            CellRangeAddress value16 = belongBigCell(cellRangeAddressList,j,16,hs);
                            CellRangeAddress value26 = belongBigCell(cellRangeAddressList,j,26,hs);
                            if((value[3] == null || value[3].equals("")) && value3 == null
                                    || (value[14] == null || value[14].equals("")) && value14 == null
                                    || (value[16] == null || value[16].equals("")) && value16 == null
                                    || (value[26] == null || value[26].equals("")) && value26 == null){
                                //如果存在，跳过插入
                                if(j != hs.getLastRowNum()){
                                    result += "第 "+ Integer.valueOf(j + 1) +" 行存在空数据";
                                }
                                continue;
                            }
                            //如果属于合并单元格并且为空，则插入此合并单元格数据
                            if((value[3] == null || value[3].equals("")) && value3 != null){
                                value[3] = parseDB(hs.getRow(value3.getFirstRow()).getCell(value3.getFirstColumn()));
                            }
                            if((value[14] == null || value[14].equals("")) && value3 != null){
                                value[14] = parseDB(hs.getRow(value14.getFirstRow()).getCell(value14.getFirstColumn()));
                            }
                            if((value[16] == null || value[16].equals("")) && value3 != null){
                                value[16] = parseDB(hs.getRow(value16.getFirstRow()).getCell(value16.getFirstColumn()));
                            }
                            if((value[26] == null || value[26].equals("")) && value3 != null){
                                value[26] = parseDB(hs.getRow(value26.getFirstRow()).getCell(value26.getFirstColumn()));
                            }
                            value[27] = value3 == belongBigCell(cellRangeAddressList,j - 1,3,hs) ? "true" : "false";
                            value[28] = value14 == belongBigCell(cellRangeAddressList,j - 1,14,hs) ? "true" : "false";
                            value[29] = value16 == belongBigCell(cellRangeAddressList,j - 1,16,hs) ? "true" : "false";
                            value[30] = value26 == belongBigCell(cellRangeAddressList,j - 1,26,hs) ? "true" : "false";
                        }
                        values.add(value);
                    }
                }
            }
        }
        return values;
    }

    /**
     * 准备工作
     * @param hc
     * @return
     */
    private  static String parseDB(HSSFCell hc) {
        CellType celltype = hc.getCellTypeEnum();
        String cellValue=null;
        if (celltype == CellType.STRING) {
            cellValue = String.valueOf(hc.getStringCellValue());
        }
        else if (celltype == CellType.NUMERIC) {
            cellValue = String.valueOf(df.format(hc.getNumericCellValue()));
        }else if(celltype == CellType.BOOLEAN){
            cellValue = String.valueOf(hc.getBooleanCellValue());
        }else if(celltype == CellType.BLANK) {
            cellValue = hc.getStringCellValue();
        }else if(celltype == CellType.ERROR)
            cellValue = "";
        return cellValue;
    }

    public static Boolean containSunday(HSSFWorkbook hwb){
        HSSFRow hssfRow = hwb.getSheetAt(0).getRow(2);
        int i = hssfRow.getLastCellNum() - hssfRow.getFirstCellNum();
        return i == 29;
    }

    public static CellRangeAddress belongBigCell(List<CellRangeAddress> list , int rowIndex , int columnIndex , HSSFSheet hssfSheet){
        for(CellRangeAddress cellRangeAddress : list){
            //判断是否是合并单元格
            if(cellRangeAddress.containsRow(rowIndex) && cellRangeAddress.containsColumn(columnIndex)){
                HSSFRow hssfRow = hssfSheet.getRow(cellRangeAddress.getFirstRow());
                HSSFCell hssfCell = hssfRow.getCell(cellRangeAddress.getFirstColumn());
                //判断整个合并单元格是否为空
                if(hssfCell != null && !parseDB(hssfCell).trim().equals("")){
                    return cellRangeAddress;
                }
            }
        }
        return null;
    }

}
