package org.rcisoft.core.validate;

import org.rcisoft.core.result.ResponseMessage;
import org.rcisoft.core.result.ValidatedMessage;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author  cy
 * Created by cy on 16/5/20.
 */
public class ValidatedResult implements Serializable{

    private static final long serialVersionUID = -118855035745939849L;

    private Boolean hasErrors = Boolean.valueOf(false);
    private Map<String, String[]> errorsMap;
    private BindingResult br;

    private ValidatedResult() {
    }

    private ValidatedResult(BindingResult br) {
        this.br = br;
        this.dealBindingResult();
    }

    public static ValidatedResult  builder(BindingResult br){
        return new ValidatedResult(br);
    }

    public Boolean hasErrors() {
        return this.hasErrors;
    }

    public Map<String, String[]> getErrorsMap() {
        return this.errorsMap;
    }

    public BindingResult getBr() {
        return this.br;
    }

    /**
     * 生成map
     * @return
     */
    public Map<String, String[]> dealBindingResult() {
        this.br = br;
        this.errorsMap = new HashMap();
        if(br.hasErrors()) {
            this.hasErrors = Boolean.valueOf(true);
            List errorlist = br.getFieldErrors();
            Iterator var3 = errorlist.iterator();

            while(var3.hasNext()) {
                FieldError fieldError = (FieldError)var3.next();
                this.errorsMap.put(fieldError.getField(), new String[]{fieldError.getDefaultMessage(), fieldError.getCode()});
            }
        }

        return this.errorsMap;
    }

    public ResponseMessage getValidatedMessage() {
        return ValidatedMessage.builder(this.errorsMap);
    }
}
