package org.rcisoft.sys.dept.controller;


import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.common.controller.ResponseController;
import org.rcisoft.common.model.TreeViewModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.sys.dept.entity.SysDept;
import org.rcisoft.sys.dept.service.SysDeptService;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.role.service.SysRoleService;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * Created by lcy on 16/7/30.
 */
@RestController
@RequestMapping("/dept")
public class SysDeptController extends ResponseController {

    @Autowired
    private SysDeptService sysDeptServiceImpl;
    @Autowired
    private SysUserService userServiceImpl;
    @Autowired
    private SysRoleService sysRoleServiceImpl;


    @ApiOperation(value="构建  treeView", notes="构建  treeView")
    @GetMapping("queryDeptsForTree")
    public List<TreeViewModel> queryTreeView(){

        return sysDeptServiceImpl.queryDeptTree();
    }
    
    @ApiOperation(value="查部门树同时根据角色反填部门角色信息", notes="查部门树同时根据角色反填部门角色信息(已经分配部门的为选中状态)")
    @ApiImplicitParam(name = "id", value = "角色ID", required = true, dataType = "varchar", paramType = "path")
    @GetMapping("queryDeptsTreeByRoleId/{id}")
    public List<TreeViewModel> queryDeptsTreeByRoleId(@PathVariable("id") String id){
        return sysDeptServiceImpl.queryDeptsTreeByRoleId(id);
    }
    
    @ApiOperation(value="构建  select", notes="构建  select")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "userId", value = "用户ID", required = false, dataType = "varchar", paramType = "path"),
            @ApiImplicitParam(name = "name", value = "角色名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "权限code", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping("queryRolesSelect")
    public List<SysRole> queryRolesSelect(SysRole sysRole){
        return sysRoleServiceImpl.queryRolesSelect(sysRole);
    }
    
    


    /**
     * ajax jquery validator  增加部门code
     * @return  true 合法   false 不合法  不能有其它的输出
     */
    @ApiOperation(value="增加部门code", notes="true:合法;false:不合法;不能有其它的输出")
    @ApiImplicitParam(name = "deptCode", value = "部门编号", required = false, dataType = "varchar")
    @PostMapping("queryDeptCodeValidate")
    public String queryDeptCodeValidate(@RequestParam("deptCode") String deptCode){

        return "true";
    }

    @ApiOperation(value="增加部门", notes="增加部门")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "Integer", paramType = "path"),
            @ApiImplicitParam(name = "parentId", value = "父机构id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "部门名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "deptCode", value = "部门编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "seqDate", value = "排序日期", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping("adminLog/deptAdd")
    public Result deptAdd(@Valid SysDept dept, BindingResult br){
        PersistModel persistModel = sysDeptServiceImpl.persistDept(dept,getToken());
        return Result.builder(persistModel,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                dept);
    }

    /**
     * 修改部门
     * @param dept
     * @return
     */
    @ApiOperation(value="修改部门", notes="修改部门")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "Integer", paramType = "path"),
            @ApiImplicitParam(name = "parentId", value = "父机构id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "部门名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "deptCode", value = "部门编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "seqDate", value = "排序日期", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping("updateDept")
    public Result update(@Valid SysDept dept, BindingResult br){
        PersistModel persistModel = sysDeptServiceImpl.mergeDept(dept,getToken());
        return Result.builder(persistModel,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                dept);
    }

    @ApiOperation(value="删除部门", notes="根据ID删除部门")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar", paramType = "path")
    @PostMapping("deptDelete/{id}")
    public Result deptDelete(@PathVariable("id") String id){
        PersistModel persistModel = sysDeptServiceImpl.removeDept(id,getToken());
        return Result.builder(persistModel,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }
    
    @ResponseBody
    @GetMapping("querySelect")
    public List<SysDept> querySelect(){
        return sysDeptServiceImpl.querySelect();
    }

}
