package org.rcisoft.sys.menu.dao;


import org.apache.ibatis.annotations.*;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.sys.menu.entity.MenuDto;
import org.rcisoft.sys.menu.entity.SysMenu;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * Created with family.
 * author: cy
 * Date: 16/6/2
 * Time: 上午10:03
 * description: menu repository
 */
@Repository
public interface SysMenuRepository extends BaseMapper<SysMenu> {


    /**
     * 根据用户id查询 menu
     * @param id
     * @return
     */
    @Select("<script>SELECT " +
            " m.*,'' as checked " +
            " FROM " +
            " s_menu m " +
            " LEFT JOIN s_role_MENU rm ON m.BUSINESS_ID = rm.MENU_ID "+
            " LEFT JOIN s_dept_role dr ON rm.ROLE_ID = dr.ROLE_ID "+ 
            " LEFT JOIN s_dept_role_user dru ON dr.BUSINESS_ID = dru.DEPT_ROLE_ID  " +
            " where m.DEL_FLAG = '0' and m.FLAG = '1' and  dru.USER_ID = #{id} order by CASE_COUNT asc,SORT asc</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysMenu> findByUserId(@Param("id") String id);
    //s_dept_role_user,,s_dept_role,,s_role_menu


    /**
     * 查询根节点    dept tree
     * @return      list
     */
    @Select("<script>SELECT "
            + " BUSINESS_ID,PARENT_ID,PARENT_IDS,HREF,TARGET,ICON,SORT,IS_SHOW,"
            + " CODE,NAME,IS_ACTIVITI,PERMISSION,REMARKS,'' as checked"
            + " FROM s_menu "
            + " WHERE "
            + " PARENT_ID IS NULL "
            + " AND DEL_FLAG = '0' order by sort asc</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysMenu> queryRootMenuTree();


    /**
     * 根据pid  查询子节点
     * @param pid  父节点id
     * @return
     */
    @Select("<script>SELECT "
            + " BUSINESS_ID,PARENT_ID,PARENT_IDS,HREF,TARGET,ICON,SORT,IS_SHOW,"
            +"  CODE,NAME,IS_ACTIVITI,PERMISSION,REMARKS,'' as checked"
            + " FROM s_menu "
            + " WHERE "
            + " DEL_FLAG = '0' AND PARENT_ID = #{pid} order by sort asc</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysMenu> queryMenuTreeByParentId(@Param("pid") String pid);

    /**
     * 逻辑删除菜单集合
     * @param list
     */
    int logicRemoveMenus(List<SysMenu> list);

    /**
     * 更新某些列   根据主键
     * 使用注解 update   值 不能加 ' '
     * @param menu
     * @return
     */
    @Update("<script>update s_menu set " +
            " HREF = #{menu.href}," +
            " TARGET = #{menu.target}," +
            " ICON = #{menu.icon}," +
            " IS_SHOW = #{menu.isShow}," +
            " REMARKS = #{menu.remarks}," +
            " SORT = #{menu.sort}," +
            " CODE = #{menu.code}," +
            " IS_ACTIVITI = #{menu.isActiviti}," +
            " PERMISSION = #{menu.permission}," +
            " NAME = #{menu.name} ," +
            " MODEL_ID = #{menu.modelId} ," +
            " CASE_COUNT = #{menu.caseCount} " +
            " where BUSINESS_ID = #{menu.businessId}</script>")
    int updateAppointedColumnByPrimaryKey(@Param("menu") SysMenu menu);

    /**
     * 带checked的根节点
     * @param roleId
     * @return
     */
    List<SysMenu> queryRootMenuTreeForSelectedByRP(@Param("roleId") String roleId);

    /**
     * left join 查询菜单 带checked 根据 parentId  roleId
     * @param roleId
     * @param id
     * @return
     */
    @Select("<script>SELECT "
            + " menu.BUSINESS_ID,menu.PARENT_ID,menu.PARENT_IDS,menu.HREF,menu.TARGET,menu.ICON,menu.SORT,menu.IS_SHOW,"
            +"  menu.CODE,menu.NAME,menu.IS_ACTIVITI,menu.PERMISSION,menu.REMARKS,"
            +"IF ( " +
            " ( " +
            "  SELECT " +
            "   count(*) " +
            "  FROM " +
            "   s_role_menu rm where rm.MENU_ID = menu.BUSINESS_ID " +
            "   and rm.ROLE_ID = #{roleId} " +
            " ) > 0, " +
            " '1', " +
            " '0' " +
            ") AS checked"
            + " FROM s_menu menu"
            + " WHERE "
            + " menu.DEL_FLAG = '0' AND menu.PARENT_ID = #{pid} order by menu.sort asc</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysMenu> queryMenuTreeByParentIdRP(@Param("roleId") String roleId, @Param("pid") String id);

    /**
     * 删除   角色菜单
     * @param menuId
     * @return
     */
    @Delete("<script>DELETE from s_role_menu where MENU_ID=#{menuId}</script>")
    int deleteRoleMenuByMenuId(@Param("menuId") String menuId);

    @Select("<script>select m.BUSINESS_ID,PARENT_ID,PARENT_IDS,NAME,CODE from s_menu m " +
            "LEFT JOIN s_role_menu rm on m.BUSINESS_ID = rm.MENU_ID " +
            "where rm.ROLE_ID = #{roleId}</script>")
    List<MenuDto> queryMeunsByRoleId(String roleId);
}
