package org.rcisoft.sys.menu.service.impl;



import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.common.model.CommonMenuModel;
import org.rcisoft.common.model.HomeMenuModel;
import org.rcisoft.common.model.SysMenuModel;
import org.rcisoft.common.model.TreeViewModel;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.menu.dao.SysMenuRepository;
import org.rcisoft.sys.menu.entity.SysMenu;
import org.rcisoft.sys.menu.service.SysMenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by lcy on 16/7/30.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class SysMenuServiceImpl implements SysMenuService {

    @Autowired
    private SysMenuRepository menuRepository;

    /**
     * 查询菜单树
     */
    @Override
    public List<TreeViewModel> queryMenuTree() {
        TreeViewModel treeRoot = null;
        List<SysMenu> MenuList = menuRepository.queryRootMenuTree();
        //返回集
        List<TreeViewModel> treeList = new ArrayList<TreeViewModel>();
        for(int i = 0; i < MenuList.size();i++){
            treeRoot = conversionMenu(MenuList.get(i));
            //查询子部门
            queryChildMenu(treeRoot);
            treeList.add(treeRoot);
        }
        return treeList;
    }


    /**
     * 添加菜单
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel persistMenu(SysMenu menu, String token) {
        menu.setCommonBusinessId();
        //增加操作
        UserUtil.setCurrentPersistOperation(menu);
        int line = menuRepository.insertSelective(menu);
        return new PersistModel(line);
    }

    /**
     * 合并菜单
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel mergeMenu(SysMenu menu, String token) {
        //修改操作
        UserUtil.setCurrentMergeOperation(menu);
        int line = menuRepository.updateAppointedColumnByPrimaryKey(menu);
        return new PersistModel(line);
    }

    /**
     * 移除菜单
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel removeMenu(String id, String token) {
        //int line = menuRepository.deleteByPrimaryKey(id);
        //逻辑删除
    	
    	int line= menuRepository.deleteRoleMenuByMenuId(id);
    	
        List<SysMenu> list = new ArrayList<SysMenu>();
        SysMenu menu = new SysMenu();
        menu.setBusinessId(id);
        UserUtil.setCurrentPersistOperation(menu);
        list.add(menu);
        //子节点
        list = this.queryChildMenuToList(id,list);
        line = menuRepository.logicRemoveMenus(list);
        return new PersistModel(line);
    }

    /**
     * 根据主键查菜单
     */
    @Override
    public SysMenu queryMenuById(String id) {

        return menuRepository.selectByPrimaryKey(id);
    }
    /**
     * 根据角色id，查询角色绑定的菜单
     */
    @Override
    public List<TreeViewModel> queryMenuTreeForSelectedByRP(String id) {
        TreeViewModel treeRoot = null;
        //根节点不能为checked
        List<SysMenu> MenuList = menuRepository.queryRootMenuTree();
        //返回集
        List<TreeViewModel> treeList = new ArrayList<TreeViewModel>();
        for(int i = 0; i < MenuList.size();i++){
            treeRoot = conversionMenu(MenuList.get(i));
            //查询子部门
            queryChildMenuByRP(id,treeRoot);
            treeList.add(treeRoot);
        }
        return treeList;
    }

    @Override
    public List<SysMenu> queryMenus() {
        return menuRepository.selectAll();
    }

    /**
     * 设置选中 active
     * @param model
     * @param activeClassName
     */
    private void setActiveMenu(CommonMenuModel model , String selectId, String activeClassName){
        if(null == model)
            return;
        List<CommonMenuModel> modelList = model.getCommonModelList();
        if(CollectionUtils.isNotEmpty(modelList)){
            if(StringUtils.isEmpty(selectId))
                modelList.get(0).setIsActive("active");
            else{
                for(CommonMenuModel sys : modelList){
                    if(sys.getId().equals(selectId)){
                        sys.setIsActive("active");
                        break;
                    }
                }
            }

        }
    }
    /**
     * 构建模型
     * @param menuModel 菜单模型
     * @param userMenu  用户菜单
     * @param model     model ID
     * @param selectId  activiId
     */
    private CommonMenuModel constructModel(CommonMenuModel menuModel, List<SysMenu> userMenu, String model, String selectId) {
        if(null == userMenu || 0 == userMenu.size())
            return null;
        for(SysMenu menu : userMenu){
            //该模块
            if(model.equals(menu.getModelId()) && "1".equals(menu.getIsShow())){
                CommonMenuModel newModel = convertMenuModel(menuModel,menu);
                menuModel = integratedMenuModel(menuModel,newModel);
            }
        }
        return menuModel;

    }

    /**
     * 转换类型
     * @param menuModel
     * @param menu
     * @return
     */
    private CommonMenuModel convertMenuModel(CommonMenuModel menuModel,SysMenu menu){
        //强转
        if(menuModel instanceof SysMenuModel){
            SysMenuModel sys = new SysMenuModel();
            sys.setId(menu.getBusinessId());
            sys.setName(menu.getName());
            sys.setParentId(menu.getParentId());
            sys.setHref(menu.getHref());
            sys.setTarget(menu.getTarget());
            sys.setIcon(menu.getIcon());
            return sys;
        }else if (menuModel instanceof HomeMenuModel){
            HomeMenuModel home = new HomeMenuModel();
            home.setId(menu.getBusinessId());
            home.setName(menu.getName());
            home.setParentId(menu.getParentId());
            home.setHref(menu.getHref());
            home.setTarget(menu.getTarget());
            home.setIcon(menu.getIcon());
            return home;
        }
        return  menuModel;
    }

    /**
     * 集成
     * @param menuModel
     * @param newModel
     * @return
     */
    private CommonMenuModel integratedMenuModel(CommonMenuModel menuModel,CommonMenuModel newModel){
        //强转  sys
        if(menuModel instanceof SysMenuModel){
            SysMenuModel sys = (SysMenuModel) menuModel;
            //初始化 第一次进入
            if(null == sys.getId() || "".equals(sys.getId().trim()))
                menuModel = newModel;
            else{
                //子菜单
                if(sys.getId().equals(newModel.getModelParentId()))
                    sys.addSysMenuModel((SysMenuModel) newModel);
                else
                    sys.forInSysMenuModel(sys,(SysMenuModel) newModel);
            }
        }
        //强转  home
        else if(menuModel instanceof HomeMenuModel){
            HomeMenuModel home = (HomeMenuModel) menuModel;
            //初始化 第一次进入
            if(null == home.getId() || "".equals(home.getId().trim()))
                menuModel = newModel;
            else{
                //子菜单
                if(home.getId().equals(newModel.getModelParentId()))
                    home.addSysMenuModel((HomeMenuModel) newModel);
                /*else
                    sys.forInSysMenuModel(sys,(SysMenuModel) newModel);*/
            }
        }
        return  menuModel;
    }

    /**
     * 查询子菜单
     * @param treeRoot
     */
    private void queryChildMenu(TreeViewModel treeRoot) {
        List<SysMenu> MenuList = menuRepository.queryMenuTreeByParentId(
                treeRoot.getId());
        List<TreeViewModel> treeView = new ArrayList<TreeViewModel>();
        if(0 == MenuList.size())
            return;
        //赋值
        for(SysMenu Menu : MenuList) {
            TreeViewModel tree = conversionMenu(Menu);
            //递归
            queryChildMenu(tree);
            treeView.add(tree);
        }
        treeRoot.setNodes(treeView);
    }
    /**
     * 根据角色查询子菜单
     * @param treeRoot
     */
    private void queryChildMenuByRP(String roleId, TreeViewModel treeRoot) {
        List<SysMenu> MenuList = menuRepository.queryMenuTreeByParentIdRP(roleId,
                treeRoot.getId());
        List<TreeViewModel> treeView = new ArrayList<TreeViewModel>();
        if(0 == MenuList.size())
            return;
        //赋值
        for(SysMenu Menu : MenuList) {
            TreeViewModel tree = conversionMenu(Menu);
            //递归
            queryChildMenuByRP(roleId,tree);
            treeView.add(tree);
        }
        treeRoot.setNodes(treeView);
    }
    /**
     * 查询子菜单    复制集合
     * @param parentId   父部门id
     */
    private List<SysMenu> queryChildMenuToList(String parentId, List<SysMenu> list) {
        List<SysMenu> menuList = menuRepository.queryMenuTreeByParentId(
                parentId);
        if(0 == menuList.size())
            return list;

        list.addAll(menuList);
        //赋值
        for(SysMenu menu : menuList) {
            queryChildMenuToList(menu.getBusinessId(),list);
        }
        return list;
    }
    /**
     * 转换  Menu to  treeView
     * @param menu
     * @return
     */
    private TreeViewModel conversionMenu(SysMenu menu) {
        TreeViewModel treeView = new TreeViewModel();
        treeView.setId(menu.getBusinessId());
        treeView.setText(menu.getName());
        treeView.setPid(menu.getParentId());
        treeView.setParentId(menu.getParentId());
        treeView.setSelectable(true);
        treeView.setTags(new String[]{menu.getCode()});
        //配置选中
        if(null != menu.getChecked() && !"".equals(menu.getChecked())){
            TreeViewModel.TreeViewStateModel  treeViewStateModel
                    = new TreeViewModel().new TreeViewStateModel();
            if("1".equals(menu.getChecked()))
                treeViewStateModel.setChecked(true);
            else if ("0".equals(menu.getChecked()))
                treeViewStateModel.setChecked(false);
            treeView.setState(treeViewStateModel);
        }
        return treeView;
    }


}
