package org.rcisoft.sys.role.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.role.service.SysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.List;

/**
 * Created by lcy on 16/7/30.
 */
@RestController
@RequestMapping("/role")
public class SysRoleController extends PaginationController<SysRole> {

    @Autowired
    private SysRoleService sysRoleServiceImpl;


    @ApiOperation(value="分页查询角色", notes="分页查询角色")
    @ApiImplicitParams({@ApiImplicitParam(name = "name", value = "角色名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "权限编号", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryRoles")
    public GridModel queryRolesByPagination(SysRole sysRole){
        sysRoleServiceImpl.queryRolesByPagination(getPaginationUtility(), sysRole);
        GridModel gridModel = getGridModelResponse();
        return  gridModel;
    }


    @ApiOperation(value="增加/修改角色", notes="增加/修改角色")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "roleName", value = "角色名称", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "角色编号", required = true, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result roleAdd(SysRole sysRole){
        int line = sysRoleServiceImpl.persistRole(sysRole,getToken());
        return Result.builder(new PersistModel(line),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                sysRole);
    }


    @ApiOperation(value="删除角色", notes="删除角色")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar", paramType = "path")
    @PostMapping(value = "/delete")
    public Result roleDelete(String businessId){

        int line = sysRoleServiceImpl.removeRole(businessId,getToken());
        return Result.builder(new PersistModel(line),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value="权限分配", notes="权限分配")
    @ApiImplicitParams({@ApiImplicitParam(name = "roleId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "menus", value = "菜单id，逗号间隔", required = true, dataType = "varchar")})
    @PostMapping(value = "/rolePermission")
    public Result rolePermission(@RequestParam("roleId") String roleId, @RequestParam("menus") String menus ){
        return Result.builder(sysRoleServiceImpl.doAuthorization(roleId,menus,getToken()),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                roleId);
    }
    
}
