package org.rcisoft.sys.role.dao;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.sys.dept.entity.DeptRole;
import org.rcisoft.sys.role.entity.SysRole;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * Created with family.
 * author: cy
 * Date: 16/6/2
 * Time: 上午10:03
 * description:
 */
@Repository
public interface SysRoleRepository extends BaseMapper<SysRole> {
    /**
     * 分页查询 roles
     * @param sysRole
     * @return
     */
    @Select("<script>select * from s_role where flag = '1' and del_flag = '0' "
    		+ " <if test=\"code !=null and code != ''\">and code = #{code} </if> "
            + " <if test=\"roleName !=null and roleName != ''\">and role_name = #{roleName} </if> " +
            " and code NOT in ('ROLE_1001','ROLE_1002','ROLE_1003')"
            + " order by create_date desc"
    		+ "</script>")
    @ResultMap(value = "SelectAllAndUserNum" )
    List<SysRole> queryRoles(SysRole sysRole);

    //获得此角色的用户数
    int countUserNum(String roleId);
    
    @Select("<script>select * from s_role  where "
    		+ "DEL_FLAG='0' "
    		+ "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<SysRole> queryRolesSelect(SysRole sysRole);

    /**
     * 根据角色id  删除对应权限
     * @param roleId
     * @return
     */
    @Delete("<script>delete from s_role_menu where role_id = #{roleId}</script>")
    int deleteRoleMenus(@Param("roleId") String roleId);
    
    
    /**
     * 根据角色id  删除对应部门
     * @param roleId
     * @return
     */
    @Delete("<script>delete from s_dept_role where role_id = #{roleId}</script>")
    int deleteRoleDepts(@Param("roleId") String roleId);

    /**
     * 批量插入         角色_权限
     * @param menuRole
     * @return
     */
    int insertBatchRoleMenu(List<Map> menuRole);
    
    
    /**
     * 批量插入         角色_部门
     * @param       deptRole
     * @return
     */
    int insertBatchRoleDept(List<Map> deptRole);
    /**
     * 分页查询 roles
     * @param sysRole
     * @return
     */
    @Select(" <script>select "
    		+ " dr.BUSINESS_ID drid,r.BUSINESS_ID rid,r.name rname,r.code rcode,d.BUSINESS_ID did,d.name dname,d.dept_code deptcode, "
    		+ " IF(( "
    		+ " SELECT count(*) from s_dept_role_user dru where dru.USER_ID=#{userId} "
    		+ " and dru.DEPT_ROLE_ID=dr.BUSINESS_ID"
    		+ " )>0,'1','0') as checked "
    		+ " FROM  s_dept_role dr "
    		+ " LEFT JOIN s_dept d ON d.BUSINESS_ID = dr.DEPT_ID "
    		+ " LEFT JOIN s_role r on dr.ROLE_ID = r.BUSINESS_ID</script>")
    @ResultMap(value = "drResultMap" )
    List<DeptRole> queryUserRolesByPagination(@Param("userId") String userId, SysRole sysRole) ;
    
    
    /**
     * 分页查询 roles
     * @param sysRole
     * @return
     */
    @Select("<script>SELECT " +
            " sysRole.BUSINESS_ID, " +
            " sysRole.NAME, " +
            " sysRole.CODE, " +
            " sysRole.DATA_SCOPE, " +
            " sysRole.FLAG, " +
            " sysRole.DEL_FLAG, " +
            " sysRole.CREATE_BY, " +
            " sysRole.create_date, " +
            "IF ( " +
            " ( " +
            "  SELECT " +
            "   count(*) " +
            "  FROM " +
            "   s_dept_ROLE dr " +
            "  WHERE " +
            "   dr.ROLE_ID = sysRole.BUSINESS_ID " +
            "  AND ur.USER_ID = #{deptId}  " +
            " ) > 0, " +
            "  '1', " +
            "  '0' " +
            " ) AS checked " +
            "FROM " +
            " s_role sysRole " +
            "WHERE " +
            " DEL_FLAG = '0' and FLAG = '1'</script>")
    @ResultMap(value = "BaseResultMap" )
    List<SysRole> queryDeptRolesByPagination(@Param("deptId") String deptId, SysRole sysRole);
     
    @Delete("<script>DELETE from s_dept_role_user where DEPT_ROLE_ID="
    		+ "("
    		+ "SELECT BUSINESS_ID from s_dept_role  where ROLE_ID=#{roleId}"
    		+ ")</script>")
    int deleteDeptRoleUserByroleId(@Param("roleId") String roleId);

    /**
     * 删除岗位
      * @param roleId
     * @return
     */
    @Delete("<script>DELETE from s_dept_role where ROLE_ID=#{roleId}</script>")
    int deleteDeptRoleByRoleId(@Param("roleId") String roleId);

    /**
     * 删除权限
     * @param roleId
     * @return
     */
    @Delete("<script>DELETE from s_role_menu where ROLE_ID=#{roleId}</script>")
    int deleteRoleMenuByRoleId(@Param("roleId") String roleId);

    @Select("<script>SELECT sr.* from s_role sr " +
            "LEFT JOIN s_r_user_role sur ON sr.business_id = sur.role_id " +
            "LEFT JOIN s_user su on su.business_id = sur.user_id " +
            "where su.business_id = #{businessId} " +
            "and sr.del_flag = '0' " +
            "and sr.flag = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> findRolesByUserId(String businessId);

    @Select("<script>SELECT sr.* from s_role sr " +
            "LEFT JOIN s_r_user_role sur ON sr.business_id = sur.role_id " +
            "LEFT JOIN s_user su on su.business_id = sur.user_id " +
            "where su.login_name = #{username} " +
            "and sr.del_flag = '0' " +
            "and sr.flag = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> queryCodeByUsername(String username);


    @Select("<script>SELECT sr.* from s_role sr " +
            "LEFT JOIN s_r_user_role sur ON sr.business_id = sur.role_id " +
            "LEFT JOIN s_user su on su.business_id = sur.user_id " +
            "where su.business_id = #{businessId} " +
            "and sr.del_flag = '0' " +
            "and sr.flag = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> queryCodeByBusinessId(String businessId);

    @Select("<script>select * from s_role where flag = '1' and del_flag = '0' "
            + "and (code = #{code} or role_name = #{roleName}) "
            + "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<SysRole> queryRolesByCodeOrName(SysRole sysRole);

    @Delete("<script>DELETE from s_r_user_role where role_id=#{id}</script>")
    int deleteUserRoleByRoleId(String id);

    @Select("<script>select * from s_role where flag = '1' and del_flag = '0' "
            + "and role_name = #{roleName}"
            + "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<SysRole> queryRolesByName(SysRole sysRole);

//    @Select("SELECT COUNT(*) FROM s_menu sm \n" +
//            "LEFT JOIN s_role_menu rm ON sm.BUSINESS_ID = rm.MENU_ID\n" +
//            "LEFT JOIN s_role sr ON rm.ROLE_ID = sr.business_id\n" +
//            "WHERE sr.`code` = #{roleCode} AND sm.BUSINESS_ID = #{businessId}")
//    int queryRoleByRoleCode(@Param("roleCode") String roleCode, @Param("businessId") String businessId);

    @Select("SELECT count(*) FROM s_menu \n" +
            "LEFT JOIN s_role_menu  ON s_menu.BUSINESS_ID = s_role_menu.MENU_ID " +
            "LEFT JOIN s_role  ON s_role_menu.ROLE_ID = s_role.business_id " +
            "WHERE s_menu.MODEL_ID = #{modelId} AND ROLE_ID = #{roleId}")
    int queryRoleByRoleCode(@Param("modelId") String modelId, @Param("roleId") String roleId);

    /**
     * 根据子表id 删除 s_r_user_role
     * @param tableByKey
     * @param businessId
     * @return
     */
    @Delete("DELETE\n" +
            "FROM\n" +
            "\ts_r_user_role\n" +
            "WHERE\n" +
            "\ts_r_user_role.business_id = (\n" +
            "\t\tSELECT\n" +
            "\t\t\tab.*\n" +
            "\t\tFROM\n" +
            "\t\t\t(\n" +
            "\t\t\t\tSELECT\n" +
            "\t\t\t\t\ts_r_user_role.business_id\n" +
            "\t\t\t\tFROM\n" +
            "\t\t\t\t\ts_r_user_role\n" +
            "\t\t\t\tLEFT JOIN s_user ON s_r_user_role.user_id = s_user.business_id\n" +
            "\t\t\t\tLEFT JOIN ${tb} ON ${tb}.`code` = s_user.login_name\n" +
            "\t\t\t\tWHERE\n" +
            "\t\t\t\t\t${tb}.business_id = #{cid}\n" +
            "\t\t\t) ab\n" +
            "\t)")
    int deleteUserRoleByChildrenId(@Param("tb") String tableByKey,@Param("cid") String businessId);
}
