package org.rcisoft.sys.role.service.impl;


import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.dept.entity.DeptRole;
import org.rcisoft.sys.role.dao.SysRoleRepository;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.role.service.SysRoleService;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lcy on 16/7/30.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysRoleServiceImpl implements SysRoleService {

    @Autowired
    private SysRoleRepository sysRoleRepository;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public int persistRole(SysRole sysRole, String token) {
        int line;
        if(sysRole.getBusinessId()!=null){//修改
            List<SysRole> roles = sysRoleRepository.queryRolesByName(sysRole);
            if(roles.size()>0){
                throw new ServiceException(ResultServiceEnums.ROLE_EXISTS);
            }
            UserUtil.setCurrentMergeOperation(sysRole);
            line = sysRoleRepository.updateByPrimaryKeySelective(sysRole);
        }else{//增加
            List<SysRole> roles = sysRoleRepository.queryRolesByCodeOrName(sysRole);
            if(roles.size()>0){
                throw new ServiceException(ResultServiceEnums.ROLE_EXISTS);
            }
            UserUtil.setCurrentPersistOperation(sysRole);
            line = sysRoleRepository.insertSelective(sysRole);
        }
        return line;
    }


    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public int removeRole(String id,String token) {
        //获得当前是此角色的人数
        int userCount = sysUserMapper.countUserByRole(id);
        //如果仍然有用户（如果用户被逻辑删除，则忽略此角色，但如果用于被停用，则依然被计算）是此角色，则该角色不可删除
        if(userCount > 0){
            throw new ServiceException(ResultServiceEnums.ROLE_HAS_USED);
        }else{
            //删除该角色所有的关联信息
            sysRoleRepository.deleteRoleMenuByRoleId(id);
            sysRoleRepository.deleteUserRoleByRoleId(id);
            int line = sysRoleRepository.deleteByPrimaryKey(id);
            log.info(UserUtil.getUserInfoProp(token,UserUtil.USER_USERNAME)+"删除了ID为"+ id+"的角色信息");
            return line;
        }
    }

    @Override
    public List<SysRole> queryRolesByPagination(PageUtil pageUtil, SysRole sysRole) {
        sysRole.setNotDeleted();
        sysRole.setStart();
        return sysRoleRepository.queryRoles(sysRole);
    }
    @Override
    public List<DeptRole> queryUserRolesByPagination(PageUtil<SysRole> paginationUtility, String userId, SysRole sysRole) {
        return sysRoleRepository.queryUserRolesByPagination(userId, sysRole);
    }
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel doAuthorization(String roleId, String role_menus, String token) {
        //删除原始权限
        int line = sysRoleRepository.deleteRoleMenus(roleId);
        //批量插入新的权限
        if(null != role_menus && !"".equals(role_menus.trim())){
            String[] menus = role_menus.split(",");
            List<Map> menuRole = new ArrayList<Map>();
            Map map = new HashMap();
            for (int i = 0 ; i < menus.length ; i++){
                map = new HashMap();
                map.put("roleId",roleId);
                map.put("menuId",menus[i]);
                map.put("id", IdGen.uuid());
                menuRole.add(map);
            }
            line= sysRoleRepository.insertBatchRoleMenu(menuRole);
        }
        log.info(UserUtil.getUserInfoProp(token,UserUtil.USER_USERNAME)+"更改了ID为"+roleId+"的权限");
        return new PersistModel(line);
    }
    
    @Override
    public List<SysRole> queryDeptRolesByPagination(PageUtil<SysRole> paginationUtility, String deptId, SysRole sysRole) {
        return sysRoleRepository.queryDeptRolesByPagination(deptId, sysRole);
    }
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel doAuthorizationDept(String roleId, String role_depts, String token) {
        //删除原始权限
        int line = sysRoleRepository.deleteRoleDepts(roleId);
        //批量插入新的权限
        if(null != role_depts && !"".equals(role_depts.trim())){
            String[] depts = role_depts.split(",");
            List<Map> deptRole = new ArrayList<Map>();
            Map map = null;
            for (int i = 0 ; i < depts.length ; i++){
                map = new HashMap();
                map.put("roleId",roleId);
                map.put("deptId",depts[i]);
                map.put("id", IdGen.uuid());
                deptRole.add(map);
            }
            line = sysRoleRepository.insertBatchRoleDept(deptRole);
        }
        log.info(UserUtil.getUserInfoProp(token,UserUtil.USER_USERNAME)+"绑定了了ID为"+roleId+"的岗位");
        return new PersistModel(line);
    }

	@Override
	public SysRole queryRoleById(String id) {
	       return sysRoleRepository.selectByPrimaryKey(id);
	}

	@Override
	public List<SysRole> queryRolesSelect(SysRole sysRole) {
		return sysRoleRepository.queryRolesSelect(sysRole);
	}




}
