package org.rcisoft.sys.user.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.menu.entity.SysMenu;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

/**
 * Created by Andrew on 2017/7/6.
 */
@RestController
@RequestMapping(value = "/user")
public class SysUserController extends PaginationController<SysUser> {

    @Autowired
    private SysUserService userServiceImpl;

    @Autowired
    private Global global;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @ApiOperation(value="增加/修改用户", notes="增加/修改用户")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "loginName", value = "登录名", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "name", value = "用户名", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "agencyCode", value = "所属教学单位", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "roleId", value = "角色Id", required = true, dataType = "varchar")})
    @PostMapping(value = "/addUser")
    public Result addUser(@ApiIgnore SysUser user){
        int line = userServiceImpl.addOrUpdateUser(user);
        return Result.builder(new PersistModel(line),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                user);
    }


    @ApiOperation(value="修改密码", notes="根据用户ID修改密码")
    @ApiImplicitParams({@ApiImplicitParam(name = "oldPassword", value = "旧密码", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "newPassword", value = "新密码", required = true, dataType = "varchar")})
    @PostMapping(value = "/changePassword")
    public Result changePassword(String oldPassword, String newPassword){
        int min_length = Integer.parseInt(global.getPASSWORD_MIN_PASSWORD());
        int max_length = Integer.parseInt(global.getPASSWORD_MAX_PASSWORD());
        int this_length = newPassword.length();
        if(this_length<min_length||this_length>max_length) {
            throw new ServiceException(ResultServiceEnums.PASSWORD_LENGTH_ERROR);
        }
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        String encodePassword = userServiceImpl.checkPassword(userId);
        boolean real =passwordEncoder.matches(oldPassword,encodePassword);
        if(!real) {
            throw new ServiceException(ResultServiceEnums.PASSWORD_ERROR);
        }
        PersistModel persistModel = new PersistModel(userServiceImpl.changePassword(userId,newPassword));
        return Result.builder(persistModel,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                "");
    }

    @ApiOperation(value = "重置密码", notes = "根据用户学号或者工号重置密码")
    @PostMapping("/resetPassword")
    @ApiImplicitParams({@ApiImplicitParam(name = "loginName", value = "登录名", required = true, dataType = "varchar")})
    public Result resetPassword (String loginName){
        PersistModel persistModel = new PersistModel(userServiceImpl.resetPassword(loginName));
        return Result.builder(persistModel,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                loginName);
    }


    @ApiOperation(value="分页查询用户", notes="分页查询用户")
    @ApiImplicitParam(name = "agencyCode", value = "教学单位Code", required = false, dataType = "varchar")
    @GetMapping(value = "/queryUsers")
    public GridModel queryUsers(SysUser sysUser){
        userServiceImpl.queryUsersByPagination(getPaginationUtility(), sysUser);
        GridModel gridModel = getGridModelResponse();
        return  gridModel;
    }

    @ApiOperation(value="删除用户", notes="删除用户")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar", paramType = "path")
    @PostMapping(value = "/delete")
    public Result roleDelete(String id){

        int line = userServiceImpl.removeUser(id,getToken());
        return Result.builder(new PersistModel(line),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="停用/启用用户", notes="停用/启用用户")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar", paramType = "path")
    @PostMapping(value = "/stopOrStart")
    public Result stopOrStart(String businessId){
        int line = userServiceImpl.stopOrStartUserByUserBusinessId(businessId);
        return Result.builder(new PersistModel(line),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

}
