package org.rcisoft.sys.user.dao;


import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bagency.entity.BAgency;
import org.rcisoft.business.bcompany.entity.BCompany;
import org.rcisoft.business.bteacher.entity.BTeacher;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.entity.UserRole;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * Created by lcy on 17/8/9.
 */
@Repository
public interface SysUserMapper extends BaseMapper<SysUser> {


    @Select("<script>select * from s_user where login_name = #{username} and del_flag = 0 </script>")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> queryUserByName(String username);

    @Insert("<script>INSERT INTO s_user" +
            "(create_date,update_date,create_by,update_by,remarks,del_flag,flag,business_id,login_name,password,name,sex,git_lab_id,git_lab_username,git_lab_impression_token,head_pic,email)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "( #{item.createDate},#{item.updateDate},#{item.createBy},#{item.updateBy},#{item.remarks},#{item.delFlag},#{item.flag},#{item.businessId},#{item.loginName},#{item.password},#{item.name},#{item.sex},#{item.gitLabId},#{item.gitLabUsername},#{item.gitLabImpressionToken},#{item.headPic},#{item.email})" +
            "</foreach></script>")
    int insertList(List<SysUser> users);

    @Insert("<script>INSERT INTO s_r_user_role" +
            "(business_id,user_id,role_id)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "(#{item.businessId},#{item.userId},#{item.roleId})" +
            "</foreach></script>")
    int insertUserRoleList(List<UserRole> users);

    @Insert("INSERT INTO s_r_user_role"+
            "(business_id,user_id,role_id)VALUES" +
            "(#{businessId},#{userId},#{roleId})")
    int insertUserRole(UserRole userRole);

    @Select("SELECT u.business_id from s_user u left join ${tab} s on u.login_name = s.code where s.business_id = #{id} and u.del_flag = 0")
    String selectUserIdByStuOrTeaId(@Param("id") String id, @Param("tab") String table);

    @Delete("delete from s_user where login_name = #{code}")
    int deleteByCode(String code);

    /**
     * 用户删除 cy 5-15
     * @param code
     * @return
     */
    @Delete("DELETE s " +
            "FROM " +
            " s_user s " +
            "LEFT JOIN b_company c ON c. CODE = s.login_name " +
            "WHERE " +
            " c.business_id = #{code}")
    int deleteByCompany(String code);


    @Select("<script>select * from s_user where business_id = #{userId} and password = #{password} limit 1</script>")
    @ResultMap(value = "BaseResultMap")
    SysUser checkPassword(@Param("userId") String userId, @Param("password") String password);

    @Update("update s_user set password =#{password} where business_id = '${userId}'")
    int changePassword(@Param("userId") String userId, @Param("password") String password);

    @Select("<script>select password from s_user where business_id = #{userId} </script>")
    String queryPasswordById(String userId);

    @Update("<script>update s_user set password = #{password} where 1=1 " +
            "and login_name = #{loginName} " +
            "and del_flag = 0</script>")
    int resetPassword(@Param("loginName") String loginNam,@Param("password") String password);

    @Select("<script>select business_id from s_user where login_name = #{studentCode} and del_flag = 0 and flag = 1</script>")
    String queryUserIdByLoginName(String studentCode);

    @Delete("<script>delete from s_r_user_role where user_id = #{businessId}</script>")
    int deleteUserRole(String businessId);

    @Select("<script>select su.*,ba.`name` as agencyName,sr.role_name as roleName,sr.business_id as roleId from s_user su " +
            "LEFT JOIN b_agency ba on su.agency_code = ba.`code` " +
            "LEFT JOIN s_r_user_role sru on su.business_id = sru.user_id " +
            "LEFT JOIN s_role sr on sru.role_id = sr.business_id " +
            "where su.is_created = '1' and su.del_flag = 0 and su.flag = 1  " +
            "<if test=\"agencyCode !=null and agencyCode != '' \">and su.agency_code = #{agencyCode} </if></script>")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> queryCreatedUsers(SysUser sysUser);

    @Select("<script>select agency_code from s_user where business_id = #{businessId}</script>")
    String queryAgencyCodeByBusinessId(String businessId);

    @Select("select * from b_agency where code = #{agencyCode} and del_flag = 0 and flag = 1")
    List<BAgency> checkTeacherAgency(String agencyCode);

    @Update("update s_user set flag =#{flag} where login_name = #{loginName} and del_flag = 0 and flag = 0")
    int stopOrStartUser(SysUser user);

    @Update("update s_user set flag =#{flag} where business_id = #{businessId} ")
    int stopOrStartUserByUserBusinessId(SysUser sysUser);

    @Select("select business_id from s_user where login_name = #{0} and del_flag = 0")
    String selectByLoginName(String loginName);

    @Select("select count(*) from s_r_user_role " +
            "left join s_user on s_user.business_id = s_r_user_role.user_id " +
            "where role_id = #{roleId} and s_user.del_flag = 0 ")
    int countUserByRole(String roleId);


    List<SysUser> selectByTeacherCode(List<BTeacher> teacherList);

    List<SysUser> selectByCompanyCode(List<BCompany> companyList);

    int batchDeleteByTeacherCode(List<BTeacher> teacherList);

    int batchDeleteByCompanyCode(List<BCompany> bCompanyList);

    int deleteTeacherUserRole(List<SysUser> userTeacherList);

    int deleteCompanyUserRole(List<SysUser> userCompanyList);
}
