package org.rcisoft.sys.user.service.impl;

import org.rcisoft.common.component.Global;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.UserConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.entity.UserRole;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class SysUserServiceImpl implements SysUserService {

	@Autowired
	private SysUserMapper sysUserMapper;

	@Autowired
	private PasswordEncoder passwordEncoder;

	@Autowired
	private Global global;



	@Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
	@Override
	public PersistModel persistUser(SysUser user) {
		user.setCommonBusinessId();
		user.setPassword(passwordEncoder.encode(global.getDEFAULT_PASSWORD()));
		int line = sysUserMapper.insertSelective(user);
		return new PersistModel(line);
	}


	@Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
	@Override
	public int addOrUpdateUser(SysUser user) {
		int line;
		if(user.getBusinessId()!=null){//修改
			UserUtil.setCurrentMergeOperation(user);
			//删除原来角色
			sysUserMapper.deleteUserRole(user.getBusinessId());
			//修改
			line = sysUserMapper.updateByPrimaryKeySelective(user);
		}else{//添加
			UserUtil.setCurrentPersistOperation(user);
			user.setPassword(passwordEncoder.encode(global.getDEFAULT_PASSWORD()));
			line = sysUserMapper.insertSelective(user);
		}
		//添加新的角色
		UserRole userRole = new UserRole(user.getBusinessId(),user.getRoleId());
		userRole.setCommonBusinessId();
		sysUserMapper.insertUserRole(userRole);
		return line;
	}

	@Override
	public List<SysUser> queryUsersByPagination(PageUtil pageUtil, SysUser sysUser) {
		sysUser.setStart();
		sysUser.setNotDeleted();
		return sysUserMapper.queryCreatedUsers(sysUser);
	}

	@Override
	public int removeUser(String id, String token) {
		SysUser user = new SysUser();
		user.setDeleted();
		user.setStop();
		user.setBusinessId(id);
		return sysUserMapper.deleteByPrimaryKey(user);
	}

	@Override
	public void updateUser(SysUser user,String uid,int line) {
		if(!user.isEmpty()){
			user.setBusinessId(uid);
			int l1 = sysUserMapper.updateByPrimaryKeySelective(user);
			if (l1 != line)
				throw new ServiceException(ResultServiceEnums.UPDATE_ERROR);
		}
	}

	@Override
	public String checkPassword(String userId) {
		return sysUserMapper.queryPasswordById(userId);
	}

	@Override
	public int changePassword(String userId, String newPassword) {
		return sysUserMapper.changePassword(userId,passwordEncoder.encode(newPassword));
	}

	@Override
	public int resetPassword(String loginName) {
		return sysUserMapper.resetPassword(loginName,passwordEncoder.encode(global.getDEFAULT_PASSWORD()));
	}

	@Override
	public int stopOrStartUserByUserBusinessId(String businessId){
		SysUser sysUser = sysUserMapper.selectByPrimaryKey(businessId);
		String status = sysUser.getFlag().equals("0") ? "1":"0";
		sysUser.setFlag(status);
		return sysUserMapper.stopOrStartUserByUserBusinessId(sysUser);
	}
}
